@@ -34,6 +34,9 @@
 /* Define the Dosemu version */
 #undef VERSION
 
+/* Define the Dosemu version code  */
+#undef DOSEMU_VERSION_CODE
+
 /* Define the Dosemu version release date */
 #undef VERDATE
 
@@ -92,6 +95,9 @@
 /* Define this to use the new keyboard code */
 #undef NEW_KBD_CODE
 
+/* Define this to use the new port I/O code */
+#undef NEW_PORT_CODE
+
 /*
  *  DEFINE this, if you have joystick support in the kernel
  *  or have the joystick module loaded (needing <linux/joystick.h>)
@@ -104,12 +110,33 @@
  */
 #undef USE_SBEMU
 
+/*
+ * This is DEFINED when the Ultra libraries are available
+ */
+#undef USE_ULTRA
+
 /* Define this if you want to use MITSHM */
 #undef HAVE_MITSHM
 
 /* Define this if you want to use a monoton micro timing (Bernd Paysan) */
 #undef MONOTON_MICRO_TIMING
 
+/* Define this if you want dosemu permanently run as (suid) root and only gain
+ * user privileges, when accessing secure relevant resources.
+ * Otherwise dosemu will permanently run as user and only temporary take over
+ * root privilege when needed.
+ */
+#undef RUN_AS_ROOT
+
+/* Define this to use the new X code */
+#undef NEW_X_CODE
+
+/* Define this to use threading code */
+#undef USE_THREADS
+
+/* Define this to use the X86 CPU emulator */
+#undef X86_EMULATOR
+
 @BOTTOM@
 
 #endif /* CONFIG_H */
