@@ -81,7 +81,8 @@ extern struct vm86_struct vm86s;
 #endif
 
 extern int  dis_8086(unsigned int, const unsigned char *,
-                     unsigned char *, int, unsigned int *, unsigned int *, unsigned int);
+                     unsigned char *, int, unsigned int *, unsigned int *,
+                     unsigned int, int);
 
 /* prototypes */
 static void* mhp_getadr(unsigned char *, unsigned int *, unsigned int *, unsigned int *);
@@ -771,7 +772,7 @@ static void mhp_disasm(int argc, char * argv[])
        }
        refseg = seg;
        rc = dis_8086(org+bytesdone, buf+bytesdone, frmtbuf, def_size,
-                  &refseg, &ref, (IN_DPMI ? dpmi_mhp_getselbase(refseg) : 0));
+                  &refseg, &ref, (IN_DPMI ? dpmi_mhp_getselbase(refseg) : 0), 16);
        for (i=0;i<rc;i++) {
            sprintf(&bytebuf[i*2], "%02X", *(buf+bytesdone+i) );
            bytebuf[(i*2)+2] = 0x00;
@@ -1211,7 +1212,12 @@ static void mhp_regs(int argc, char * argv[])
      mhp_printf( "General Protection Fault, ");
 
  if (!traceloop)
-  mhp_printf( "system state: %s%s%s\n",
+  mhp_printf( "system state: %s%s%s%s\n",
+#ifdef X86_EMULATOR
+       config.cpuemu > 1 ? "emulated," : "",
+#else
+       "",
+#endif
        mhpdbgc.stopped ? "stopped" : "running",
        IN_DPMI ? " in DPMI" : (in_dpmi?" in real mode while in DPMI":""),
        IN_DPMI ?(dpmi_mhp_getcsdefault()?"-32bit":"-16bit") : "");
@@ -1336,7 +1342,7 @@ int mhp_getcsip_value()
 
 void mhp_modify_eip(int delta)
 {
-  extern dpmi_mhp_modify_eip(int delta);
+  extern int dpmi_mhp_modify_eip(int delta);
   if (IN_DPMI) dpmi_mhp_modify_eip(delta);
   else LWORD(eip) +=delta;
 }
