@@ -12,6 +12,8 @@
  *
  * maintainer:
  * Alistair MacDonald <alistair@slitesys.demon.co.uk>
+ * David Brauman <crisk@netvision.net.il>
+ * Michael Karcher <karcher@dpk.berlin.fido.de>
  *
  * DANG_END_MODULE
  */
@@ -46,12 +48,25 @@
 static int mixer_fd = -1;
 static int dsp_fd   = -1;
 static long int block_size = 0;
+static long int sound_frag = 0xc;
 /* MPU static vars */
 static int mpu_fd = -1;	             /* -1 = closed */
 static boolean mpu_disabled = FALSE; /* TRUE if MIDI output disabled */
 
 extern void sb_set_speed (void);   /* From sound.c */
 
+void linux_sb_dma_set_blocksize(__u16 val)
+{
+  __u16 i, tmp;
+
+  for (i = 0, tmp = val; tmp > 1; i++, tmp = tmp >> 1)
+    ;
+
+  sound_frag = i;
+
+  S_printf ("SB:[Linux] DMA blocksize set to %u (%lu)\n", val, sound_frag);
+}
+
 void linux_sb_write_mixer(int ch, __u8 val)
 {
   int newsetting;
@@ -136,8 +151,12 @@ void linux_sb_disable_speaker(void)
 {
   if (dsp_fd != -1)
   {
+    S_printf ("SB:[Linux] Syncing DSP\n");
+    ioctl(dsp_fd, SNDCTL_DSP_SYNC);
     close (dsp_fd);
     dsp_fd = -1;
+  } else {
+    S_printf ("SB:[Linux] DSP already closed\n");
   }
 }
 
@@ -194,7 +213,7 @@ static void linux_sb_DAC_write (int bits, __u8 value)
 
 int linux_sb_get_version(void)
 {
-#ifdef 0
+#if 0
   long int sound_frag = SOUND_FRAG;
 #endif
   int tmp, version = 0;
@@ -215,9 +234,23 @@ int linux_sb_get_version(void)
        * sample in with more than 13 kHz; however at least one version of
        * VoxWare didn't return this error (or is this fixed now?)
        * on the other site sampling in is not the question yet, so
-       * I return SB 1.5 (because have one)
+       * I return SB 1.5 (because have one) - Michael Beck
+       * -----
+       * But I need SB 2.0 for my MOD-Player to work on my soundcard!
+       * An important difference is the high-speed feature (DSP-command 0x9X)
+       * As you said, the SB 1.5 can sample slower than the SB 2.0, because the
+       * SB 2.0 is using the high-speed command (DSP: 0x99). BUT: the high-
+       * speed command is existing for playback, and without HS you can play
+       * back up to 22 kHz, and with HS you can play back up to 44 kHz (!)
+       *            (-Karch)       
        */
-      version = SB_OLD;
+
+       tmp=44100;
+       ioctl(dsp_fd, SNDCTL_DSP_SPEED,&tmp);
+       if(tmp < 44100)
+	 version = SB_OLD;
+       else
+	 version = SB_20;
     }
     else {
       /*
@@ -231,7 +264,7 @@ int linux_sb_get_version(void)
         version = SB_16;
     }
 
-#ifdef 0
+#if 0
     /* reset to 8 bit per sample and mono */
     tmp = 0;
     ioctl(dsp_fd, SNDCTL_DSP_STEREO, &tmp);
@@ -271,14 +304,20 @@ void linux_sb_dma_start_init(__u32 command)
 #ifndef SOUND_FRAG
 #error SOUND_FRAG not defined!
 #endif
-  long int sound_frag = SOUND_FRAG;
+/*  long int sound_frag = SOUND_FRAG; */
   long int samplesize = AFMT_U8;
 
+  extern long int sound_frag;
   extern long int block_size;
 
+  long int fragments = 0x0020000 | sound_frag;
+
   switch(command)
   {
   case 0x14: /* 8-bit DMA */
+    S_printf ("SB:[Linux] 8-bit DMA starting\n");
+    break;
+  case 0x1C: /* 8-bit DMA (Auto-Init) */
     S_printf ("SB:[Linux] 8-bit DMA (Auto-Init) starting\n");
     break;
   case 0x90: /* 8-bit DMA (Auto-Init, High Speed) */
@@ -290,7 +329,7 @@ void linux_sb_dma_start_init(__u32 command)
     break;
   };
 
-  ioctl(dsp_fd, SNDCTL_DSP_SETFRAGMENT, &sound_frag);
+  ioctl(dsp_fd, SNDCTL_DSP_SETFRAGMENT, &fragments);
   ioctl(dsp_fd, SNDCTL_DSP_SAMPLESIZE, &samplesize);
 
   ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &block_size);
@@ -298,8 +337,9 @@ void linux_sb_dma_start_init(__u32 command)
 
 
 void linux_sb_dma_start_complete (void) {
-  extern long int block_size;
-
+   extern long int block_size; 
+   long int fragsize;
+   
   /*
    * This is the important part:
    *
@@ -316,10 +356,26 @@ void linux_sb_dma_start_complete (void) {
    * 
    * The final value sets the number of bytes to try and transfer.
    */
-
-/*dma_install_handler(config.sb_dma, -1, dsp_fd, sb_dma_handler, block_size);*/
+   
+   /* 
+    * **CRISK**
+    * 
+    * Now using the "blocksize" as set by the user, when set.
+    * Potential bug -> this may need resetting. No idea how the actual
+    * thing works.
+    *
+    */
+   
+   fragsize = (1 << sound_frag);
+   if (!fragsize) 
+     {
+	fragsize = block_size;
+	S_printf("SB:[Linux] Using default blocksize\n");
+     }
+   S_printf ("SB:[Linux] DMA start completed (blocksize %lu)\n",fragsize);
+   dma_install_handler(config.sb_dma, -1, dsp_fd, sb_dma_handler, fragsize);
 /*dma_install_handler(config.sb_dma, -1, dsp_fd, sb_dma_handler, 1);*/
-  dma_install_handler(config.sb_dma, -1, dsp_fd, sb_dma_handler, SOUND_SIZE);
+/*dma_install_handler(config.sb_dma, -1, dsp_fd, sb_dma_handler, SOUND_SIZE);*/
 }
 
 int linux_sb_dma_complete_test(void)
@@ -351,7 +407,7 @@ void linux_sb_dma_complete(void)
 	S_printf ("SB:[Linux] DMA Completed\n");
 }
 
-#ifdef 0
+#if 0
 void start_dsp_dma(void)
 {
   int real_speed, trash;
@@ -383,13 +439,15 @@ void start_dsp_dma(void)
 
 void linux_sb_set_speed (__u16 speed, __u8 stereo_mode)
 {
-  int rate, mode;
+  int rate, channels;
   rate = speed;
-  mode = stereo_mode;
+
+  /* stereo_mode is actually 2 is stereo is requested - Karcher */
+  channels = (stereo_mode != 0) +1; /* not happy with this constuct - AM */
 
   if (dsp_fd != -1)
   {
-    ioctl(dsp_fd, SNDCTL_DSP_STEREO, &mode);
+    ioctl(dsp_fd, SNDCTL_DSP_CHANNELS, &channels);
     ioctl(dsp_fd, SNDCTL_DSP_SPEED, &rate);
     
     S_printf ("SB:[Linux] (actual rate : %u)\n", rate);
@@ -435,9 +493,11 @@ int SB_driver_init (void) {
   SB_driver.DMA_stop            = NULL;
   SB_driver.DMA_complete_test   = linux_sb_dma_complete_test;
   SB_driver.DMA_complete        = linux_sb_dma_complete;
-
+  SB_driver.DMA_set_blocksize   = linux_sb_dma_set_blocksize;
+   
   /* Miscellaneous Functions */
   SB_driver.set_speed           = linux_sb_set_speed;
+  SB_driver.play_buffer         = NULL;
 
   /*
    * This determines a suitable value for the SB Version that we can 
@@ -454,7 +514,9 @@ void linux_mpu401_data_write(__u8 data)
 	   'open on demand' strategy. */
 	if (mpu_fd == -1) {
 	  	if (mpu_disabled) return;
-		mpu_fd = open("/var/run/dosemu-midi",O_WRONLY | O_CREAT, 0777);
+		/* Added NONBLOCK to prevent hanging - Karcher */
+		mpu_fd = open("/var/run/dosemu-midi", 
+			      O_WRONLY | O_CREAT | O_NONBLOCK, 0777);
 		if (mpu_fd == -1) {
 			mpu_disabled = TRUE;
 			S_printf("MPU401:[Linux] Failed to open file 'midi' (%s)\n",
