@@ -59,6 +59,9 @@ GetDebugFlagsHelper(char *debugStr)
   debugStr[i++] = DebugFlag(d.pd);       debugStr[i++] = 'P';
   debugStr[i++] = DebugFlag(d.request);  debugStr[i++] = 'r';
   debugStr[i++] = DebugFlag(d.sound);    debugStr[i++] = 'S';
+#ifdef X86_EMULATOR
+  debugStr[i++] = DebugFlag(d.emu);      debugStr[i++] = 'e';
+#endif
 
   debugStr[i] = 0;
   dbug_printf("debugStr is %s\n", debugStr);
