@@ -65,17 +65,15 @@ static void mrp_read_joystick(void);
 #endif
 
 void kill_time(long usecs) {
-   struct timeval scr_tv, t_start, t_end;
+   hitimer_t t_start;
    long t_dif;
    scr_tv.tv_sec = 0L;
    scr_tv.tv_usec = usecs;
 
-   gettimeofday(&t_start, NULL);
+   t_start = GETusTIME(0);
    while ((int) select(STDIN_FILENO, NULL, NULL, NULL, &scr_tv) < (int) 1)
      {
-        gettimeofday(&t_end, NULL);
-        t_dif = ((t_end.tv_sec * 1000000 + t_end.tv_usec) -
-                 (t_start.tv_sec * 1000000 + t_start.tv_usec));
+	t_dif = (long)(GETusTIME(0)-t_start);
 
         if ((t_dif >= usecs) || (errno != EINTR))
           return ;
@@ -182,6 +180,11 @@ static void process_master_boot_record(void)
 /* returns 1 if dos_helper() handles it, 0 otherwise */
 static int dos_helper(void)
 {
+#ifdef X86_EMULATOR
+  extern void enter_cpu_emu(void);
+  extern void leave_cpu_emu(void);
+#endif
+
   switch (LO(ax)) {
   case DOS_HELPER_DOSEMU_CHECK:			/* Linux dosemu installation test */
     LWORD(eax) = 0xaa55;
@@ -218,7 +221,7 @@ static int dos_helper(void)
 
     i_printf("I/O perms: 0x%04x 0x%04x %d\n", LWORD(ebx), LWORD(ecx), cflag);
     if (set_ioperm(LWORD(ebx), LWORD(ecx), cflag)) {
-      error("ERROR: SET_IOPERMS request failed!!\n");
+      error("SET_IOPERMS request failed!!\n");
       CARRY;			/* failure */
     } else {
       if (cflag)
@@ -243,7 +246,7 @@ static int dos_helper(void)
       unsigned long sp;
 
       if (!config.mapped_bios) {
-	error("ERROR: CAN'T DO VIDEO INIT, BIOS NOT MAPPED!\n");
+	error("CAN'T DO VIDEO INIT, BIOS NOT MAPPED!\n");
 	return 1;
       }
       if (set_ioperm(0x3b0, 0x3db - 0x3b0, 1))
@@ -294,19 +297,31 @@ static int dos_helper(void)
      
   case DOS_HELPER_VIDEO_INIT:
     v_printf("Starting Video initialization\n");
+    /* DANG_BEGIN_REMARK
+     * Some video BIOSes need access to the PIT timer 2, and some
+     * (e.g. Matrox) directly read the timer output on port 0x61.
+     * If we don't allow video port access, this will be totally
+     * emulated; else, we give temporary access to the needed ports
+     * (timer at 0x42, timer config at 0x43 and timer out/speaker at 0x61),
+     * provided they were not previously enabled by SPKR_NATIVE - AV
+     * DANG_END_REMARK
+     */
     if (config.allowvideoportaccess) {
       if (config.speaker != SPKR_NATIVE) {
-	v_printf("Giving access to port 0x42\n");
-	set_ioperm(0x42, 1, 1);
+	v_printf("Giving temporary access to PIT#2\n");
+	set_ioperm(0x42, 2, 1);		/* port 0x43 too! */
+	set_ioperm(0x61, 1, 1);
       }
       in_video = 1;
     }
-    /* Many video BIOSes use hi interrupt vector locations as
+    /* DANG_BEGIN_REMARK
+     * Many video BIOSes use hi interrupt vector locations as
      * scratchpad area - this is because they come before DOS and feel
      * safe to do it. But we are initializing vectors before video, so
-     * this will cause trouble. I assume no video BIOS will ever:
+     * this only causes trouble. I assume no video BIOS will ever:
      * - change vectors < 0xe0 (0:380-0:3ff area)
-     * - change anything in the vector area _after_ installation
+     * - change anything in the vector area _after_ installation - AV
+     * DANG_END_REMARK
      */
     v_printf("Save hi vector area\n");
     MEMCPY_2UNIX(save_hi_ints,0x380,128);
@@ -316,13 +331,15 @@ static int dos_helper(void)
     v_printf("Finished with Video initialization\n");
     if (config.allowvideoportaccess) {
       if (config.speaker != SPKR_NATIVE) {
-        v_printf("Removing access to port 0x42\n");
-        set_ioperm(0x42, 1, 0);
+        v_printf("Removing temporary access to PIT#2\n");
+        set_ioperm(0x42, 2, 0);
+        set_ioperm(0x61, 1, 0);
       }
       in_video = 0;
     }
     v_printf("Restore hi vector area\n");
     MEMCPY_2DOS(0x380,save_hi_ints,128);
+    /* The video BIOS just wrote its entry point into the int0x10 vector */
     if (mice->intdrv) { /* grab int10 back from video card for mouse */
         void bios_f000(), bios_f000_int10_old();
         us *ptr = (us*)((BIOSSEG << 4) +
@@ -458,6 +475,14 @@ static int dos_helper(void)
   case DOS_HELPER_CHDIR:
         LWORD(eax) = chdir(SEG_ADR((char *), es, dx));
         break;
+#ifdef X86_EMULATOR
+  case DOS_HELPER_CPUEMUON:
+  	if (config.cpuemu && !in_dpmi) enter_cpu_emu();
+        break;
+  case DOS_HELPER_CPUEMUOFF:
+  	if (config.cpuemu && !in_dpmi) leave_cpu_emu();
+        break;
+#endif
   case DOS_HELPER_MBR:
     if (LWORD(eax) == 0xfffe) {
       process_master_boot_record();
@@ -477,7 +502,7 @@ static int dos_helper(void)
 
 
   default:
-    error("ERROR: bad dos helper function: AX=0x%04x\n", LWORD(eax));
+    error("bad dos helper function: AX=0x%04x\n", LWORD(eax));
     return 0;
   }
 
@@ -518,8 +543,8 @@ static void int15(u_char i)
     HI(ax) = 0x86;
     /*k_printf("INT15 0x4f CARRY=%x AX=%x\n", (LWORD(eflags) & CF),LWORD(eax));*/
     k_printf("INT15 0x4f CARRY=%x AX=%x\n", (READ_FLAGS() & CF),LWORD(eax));
-/*
     CARRY;
+/*
     if (LO(ax) & 0x80 )
       if (1 || !(LO(ax)&0x80) ){
 	fprintf(stderr, "Carrying it out\n");
@@ -949,7 +974,7 @@ static int ms_dos(int nr)
       return 1;
       break;
     }
-  error("ERROR: dos_ioctl shouldn't get here. XXX\n");
+  error("dos_ioctl shouldn't get here. XXX\n");
   return 0;
 #endif
 
@@ -988,7 +1013,7 @@ run_int(int i)
   /* clear TF (trap flag, singlestep), VIF/IF (interrupt flag), and
    * NT (nested task) bits of EFLAGS
    * NOTE: IF-flag only, because we are not sure that we will test it in
-   *       some of our own software (...we all are human beeings)
+   *       some of our own software (...we all are human beings)
    *       For vm86() 'VIF' is the candidate to reset in order to do CLI !
    */
   WRITE_FLAGSE(READ_FLAGSE() & ~(VIF | TF | IF | NT));
@@ -1021,7 +1046,9 @@ int can_revector(int i)
   case 0x2f:			/* needed for XMS, redirector, and idling */
   case DOS_HELPER_INT:		/* e6 for redirector and helper (was 0xfe) */
   case 0xe7:			/* for mfs FCB helper */
+#ifdef USE_INT_QUEUE
   case 0xe8:			/* for int_queue_run return */
+#endif
     return REVECT;
 
   case 0x74:			/* needed for PS/2 Mouse */
@@ -1174,7 +1201,7 @@ static void int28(u_char i) {
     /* the hogthreshold value just got redefined to be the 'garrot' value */
     static int time_count = 0;
     if (++time_count >= config.hogthreshold) {
-      usleep(INT2F_IDLE_USECS);
+      usleep(INT28_IDLE_USECS);
       time_count = 0;
     }
   }
@@ -1326,7 +1353,7 @@ static void int33(u_char i) {
 /* It seems that the only mouse sub-function that could be plausibly used to 
  * poll the mouse is AX=3 - get mouse buttons and position. 
  * The mouse driver should have left AX=3 unaltered during its call.
- * The correct responce should have the buttons in the low 3 bits in BX and 
+ * The correct response should have the buttons in the low 3 bits in BX and 
  * x,y in CX,DX. 
  * Some programs seem to interleave calls to read mouse with various other 
  * sub-functions (Esp. 0x0b  0x05 and 0x06)
@@ -1374,6 +1401,7 @@ static void inte7(u_char i) {
   run_int(0xe7);
 }
 
+#ifdef USE_INT_QUEUE
 /* End function for interrupt calls from int_queue_run() */
 static void inte8(u_char i) {
   static unsigned short *csp;
@@ -1405,6 +1433,7 @@ static void inte8(u_char i) {
   h_printf("e8 int_queue: shouldn't get here\n");
   show_regs(__FILE__,__LINE__);
 }
+#endif /* USE_INT_QUEUE */
 
 /*
  * DANG_BEGIN_FUNCTION DO_INT 
@@ -1431,7 +1460,7 @@ if (i== 0x2f) {
 }
 #endif
 
-  if ((LWORD(cs)) != BIOSSEG && IS_REDIRECTED(i) && can_revector(i)){
+  if ((LWORD(cs) != BIOSSEG) && IS_REDIRECTED(i) && can_revector(i)){
     run_int(i);
     return;
   }
@@ -1465,6 +1494,7 @@ if (i== 0x2f) {
 #endif
 }
 
+#ifdef USE_INT_QUEUE
 /*
  * Called to queue a hardware interrupt - will call "callstart"
  * just before the interrupt occurs and "callend" when it finishes
@@ -1604,6 +1634,7 @@ void int_queue_run(void)
 	   current_interrupt,
 	   int_queue_head[int_queue_running].int_queue_return_addr);
 }
+#endif /* USE_INT_QUEUE */
 
 /*
  * DANG_BEGIN_FUNCTION setup_interrupts
@@ -1630,6 +1661,7 @@ void setup_interrupts(void) {
 #endif
     interrupt_function[i] = default_interrupt;
     if ((i & 0xf8) == 0x60) { /* user interrupts */
+	/* show also EMS (int0x67) as disabled */
 	SETIVEC(i, 0, 0);
     } else {
 	SETIVEC(i, BIOSSEG, 16 * i);
@@ -1645,6 +1677,7 @@ void setup_interrupts(void) {
   interrupt_function[0xd] = int_a_b_c_d_e_f;
   interrupt_function[0xe] = int_a_b_c_d_e_f;
   interrupt_function[0xf] = int_a_b_c_d_e_f;
+  /* This is called only when revectoring int10 */
   interrupt_function[0x10] = int10;
   interrupt_function[0x11] = int11;
   interrupt_function[0x12] = int12;
@@ -1669,7 +1702,9 @@ void setup_interrupts(void) {
 #endif
   interrupt_function[0xe6] = inte6;
   interrupt_function[0xe7] = inte7;
+#ifdef USE_INT_QUEUE
   interrupt_function[0xe8] = inte8;
+#endif
 
   /* Let kernel handle this, no need to return to DOSEMU */
   SETIVEC(0x1c, 0xf010, 0xc0);
@@ -1694,7 +1729,12 @@ void setup_interrupts(void) {
   SETIVEC(0x09, INT09_SEG, INT09_OFF);
   SETIVEC(0x08, INT08_SEG, INT08_OFF);
 
-  install_int_10_handler();	/* Install the handler for video-interrupt */
+  /* Install new handler for video-interrupt into bios_f000_int10ptr,
+   * for video initialization at f800:4200
+   * If config_vbios_seg=0xe000 -> e000:3, else c000:3
+   * Next will be the call to int0xe6,al=8 which starts video BIOS init
+   */
+  install_int_10_handler();
 
   /* This is an int e7 used for FCB opens */
   SETIVEC(0xe7, INTE7_SEG, INTE7_OFF);
