@@ -19,9 +19,6 @@
 #if X_GRAPHICS
 #include "X.h"
 #include "vgaemu.h"
-#ifdef VESA /* root@zaphod */
-#include "vesa.h"
-#endif
 #endif
 
 /* a bit brutal, but enough for now */
@@ -408,6 +405,12 @@ do_text_mode:
     WRITE_BYTE(BIOS_VIDEO_MODE, video_mode=mode&0x7f);
     break;
 
+#ifdef NEW_X_CODE
+case 0x0d:
+case 0x0e:
+case 0x10:
+case 0x12:
+#endif
 case 0x13:	/*Not finished ! */
 case 0x5c:
 case 0x5d:
@@ -433,6 +436,65 @@ error:
   return 0;
 }    
 
+#ifdef NEW_X_CODE
+/*
+ * new_set_video_mode() is called to set a VESA mode as set_video_mode()
+ * really has problems getting its job done.
+ */
+
+boolean new_set_video_mode(int mode) {
+  vga_mode_info *vmi;
+  int clear_mem = 1;
+
+  mode &= 0xffff;
+
+  v_printf("new_set_video_mode: mode = 0x%02x\n", mode);
+
+  if((vmi = vga_emu_find_mode(mode, NULL)) == NULL) {
+    v_printf("new_set_video_mode: undefined video mode\n");
+    return 0;
+  }
+
+  if(Video->setmode == NULL) {
+    v_printf("new_set_video_mode: no setmode handler!\n");
+    return 0;
+  }
+
+  video_mode = mode;
+
+  if(mode >= 0x80 && mode < 0x100) {
+    mode &= 0x7f;
+    clear_mem = 0;
+  }
+  if(mode & 0x8000) {
+    mode &= ~0x8000;
+    clear_mem = 0;
+  }
+
+  if(config.cardtype == CARD_MDA) video_mode = 7;
+
+  WRITE_BYTE(BIOS_VIDEO_MODE, video_mode);
+
+  if(vmi->mode_class == TEXT) {
+    gfx_mode = TEXT;
+    clear_scroll_queue();
+    if(clear_mem) clear_screen(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE), 7);
+  }
+
+  li = vmi->text_height;
+  co = vmi->text_width;
+  if(li > MAX_LINES) li = MAX_LINES;
+  WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
+  WRITE_WORD(BIOS_SCREEN_COLUMNS, co);
+  WRITE_WORD(BIOS_FONT_HEIGHT, vmi->char_height);
+  Video->setmode(vmi->mode_class, co, li);
+
+  if(clear_mem && vmi->mode_class == TEXT) clear_screen(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE), 7);
+
+  return 1;
+}    
+#endif
+
 /******************************************************************/
 
 /* the actual int10 handler */
@@ -538,7 +600,11 @@ void int10()
       }
       if (config.console_video) set_vc_screen_page(page);
 #if X_GRAPHICS
+#ifdef NEW_X_CODE
+      if (config.X) vga_emu_set_text_page(page, TEXT_SIZE);
+#else
       if (config.X) set_vgaemu_page(page);      /*root@sjoerd*/
+#endif
 #endif
 
       WRITE_BYTE(BIOS_CURRENT_SCREEN_PAGE, video_page = page);
@@ -675,6 +741,7 @@ void int10()
         int i, count;
         unsigned char* src;
         unsigned char r, g, b, index;
+	DAC_entry rgb;
 
         switch(LO(ax))
           {
@@ -696,6 +763,26 @@ void int10()
               }
             break;
 
+          case 0x15:  /* Read Individual DAC Register */
+            DAC_read_entry(&rgb, (unsigned char)LO(bx));
+            (unsigned char)HI(dx) = rgb.r;
+            (unsigned char)HI(cx) = rgb.g;
+            (unsigned char)LO(cx) = rgb.b;
+            break;
+
+          case 0x17:  /* Read Block of DAC Registers */
+            index=(unsigned char)LO(bx);
+            count=LWORD(ecx);
+            src=SEG_ADR((unsigned char*),es,dx);
+            for(i=0; i<count; i++, index++)
+              {
+                DAC_read_entry(&rgb, index); 
+                src[i*3]=rgb.r;
+                src[i*3+1]=rgb.g;
+                src[i*3+2]=rgb.b;
+              }
+            break;
+
           default:
             break;
           }
@@ -855,6 +942,10 @@ void int10()
     }
     break;
     
+  case 0xcc:			/* called from NC 5.0 */
+    _CX = 0; _AL = 0xff;
+    break;
+
   case 0xfe:			/* get shadow buffer..return unchanged */
   case 0xff:			/* update shadow buffer...do nothing */
     break;
@@ -865,12 +956,10 @@ void int10()
 #endif
 
 #if X_GRAPHICS
-#ifdef VESA /* root@zaphod */
   case 0x4f:                    /* vesa interrupt */
     if(config.X)
       do_vesa_int();
     break;
-#endif
 #endif
 
   default:
