@@ -6,12 +6,14 @@
  * Maintainers: J. Lawrence Stephan
  *              Scott Buchholz
  *
+ * REMARK
  * This is the timer emulation for DOSEMU.  It emulates the Programmable
  * Interval Timer (PIT), and also handles IRQ0 interrupt events.
  * A lot of animation and video game software are dependant on this module
  * for high frequency timer interrupts (IRQ0).
  *
- * This code will actually generate 18.2 interrupts/second.  It will even
+ * This code will actually generate 18.2 DOS interrupts/second (the code
+ * here itself will be triggered about 100 times per second). It will even
  * happily attempt to generate faster clocks, right up to the point where
  * it chokes.  Since the absolute best case timing we can get out of Linux
  * is 100Hz, figure that anything approaching or exceeding that isn't going
@@ -21,6 +23,7 @@
  *
  * Speaker emulation, now including port 61h, is also in here. [rz]
  *
+ * /REMARK
  * DANG_END_MODULE
  *
  */
@@ -37,7 +40,6 @@
 #include "port.h"
 #include "timers.h"
 #include "iodev.h"
-#include "port.h"
 #include "int.h"
 #include "pic.h"
 #include "dpmi.h"
@@ -47,9 +49,7 @@
 #endif
 
 
-#ifndef MONOTON_MICRO_TIMING
-#define CLOCK_TICK_RATE   1193180     /* underlying clock rate in HZ */
-#else  /* MONOTON_MICRO_TIMING */
+#ifdef MONOTON_MICRO_TIMING
 extern long   pic_itime[33];
 #endif /* MONOTON_MICRO_TIMING */
 
@@ -75,40 +75,39 @@ extern int    port61;
  */
 void initialize_timers(void)
 {
-  struct timeval cur_time;
+  hitimer_t cur_time;
 
-  gettimeofday(&cur_time, NULL);
+  cur_time = GETtickTIME(0);
 
   pit[0].mode        = 3;
+  pit[0].outpin      = 0;
   pit[0].cntr        = 0x10000;
-  pit[0].time        = cur_time;
+  pit[0].time.td     = cur_time;
   pit[0].read_latch  = -1;
   pit[0].write_latch = 0;
   pit[0].read_state  = 3;
   pit[0].write_state = 3;
 
   pit[1].mode        = 2;
+  pit[1].outpin      = 0;
   pit[1].cntr        = 18;
-  pit[1].time        = cur_time;
+  pit[1].time.td     = cur_time;
   pit[1].read_latch  = -1;
   pit[1].write_latch = 18;
   pit[1].read_state  = 3;
   pit[1].write_state = 3;
 
   pit[2].mode        = 0;
+  pit[2].outpin      = 0;
   pit[2].cntr        = -1;
-  pit[2].time        = cur_time;
+  pit[2].time.td     = cur_time;
   pit[2].read_latch  = -1;
   pit[2].write_latch = 0;
   pit[2].read_state  = 3;
   pit[2].write_state = 3;
 
   ticks_accum   = 0;
-#ifndef MONOTON_MICRO_TIMING
-  timer_div     = (pit[0].cntr * 10000) / CLOCK_TICK_RATE;
-#else /* MONOTON_MICRO_TIMING */
   timer_div     = (pit[0].cntr * 10000) / PIT_TICK_RATE;
-#endif /* MONOTON_MICRO_TIMING */
 
   timer_tick();  /* a starting tick! */
 
@@ -127,7 +126,7 @@ void initialize_timers(void)
  */
 void timer_tick(void)
 {
-  struct timeval tp;
+  hitimer_u tp;
   u_long time_curr;
   static u_long time_old = 0;       /* Preserve value for next call */
 
@@ -136,12 +135,11 @@ void timer_tick(void)
     return;
   }
 
-  /* Get system time */
-  gettimeofday(&tp, NULL);
+  /* Get system time in usec */
+  tp.td = GETusTIME(0);
 
   /* compute the number of 100usecs since we started */
-  time_curr  = (tp.tv_sec - pit[0].time.tv_sec) * 10000;
-  time_curr += (tp.tv_usec - pit[0].time.tv_usec) / 100;
+  time_curr  = (tp.td - pit[0].time.td) / 100;
   
   /* Reset old timer value to 0 if time_curr wrapped around back to 0 */
   if (time_curr < time_old) time_old = 0;
@@ -158,145 +156,201 @@ void timer_tick(void)
 
 
 #ifdef NEW_KBD_CODE
-/* this does the EMULATED mode speaker emulation */
-void do_sound(Bit16u period) {
-  if ((port61 & 3) == 3 && (pit[2].mode == 2 || pit[2].mode == 3)) {
-     if (config.console && !config.X) {
-        i_printf("SPEAKER: on, period=%d\n",pit[2].write_latch);
-        do_ioctl(console_fd,KIOCSOUND,pit[2].write_latch & 0xffff);
-     }
-     else {
-        i_printf("SPEAKER: Dumb emulated beep\n");
-        putchar('\007');
-     }
-  }
-  else if (config.console && !config.X) {
-     i_printf("SPEAKER: sound OFF!\n");
-     do_ioctl(console_fd,KIOCSOUND,0);
-  }
+#include "speaker.h"
+
+/* DANG_BEGIN_FUNCTION do_sound
+ *
+ * do_sound handles the _emulated_ mode pc-speaker emulation.  
+ *
+ * As far as I can determine all cases of the pc-speaker are now
+ * emulated.  But I am not sure where Rainer Zimmerman got his
+ * (pit[2].mode == 2) || (pit[2].mode == 3) test in the original
+ * implementation, it doesn't seem to cause problems though.
+ *
+ * The implementation of speaker_on & speaker_off can be found in 
+ * src/base/speaker.c
+ *
+ * Major Changes from version written by Rainter Zimmerman.
+ *
+ * o Added support for programs that control the directly through bit 1
+ *   of port61. 
+ *
+ * o Added a generic interface to allow multiple speaker backends.
+ *
+ * o Implemented X speaker code so the emulated speaker now works in X.
+ *
+ * --EB 21 September 1997
+ * DANG_END_FUNCTION
+ */
+void do_sound(Bit16u period)
+{
+	/* Note I assume that a sound before after another will kill
+	 * the previous sound I had a hard time getting X to do that.
+	 * If it becomes a problem possibly tuning sound_duration is
+	 * the answer.  I suggest 200ms but as I don't have that
+	 * problem now I'm not worring about it.
+	 */
+	static const unsigned sound_duration = 990;  /* in miliseconds */
+	switch (port61 & 3) {
+	case 3:		/* speaker on & speaker control through timer channel 2 */
+		if ((pit[2].mode == 2) || (pit[2].mode == 3)) {		/* is this test needed? */
+			speaker_on(sound_duration,period);
+		}
+		else {
+			speaker_off();	/* is this correct? */
+		}
+		break;
+	case 2:		/* speaker on & direct speaker through bit 1 */
+		speaker_on(sound_duration, 0xfff); /* on as long as possible */
+		break;
+	case 1:		/* speaker off & speaker control through timer channel 2 */
+	case 0:		/* speaker off & direct speaker through bit 1 */
+		speaker_off();
+		break;
+	}
 }
-#endif
+#endif /* NEW_KBD_CODE */
 
  
 static void pit_latch(int latch)
 {
-  struct timeval cur_time;
+  hitimer_u cur_time;
   u_long         ticks;
+  pit_latch_struct *p = &pit[latch];
 
   /* check for special 'read latch status' mode */
-  if (pit[latch].mode & 0x80) {
-    pit[latch].mode = pit[latch].mode & 0x07;
+  if (p->mode & 0x80) {
     /*
      * Latch status:
-     *   bit 7   = state of OUT pin (uncomputed here -- always 0)
+     *   bit 7   = state of OUT pin
      *   bit 6   = null count flag (1 == no cntr set, 0 == cntr available)
      *   bit 4-5 = read latch format
      *   bit 1-3 = read latch mode
      *   bit 0   = BCD flag (1 == BCD, 0 == 16-bit -- always 0)
      */
-    pit[latch].read_latch = (pit[latch].read_state << 4) |
-                            (pit[latch].mode << 1);
-    if (pit[latch].cntr == -1)
-      pit[latch].read_latch |= 0x40;
-    return;
+    p->read_latch = (p->read_state << 4) |
+                    ((p->mode & 7) << 1) |
+                     p->outpin;
+    if (p->cntr == -1)
+      p->read_latch |= 0x40;
+    return;	/* let bit 7 on */
   }
 
-  switch (pit[latch].mode) {
-    default:    /* just in case... */
-    case 0x00:  /* mode 0   -- countdown, interrupt, wait*/
-    case 0x01:  /* mode 1   -- countdown, wait */
-    case 0x04:  /* mode 4   -- countdown, wait */
-    case 0x05:  /* mode 5   -- countdown, wait */
-      if (pit[latch].cntr != -1) {
-	gettimeofday(&cur_time, NULL);
-#ifndef MONOTON_MICRO_TIMING
-	ticks = (cur_time.tv_sec - pit[latch].time.tv_sec) * CLOCK_TICK_RATE +
-	        ((cur_time.tv_usec - pit[latch].time.tv_usec) * 1193) / 1000;
-#else /* MONOTON_MICRO_TIMING */
-	ticks = (cur_time.tv_sec - pit[latch].time.tv_sec) * PIT_TICK_RATE +
-	        PIT_MS2TICKS(cur_time.tv_usec - pit[latch].time.tv_usec);
-#endif /* MONOTON_MICRO_TIMING */
+  cur_time.td = GETtickTIME(0);
 
-	if (ticks > pit[latch].cntr) {
-	  pit[latch].cntr = -1;
-	  pit[latch].read_latch = pit[latch].write_latch;
+  if ((p->mode & 2)==0) {
+    /* non-periodical modes 0,1,4,5 - used mainly by games
+     * count down to 0, then set output, wrap and continue counting (gate=1)
+     * only modes 0,4 allow reloading of the counter on the fly
+     * (thus modes 1,5 are not correctly implemented)
+     * we are just not interested in the gate/output pins...
+     */
+    /* mode 0   -- interrupt on terminal count, ctr reload=Y */
+    /* mode 4   -- software triggered pulse, ctr reload=Y */
+    /* mode 1   -- programmable monoflop, ctr reload=N */
+    /* mode 5   -- hardware triggered pulse, ctr reload=N */
+      /* should have been initialized to the value in write_latch */
+
+      if (p->cntr != -1) {
+	ticks = (cur_time.td - p->time.td);
+
+	if (ticks > p->cntr) {	/* time has elapsed */
+	  if ((p->mode&0x40)==0)
+	  	p->cntr = -1;
+	  p->outpin = (p->mode&4? 0x00: 0x80);
 	} else {
-	  pit[latch].read_latch = pit[latch].cntr - ticks;
+	  p->read_latch = p->cntr - ticks;
+	  p->outpin = (p->mode&4? 0x80: 0x00);
 	}
-      } else {
-	pit[latch].read_latch = pit[latch].write_latch;
       }
-      break;
-
-    case 0x02:  /* mode 2,6 -- countdown, reload */
-    case 0x06:
-#ifndef MONOTON_MICRO_TIMING
-      ticks = pit[latch].cntr - (pic_dos_time % pit[latch].cntr);
-      pit[latch].read_latch = pit[latch].cntr - ticks % pit[latch].cntr;
-#else /* MONOTON_MICRO_TIMING */
-      gettimeofday(&cur_time, NULL);
-      pic_sys_time = cur_time.tv_sec*PIT_TICK_RATE 
-	+ PIT_MS2TICKS(cur_time.tv_usec);
-      pic_sys_time += (pic_sys_time == -1);
-      if(latch == 0) {
-	ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
-	if(((int)ticks) < 0) {
-	  pic_request(PIC_IRQ0);
-	  run_irqs();
-	  ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
-	}
-      } else {
-	ticks = pit[latch].cntr - (pic_sys_time % pit[latch].cntr);
+      if (p->cntr == -1) {
+	p->read_latch = p->write_latch;
+	p->outpin = (p->mode&4? 0x00: 0x80);
       }
-      pit[latch].read_latch = ticks;
-#endif /* MONOTON_MICRO_TIMING */
-      break;
-
-    case 0x03:  /* mode 3,7 -- countdown by 2(?), interrupt, reload */
-    case 0x07:
-      gettimeofday(&cur_time, NULL);
-#ifndef MONOTON_MICRO_TIMING
-      /* fancy calculations to avoid overflow */
-      ticks = (cur_time.tv_sec - pit[latch].time.tv_sec) % pit[latch].cntr;
-      ticks = ticks * (CLOCK_TICK_RATE % pit[latch].cntr) +
-	      ((cur_time.tv_usec - pit[latch].time.tv_usec) * 1193) / 1000;
-      pit[latch].read_latch = pit[latch].cntr - (2*ticks) % pit[latch].cntr;
-#else /* MONOTON_MICRO_TIMING */
-      pic_sys_time = cur_time.tv_sec*PIT_TICK_RATE 
-	+ PIT_MS2TICKS(cur_time.tv_usec);
-      pic_sys_time += (pic_sys_time == -1);
-      ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
-      if(latch == 0) {
-	ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
-	if(((int)ticks) < 0) {
-	  pic_request(PIC_IRQ0);
-	  run_irqs();
-	  ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
-	}
-      } else {
-	ticks = pit[latch].cntr - (pic_sys_time % pit[latch].cntr);
-      }
-      pit[latch].read_latch = (2*ticks) % pit[latch].cntr;
-#endif /* MONOTON_MICRO_TIMING */
-      break;
   }
+#ifndef MONOTON_MICRO_TIMING
+  /* mode 2 -- rate generator */
+  /* mode 6 -- ??? */
+  else if ((p->mode & 3)==2) {
+      ticks = p->cntr - (pic_dos_time % p->cntr);
+      p->read_latch = p->cntr - ticks % p->cntr;
+      p->outpin = (p->read_latch? 0x80: 0x00);
+  }
+  /* mode 3 -- square-wave generator */
+  /* mode 7 -- ??? */
+  else {
+      /* no more fancy calculations to avoid overflow */
+      ticks = (cur_time.td - p->time.td) % p->cntr;
+      /* ticks is now a value which increases from 0 to cntr, and
+         is greater than cntr/2 in the second half of the cycle */
+      ticks = 2 * (ticks % p->cntr);
+      p->outpin = (ticks >= p->cntr? 0x80: 0x00);
+      p->read_latch = p->cntr - ticks % p->cntr;
+  }
+#else /* MONOTON_MICRO_TIMING */
+  else {
+    /* mode 2 -- rate generator */
+    /* mode 6 -- ??? */
+    /* mode 3 -- square-wave generator, countdown by 2 */
+    /* mode 7 -- ??? */
+      pic_sys_time = cur_time.t.tl;	/* modulo 2^32 */
+      pic_sys_time += (pic_sys_time == NEVER);
+
+      if (latch == 0) {
+	/* when current time is greater than irq time, call pic_request
+	   which will then point pic_itime to next interrupt */
+
+	if (((pic_itime[PIC_IRQ0]-pic_sys_time) < 0) && ((p->mode&0x40)==0)) {
+	  r_printf("PIT: pit_latch, pic_request IRQ0 mode 2/3\n");
+	  pic_request(PIC_IRQ0);
+	}
+	/* while current time is less than next irq time, ticks decrease;
+	   we need no modulo operation */
+	ticks = pic_itime[PIC_IRQ0] - pic_sys_time;
+      } else {
+	ticks = p->cntr - (pic_sys_time % p->cntr);
+      }
+      if ((p->mode & 3)==3) {
+        /* ticks is now a value which decreases from cntr to 0, and
+           is greater than cntr/2 in the first half of the cycle */
+	ticks *= 2;
+	if (ticks >= p->cntr) {
+	  p->outpin = 0x00; p->read_latch = (ticks-p->cntr) & 0xfffe;
+	}
+	else {
+	  p->outpin = 0x80; p->read_latch = ticks;
+	}
+      }
+      else {
+        p->read_latch = ticks;
+        p->outpin = (p->read_latch? 0x80: 0x00);
+      }
+  }
+#endif /* MONOTON_MICRO_TIMING */
+  if (p->mode & 0x40)
+    p->mode = (p->mode & 7) | 0x80;
+
 #if 0 /* for debugging */
   i_printf("pit_latch:  latched value 0x%4.4x for timer %d\n",
-	   pit[latch].read_latch, latch);
+	   p->read_latch, latch);
 #endif
 }
 
-Bit8u pit_inp(Bit32u port)
+/* This is called also by port 0x61 - some programs can use timer #2
+ * as a GP timer and read bit 5 of port 0x61 (e.g. Matrox BIOS)
+ */
+Bit8u pit_inp(ioport_t port)
 {
   int ret = 0;
   port -= 0x40;
 
-  if (port == 2 && config.speaker == SPKR_NATIVE) {
-    return safe_port_in_byte(0x42);
+  if (port == 2) {
+    if (config.speaker == SPKR_NATIVE) return safe_port_in_byte(0x42);
+    pit_latch(port);
+    return(pit[port].outpin);
   }
-
-  if (port == 1)
-    i_printf("PORT:  someone is reading the CMOS refresh time?!?");
+  else if (port == 1)
+    i_printf("PIT:  someone is reading the CMOS refresh time?!?");
 
   if (pit[port].read_latch == -1)
     pit_latch(port);
@@ -309,6 +363,8 @@ Bit8u pit_inp(Bit32u port)
       break;
     case 3: /* read LSB followed by MSB */
       ret = (pit[port].read_latch & 0xff);
+      if (pit[port].mode & 0x80) pit[port].mode &= 7;	/* moved here */
+        else
       pit[port].read_state = 0;
       break;
     case 1: /* read MSB */
@@ -326,7 +382,7 @@ Bit8u pit_inp(Bit32u port)
   return ret;
 }
 
-void pit_outp(Bit32u port, Bit8u val)
+void pit_outp(ioport_t port, Bit8u val)
 {
 
   port -= 0x40;
@@ -363,24 +419,16 @@ void pit_outp(Bit32u port, Bit8u val)
     else
       pit[port].cntr = pit[port].write_latch;
 
-    gettimeofday(&pit[port].time, NULL);
+    pit[port].time.td = GETtickTIME(0);
 
     if (port == 0) {
       ticks_accum   = 0;
-#ifndef MONOTON_MICRO_TIMING
-      timer_div     = (pit[0].cntr * 10000) / CLOCK_TICK_RATE;
-#else /* MONOTON_MICRO_TIMING */
       timer_div     = (pit[0].cntr * 10000) / PIT_TICK_RATE;
-#endif /* MONOTON_MICRO_TIMING */
       if (timer_div == 0)
 	timer_div = 1;
 #if 0
       i_printf("timer_interrupt_rate requested %.3g Hz, granted %.3g Hz\n",
-#ifndef MONOTON_MICRO_TIMING
-	       CLOCK_TICK_RATE/(double)pit[0].cntr, 10000.0/timer_div);
-#else /* MONOTON_MICRO_TIMING */
 	       PIT_TICK_RATE/(double)pit[0].cntr, 10000.0/timer_div);
-#endif /* MONOTON_MICRO_TIMING */
 #endif
     }
     else if (port == 2 && config.speaker == SPKR_EMULATED) {
@@ -396,12 +444,12 @@ void pit_outp(Bit32u port, Bit8u val)
   }
 }
 
-Bit8u pit_control_inp(Bit32u port)
+Bit8u pit_control_inp(ioport_t port)
 {
   return 0;
 }
 
-void pit_control_outp(Bit32u port, Bit8u val)
+void pit_control_outp(ioport_t port, Bit8u val)
 {
   int latch = (val >> 6) & 0x03;
 
@@ -409,6 +457,26 @@ void pit_control_outp(Bit32u port, Bit8u val)
   i_printf("PORT: outp(0x43, 0x%x)\n",val);
 #endif
 
+  /* Timer commands (00-BF):
+   *  bit 6-7 = Timer (0,1,2)
+   *		3 is not a timer but a special read-back command
+   *
+   *    bit 4-5 = command 00=latch 01=LSB mode 10=MSB mode 11=16bit mode
+   *    bit 1-3 = mode selection
+   *                    mode 0(000) 1(001) 2(010 or 110) 3(011 or 111)
+   *                         4(100) 5(101)
+   *			6(110) and 7(111) undefined?
+   *    bit 0   = binary(0), BCD(1)
+   *
+   *  0xh	counter latch timer 0
+   *  1xh	timer 0 LSB mode
+   *			modes: 0,2,3,4 - 1,5 not applicable
+   *  2xh	timer 0 MSB mode
+   *			modes: 0,2,3,4 - 1,5 not applicable
+   *  3xh	timer 0 16bit mode
+   *			modes: 0,2,4 - 3 typical - 1,5 not applicable
+   *  modes 1,5 applicable only to timer 2 [van Gilluwe,1994]
+   */
   switch (latch) {
     case 2:
       if (config.speaker == SPKR_NATIVE) {
@@ -424,7 +492,7 @@ void pit_control_outp(Bit32u port, Bit8u val)
       /* set the time base for the counter - safety code for programs
        * which use a non-periodical mode without reloading the counter
        */
-       gettimeofday(&pit[latch].time, NULL);
+       pit[latch].time.td = GETtickTIME(0);
       }
 #endif
       if ((val & 0x30) == 0)
@@ -448,9 +516,9 @@ void pit_control_outp(Bit32u port, Bit8u val)
 	if (val & 0x08) pit_latch(2);
       }
       else if ((val & 0x10) == 0) {   /* latch status words? */
-	if (val & 0x02) pit[0].mode |= 0x80;
-	if (val & 0x04) pit[1].mode |= 0x80;
-	if (val & 0x08) pit[2].mode |= 0x80;
+	if (val & 0x02) { pit[0].mode |= 0x40; pit_latch(0); }
+	if (val & 0x04) { pit[1].mode |= 0x40; pit_latch(1); }
+	if (val & 0x08) { pit[2].mode |= 0x40; pit_latch(2); }
       }
       break;
   }
@@ -523,18 +591,41 @@ void timer_int_engine(void)
 
 #ifdef NEW_KBD_CODE
 /* reads/writes to the speaker control port (0x61)
+ * Port 0x61 is really more complex than a speaker enable bit... look here:
+ * [output]:
+ * Bit(s)	Description
+ *  7	pulse to 1 for IRQ1 reset (PC,XT)
+ *  6-4	reserved
+ *  3	I/O channel parity check disable
+ *  2	RAM parity check disable
+ *  1	speaker data enable
+ *  0	timer 2 gate to speaker enable
+ *
+ * [input]:
+ * Bit(s)	Description
+ *  7	RAM parity error occurred
+ *  6	I/O channel parity error occurred
+ *  5	mirrors timer 2 output condition
+ *  4	toggles with each refresh request
+ *  3	NMI I/O channel check status
+ *  2	NMI parity check status
+ *  1	speaker data status
+ *  0	timer 2 clock gate to speaker status
  */
-Bit8u spkr_io_read(Bit32u port) {
+Bit8u spkr_io_read(ioport_t port) {
    if (port==0x61)  {
       if (config.speaker == SPKR_NATIVE)
          return port_safe_inb(0x61);
       else
-         return port61;
+	 /* keep the connection between port 0x61 and PIT timer#2 */
+         return ((*((Bit8u *)&pic_sys_time)&0x10) | /* or anything that toggles quick enough */
+		(pit_inp(0x42)? 0x20:0) |	/* outpin: 00 or 80 */
+		(port61&0xcf));
    }
    return 0xff;
 }
 
-void spkr_io_write(Bit32u port, Bit8u value) {
+void spkr_io_write(ioport_t port, Bit8u value) {
    if (port==0x61) {
       switch (config.speaker) {
        case SPKR_NATIVE:
@@ -547,7 +638,7 @@ void spkr_io_write(Bit32u port, Bit8u value) {
 	  break;
 
        case SPKR_OFF:
-          port61 = value & 0x0f;
+          port61 = value & 0x0c;
           break;
       }
    }
@@ -556,6 +647,7 @@ void spkr_io_write(Bit32u port, Bit8u value) {
 
 void pit_init(void)
 {
+#ifdef NEW_PORT_CODE
   emu_iodev_t  io_device;
 
   /* 8254 PIT (Programmable Interval Timer) */
@@ -563,68 +655,82 @@ void pit_init(void)
   io_device.write_portb  = pit_outp;
   io_device.read_portw   = NULL;
   io_device.write_portw  = NULL;
-  io_device.handler_name = "8254 PIT";
+  io_device.read_portd   = NULL;
+  io_device.write_portd  = NULL;
+  io_device.handler_name = "8254 Timer0";
   io_device.start_addr   = 0x0040;
-  io_device.end_addr     = 0x0042;
+  io_device.end_addr     = 0x0040;
   io_device.irq          = 0;
-  port_register_handler(io_device);
+  io_device.fd = -1;
+  port_register_handler(io_device, 0);
+
+  io_device.handler_name = "8254 Timer1";
+  io_device.start_addr = 0x0041;
+  io_device.end_addr = 0x0041;
+  io_device.irq = EMU_NO_IRQ;
+  port_register_handler(io_device, 0);
+
+  io_device.handler_name = "8254 Timer2";
+  io_device.start_addr = 0x0042;
+  io_device.end_addr = 0x0042;
+  port_register_handler(io_device, config.speaker==SPKR_NATIVE? PORT_FAST:0);
 
   io_device.read_portb   = pit_control_inp;
   io_device.write_portb  = pit_control_outp;
+  io_device.handler_name = "8254 Ctrl02";
   io_device.start_addr   = 0x0043;
   io_device.end_addr     = 0x0043;
-  port_register_handler(io_device);
+  port_register_handler(io_device, 0);
 
+#ifdef NEW_KBD_CODE
+  /* register_handler for port 0x61 is in keyboard code */
+  port61 = (config.speaker==SPKR_OFF? 0x0c:0x0e);
+#endif
 #if 0
   io_device.start_addr   = 0x0047;
   io_device.end_addr     = 0x0047;
-  port_register_handler(io_device);
+  port_register_handler(io_device, 0);
 #endif
-
-#ifdef NEW_KBD_CODE /* This code conflicts with the old keyboard code! 16 Sept 96*/
-  io_device.read_portb   = spkr_io_read;
-  io_device.write_portb  = spkr_io_write;
-  io_device.handler_name = "Speaker port";
-  io_device.start_addr   = 0x0061;
-  io_device.end_addr     = 0x0061;
-  io_device.irq          = EMU_NO_IRQ;
-  port_register_handler(io_device);
 #endif
 }
 
 void pit_reset(void)
 {
-  struct timeval cur_time;
+  hitimer_t cur_time;
 
-  gettimeofday(&cur_time, NULL);
+  cur_time = GETtickTIME(0);
 
   pit[0].mode        = 3;
+  pit[0].outpin      = 0;
   pit[0].cntr        = 0x10000;
-  pit[0].time        = cur_time;
+  pit[0].time.td     = cur_time;
   pit[0].read_latch  = 0xffffffff;
   pit[0].write_latch = 0;
   pit[0].read_state  = 3;
   pit[0].write_state = 3;
 
   pit[1].mode        = 2;
+  pit[1].outpin      = 0;
   pit[1].cntr        = 18;
-  pit[1].time        = cur_time;
+  pit[1].time.td     = cur_time;
   pit[1].read_latch  = 0xffffffff;
   pit[1].write_latch = 18;
   pit[1].read_state  = 3;
   pit[1].write_state = 3;
 
   pit[2].mode        = 0;
+  pit[2].outpin      = 0;
   pit[2].cntr        = 0x10000;
-  pit[2].time        = cur_time;
+  pit[2].time.td     = cur_time;
   pit[2].read_latch  = 0xffffffff;
   pit[2].write_latch = 0;
   pit[2].read_state  = 3;
   pit[2].write_state = 3;
 
   pit[3].mode        = 0;
+  pit[3].outpin      = 0;
   pit[3].cntr        = 0x10000;
-  pit[3].time        = cur_time;
+  pit[3].time.td     = cur_time;
   pit[3].read_latch  = 0xffffffff;
   pit[3].write_latch = 0;
   pit[3].read_state  = 3;
@@ -634,7 +740,7 @@ void pit_reset(void)
   timer_handle = timer_create(pit_timer_func, NULL, pit_timer_usecs(0x10000));
 #endif
 #ifdef NEW_KBD_CODE
-  port61 = 0;
+  port61 = (config.speaker==SPKR_OFF? 0x0c:0x0e);
   do_ioctl(console_fd,KIOCSOUND,0);    /* sound off */
 #endif
 }
