@@ -1,9 +1,11 @@
 /*  DANG_BEGIN_MODULE
  *
+ * REMARK
  *  pic.c is a fairly complete emulation of both 8259 Priority Interrupt 
  *  Controllers.  It also includes provision for 16 lower level interrupts.
  *  This implementation supports the following i/o commands:
  *
+ * VERB
  *      ICW1    bits 0 and 1     number of ICWs to expect
  *      ICW2    bits 3 - 7       base address of IRQs
  *      ICW3    no bits          accepted but ignored
@@ -15,6 +17,7 @@
  *                               select special mask mode
  *
  *     Reads of both pic ports are supported completely.
+ * /VERB
  *
  *  An important concept to understand in pic is the interrupt level.
  *  This is a value which represents the priority of the current interrupt.  
@@ -37,9 +40,87 @@
  *  or 0r will turn off debugging messages.  Flags may be combined:
  *  to get both 1r and 2r, use a value of 3r.
  *
+ * /REMARK
  * DANG_END_MODULE
  */
 
+/*
+ * > 
+ * > In 0.66.x the new interrupt is scheduled too early, and restore_rm_regs()
+ * > is never called. Then the crash.
+ * > 
+ * > There are at least 2 problems with this kind of code:
+ * > 1) avoiding interrupt reentrancy
+ * > 2) avoiding too early interrupt scheduling
+ * > 
+ *
+ * You have run into an old problem, which was created through attempts
+ * (not by me) to speed up pic by creating holes in its re-entrancy protection.
+ * As I released pic, it was quite safe on this score, however, that made
+ * dosemu unpleasantly slow.  For everyone's edification, here's the problem:
+ * 
+ * As dos is designed, it is sometimes necessary to presume that an interrupt
+ * (most often the timer, but sometimes the keyboard or even a serial port)
+ * could not possibly re-trigger before a certain amount of time has elapsed.
+ * Sloppy dos programmers have also often made this assumption even when it was
+ * not necessary.
+ *
+ * Dosemu breaks this assumption, because dos doesn't always have the entire
+ * machine, and interrupts can "pile-up" while other processes are running.
+ * As far as pic (hardware or emulated) is concerned, an interrupt ends
+ * when the interrupt acknowledge is sent to pic.  However, to avoid
+ * re-entrancy, pic.c points the iret to a HLT, which is trapped, and used to
+ * allow re-triggerring of an interrupt.  This works, and would completely
+ * eliminate the above problem, but there's more to the story.
+ *
+ * Since some programs put great piles of code inside an interrupt ("I've ACK'd
+ * the interrupt, what's the hurry on the IRET?"), this kind of hold-off slows
+ * dosemu down.  A few (2 or 3) people have modified pic since I released it
+ * to create holes in the hold-off mechanism, citing the need for more speed.
+ * They were certainly right that speed was needed, but I could never get them
+ * to understand the problem they were creating.  These changes are what's
+ * creating the re-entrancy, I believe.
+ *
+ * To make matters worse, dos uses IRETs for software interrupts, even inside
+ * hardware interrupt routines.  So the only reliable way to catch the IRET of
+ * a hardware interrupt is to point it to a HLT by modifying the stack.  Please
+ * don't question this, I've many hours of debugging time that demonstrated
+ * this quite thoroughly.
+ *
+ * BUT...task-switching environments (MS-Win) use hardware interrupts (mostly
+ * timer and keyboard, but sometimes serial, too) to switch tasks. When a new
+ * task is created, there is no address pointing to a HLT on the (new) stack,
+ * so the end of the interrupt code will never be sensed, and the re-trigger
+ * protection will lock up dosemu.  So it is necessary to detect this event,
+ * and decide that the interrupt has terminated, or we'll never let the timer
+ * (or keyboard, etc. as the case may be) interrupt re-trigger.  So, in a way,
+ * we MUST create some kind of hole in the hold-off process.
+ *
+ * The entire pic debate is one of finding the right kind of hole to make, and
+ * the trade-off is robustness vs. speed. Since the task switch problem only
+ * occurs on the initial invocation of a dos task (when a new stack is built),
+ * I elected to use a conservative hole, to maintain robustness.  Others have
+ * since modified pic.c to gain speed, often without adequate testing of
+ * robustness.
+ * 
+ * There's lots of re-entrancy protection in pic, but it has largely been
+ * defeated by speed-up efforts.  Perhaps the best fix is to re-visit those
+ * speed-up efforts, and see if a better solution can't be found.
+ *
+ * My next thing to try was to put back the correct pic_icount stuff and see if
+ * I can't set up separate HLTs for each IRQ, automatically releasing any
+ * pending (pic_pirr) interrupts when their HLT is encountered.  The pic_icount
+ * and related stuf may still be needed for stack switches.  This may be
+ * enough to gain the full speed-up without creating significant holes.
+ *
+ * The other thing I want to do is some timer changes to better guarantee
+ * interrupts get activated in their proper order.  This is a bit messy, and
+ * I've got the code somewhere. It involves threading the timer queue.  Some of
+ * it may already be done, but I think not.
+ *
+ * Larry
+ */
+
 #include <stdio.h>
 #include "config.h"
 #include "port.h"
@@ -51,10 +132,9 @@
 #else
 #include "bitops.h"
 #endif
+#include "timers.h"
 #include "pic.h"
-#ifdef MONOTON_MICRO_TIMING
 #include "iodev.h"
-#endif /* MONOTON_MICRO_TIMING */
 #include "memory.h"
 #ifdef __linux__
 #include <linux/linkage.h>
@@ -63,7 +143,7 @@
 #include <sys/time.h>
 #include "cpu.h"
 #include "emu.h"
-#include "../dpmi/dpmi.h"
+#include "dpmi.h"
 #include "serial.h"
 #include "int.h"
 #include "ipx.h"
@@ -77,6 +157,17 @@ extern void timer_int_engine(void);
 static unsigned long pic1_isr;         /* second isr for pic1 irqs */
 static unsigned long pic_irq2_ivec = 0;
 
+/*
+ * pic_pirr contains a bit set for each interrupt which has attempted to
+ * re-trigger.
+ *
+ * pic_icount is an attempt to count active interrupts.  Pending (pic_pirr)
+ * interrupts are released when pic_icount gets back to zero (as designed).
+ * Haven't looked how it is now, this is one thing others have played with.
+ *
+ * pic_watch() has the primary job of releasing interrupts when a stack switch
+ * occurs.
+ */
 
 #if 0
 static unsigned long pic1_mask;        /* bits set for pic1 levels */
@@ -237,8 +328,8 @@ char ci,cc;
  * Pic maintains two stacks of the current interrupt level. an internal one
  * is maintained by run_irqs, and is valid whenever the emulator code for
  * an interrupt is active.  These functions maintain an external stack,
- * winch is valid from the time the dos interrupt code is called until
- * that code has issued all necessary EOIs.  Because pic will not necessarily 
+ * which is valid from the time the dos interrupt code is called until
+ * the code has issued all necessary EOIs.  Because pic will not necessarily 
  * get control immediately after an EOI, another EOI (for another interrupt)
  * could occur.  This external stack is kept strictly synchronized with
  * the actions of the dos code to avoid any problems.  pic_push and pic_pop
@@ -306,13 +397,11 @@ unsigned char int_num;
  * They are called by the code that emulates inb and outb instructions.
  * Each function implements both ports for the pic:  pic0 is on ports
  * 0x20 and 0x21; pic1 is on ports 0xa0 and 0xa1.  These functions take
- * two arguements: a port number (0 or 1) and a value to be written.
+ * two arguments: a port number (0 or 1) and a value to be written.
  *
  * DANG_END_FUNCTION
  */
-void write_pic0(port,value)
-Bit32u port;
-Bit8u value;
+void write_pic0(ioport_t port, Bit8u value)
 {
 
 /* if port == 0 this must be either an ICW1, OCW2, or OCW3
@@ -377,9 +466,7 @@ else                              /* icw2, icw3, icw4, or mask register */
 }  
 
 
-void write_pic1(port,value)
-Bit32u port;
-Bit8u value;
+void write_pic1(ioport_t port, Bit8u value)
 {
 /* if port == 0 this must be either an ICW1, OCW2, or OCW3 */
 /* if port == 1 this must be either ICW2, ICW3, ICW4, or load IMR */
@@ -448,8 +535,7 @@ else                         /* icw2, icw3, icw4, or mask register */
  *
  * DANG_END_FUNCTION
  */
-Bit8u read_pic0(port)
-Bit32u port;
+Bit8u read_pic0(ioport_t port)
 {
   port -= 0x20;
   if(port)		return((unsigned char)get_pic0_imr());
@@ -458,8 +544,7 @@ Bit32u port;
 }
 
 
-Bit8u read_pic1(port)
-Bit32u port;
+Bit8u read_pic1(ioport_t port)
 {
   port -= 0xa0;
   if(port)		return((unsigned char)get_pic1_imr());
@@ -659,11 +744,19 @@ if (!(REG(eflags) & VIF)) return;
  *  here and fake cs:ip to PIC_[SEG,OFF], where there is a hlt.  This makes 
  *  the irq generate a sigsegv, which calls pic_iret when it completes.
  *  pic_iret then pops the real cs:ip from the stack.
- *  This routine is RE-ENTRANT - it calls run_irqs, which
+ *  This routine is RE-ENTRANT - it calls run_irqs,
  *  which may call an interrupt routine,
  *  which may call do_irq().  Be Careful!  !!!!!!!!!!!!!!!!!! 
  *  No single interrupt is ever re-entered.
  *
+ * Callers:
+ * base/misc/ioctl.c
+ * base/keyboard/n_serv_8042.c
+ * base/keyboard/keyboard-server.c
+ * base/serial/ser_irq.c
+ * dosext/sound/sound.c
+ * dosext/net/net/pktnew.c
+ *
  * DANG_END_FUNCTION
  */
 int do_irq()
@@ -708,8 +801,13 @@ g_printf("+%d",(int)pic_ilevel);
      }
       pic_icount++;
       pic_wcount++;
+
+ /* enter PIC loop - we can continue only when pic_isr has been cleared */
       while(!fatalerr && test_bit(pic_ilevel,&pic_isr))
       {
+	if (d.request>2)
+		r_printf("------ PIC: intr loop ---%06lx--%06lx----\n",
+			pic_vm86_count,pic_dpmi_count);
 	if (in_dpmi ) {
           ++pic_dpmi_count;
 	  pic_print(2, "Initiating DPMI irq lvl ", pic_ilevel, " in do_irq");
@@ -730,7 +828,9 @@ g_printf("+%d",(int)pic_ilevel);
         pkt_check_receive(0);
 #endif
 #endif
+#ifdef USE_INT_QUEUE
         int_queue_run();        /*  delete when moved to timer stuff */
+#endif
       }
       pic_sti();
       return(0);
@@ -745,7 +845,7 @@ g_printf("+%d",(int)pic_ilevel);
  * pic_request triggers an interrupt.  There is presently no way to
  * "un-trigger" an interrupt.  The interrupt will be initiated the
  * next time pic_run is called, unless masked or superceded by a 
- * higher priority interrupt.  pic_request takes one arguement, an
+ * higher priority interrupt.  pic_request takes one argument, an
  * interrupt level, which specifies the interrupt to be triggered.
  * If that interrupt is already active, the request will be queued 
  * until all active interrupts have been completed.  The queue is
@@ -876,12 +976,13 @@ unsigned long pic_newirr;
  * pic_watch is a watchdog timer for pending interrupts.  If pic_iret
  * somehow fails to activate a pending interrupt request for 2 consecutive 
  * timer ticks, pic_watch will activate them anyway.  pic_watch is called
- * by timer_tick, the interval timer signal handler.
+ * ONLY by timer_tick, the interval timer signal handler, so the two functions
+ * will probably be merged.
  *
  * DANG_END_FUNCTION
  */
 inline void pic_watch(s_time)
-struct timeval* s_time;
+hitimer_u *s_time;	/* time in us, 64-bit unsigned */
 {
 int timer,t_time;
 unsigned long pic_newirr;
@@ -893,16 +994,21 @@ unsigned long pic_newirr;
   pic_wirr|=pic_pirr;   	  /* set a new pending list for next time */
 /*  pic_activate(); */
 
-  /*  calculate new sys_time */
+/*  calculate new sys_time
+ *  non-monoton: values are kept modulo (tick_rate*15min)
+ *  monoton:     values are kept modulo 2^32 (exactly 1 hour)
+ */
 #ifndef MONOTON_MICRO_TIMING
+#if 0	/* original code, see comments in timers.h */
   t_time = (s_time->tv_sec%900)*1193047 
            + (s_time->tv_usec*1193)/1000  /* This is usec * 1.193047 split */
            + s_time->tv_usec/21277;        /* up to fit in 32 bit integer math */
+#endif
+  t_time = nmUStoTICK(s_time->td % 900000000);
 #else /* MONOTON_MICRO_TIMING */
-  t_time = s_time->tv_sec*PIT_TICK_RATE 
-           + PIT_MS2TICKS(s_time->tv_usec);
-
+  t_time = UStoTICK(s_time->td);
 #endif /* MONOTON_MICRO_TIMING */
+
   /* check for any freshly initiated timers, and sync them to s_time */
   pic_print(2,"pic_itime[1]= ",pic_itime[1]," ");
 #ifndef MONOTON_MICRO_TIMING
@@ -912,9 +1018,9 @@ unsigned long pic_newirr;
      for (timer=0;timer<33;++timer) { 
        if(pic_itime[timer]>=pic_dos_time) {
 	 if (pic_itime[timer] != NEVER)
-          pic_itime[timer] -= 900*1193047;
+          pic_itime[timer] -= 900*PIC_TICK_RATE;
          if (pic_ltime[timer] != NEVER)
-          pic_ltime[timer] -= 900*1193047;
+          pic_ltime[timer] -= 900*PIC_TICK_RATE;
           }
        else pic_ltime[timer] = pic_itime[timer] = NEVER;
      }
@@ -922,7 +1028,7 @@ unsigned long pic_newirr;
   pic_sys_time=t_time;
 #else  /* MONOTON_MICRO_TIMING */
   /* wraparound now is smoothly at 1h interval */
-  pic_sys_time=t_time + (t_time == -1);
+  pic_sys_time=t_time + (t_time == NEVER);
 #endif /* MONOTON_MICRO_TIMING */
   pic_print(2,"pic_sys_time set to ",pic_sys_time," ");
   pic_dos_time = pic_itime[32];
@@ -1048,32 +1154,30 @@ void pic_sched(int ilevel, int interval)
 void pic_init(void)
 {
   /* do any one-time initialization of the PIC */
+#ifdef NEW_PORT_CODE
   emu_iodev_t  io_device;
-  emu_hlt_t    hlt_hdlr;
 
   /* 8259 PIC (Programmable Interrupt Controller) */
   io_device.read_portb   = read_pic0;
   io_device.write_portb  = write_pic0;
   io_device.read_portw   = NULL;
   io_device.write_portw  = NULL;
-  io_device.handler_name = "8259 PIC";
+  io_device.read_portd   = NULL;
+  io_device.write_portd  = NULL;
+  io_device.handler_name = "8259 PIC0";
   io_device.start_addr   = 0x0020;
   io_device.end_addr     = 0x0021;
   io_device.irq          = EMU_NO_IRQ;
-  port_register_handler(io_device);
+  io_device.fd = -1;
+  port_register_handler(io_device, 0);
 
+  io_device.handler_name = "8259 PIC1";
   io_device.start_addr = 0x00A0;
   io_device.end_addr   = 0x00A1;
   io_device.read_portb   = read_pic1;
   io_device.write_portb  = write_pic1;
-  port_register_handler(io_device);
-
-  hlt_hdlr.name       = "PIC";
-  hlt_hdlr.start_addr = 0x0fff;
-  hlt_hdlr.end_addr   = 0x0fff;
-  hlt_hdlr.func       = (emu_hlt_func)pic_iret;
-  hlt_register_handler(hlt_hdlr);
-
+  port_register_handler(io_device, 0);
+#endif
 }
 
 void pic_reset(void)
