@@ -2,12 +2,14 @@
 #include <termios.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
 
 #include "config.h"
 #include "emu.h"
+#include "timers.h"
 #include "video.h"
 #include "mouse.h"
 #include "serial.h"
@@ -18,7 +20,9 @@
 
 #include "dos2linux.h"
 #include "priv.h"
-
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 
 /*
  * XXX - the mem size of 734 is much more dangerous than 704. 704 is the
@@ -37,6 +41,34 @@ static void     check_for_env_autoexec_or_config(void);
 int     parse_debugflags(const char *s, unsigned char flag);
 static void     usage(void);
 
+/*
+ * DANG_BEGIN_FUNCTION cpu_override
+ * 
+ * description: 
+ * Process user CPU override from the config file ('cpu xxx') or
+ * from the command line. Returns the selected CPU identifier or
+ * -1 on error. 'config.realcpu' should have already been defined.
+ * 
+ * DANG_END_FUNCTION
+ * 
+ */
+int cpu_override (int cpu)
+{
+    switch (cpu) {
+	case 2:
+	    return CPU_286;
+	case 5: case 6:
+	    if (config.realcpu > CPU_486) return CPU_586;
+	/* fall thru */
+	case 4:
+	    if (config.realcpu > CPU_386) return CPU_486;
+	/* fall thru */
+	case 3:
+	    return CPU_386;
+    }
+    return -1;
+}
+
 /*
  * DANG_BEGIN_FUNCTION config_defaults
  * 
@@ -50,7 +82,87 @@ static void     usage(void);
 static void
 config_defaults(void)
 {
+    FILE *fs;
+    char Line[256];
+    int k = 386;
+
+    vm86s.cpu_type = CPU_386;
+    /* defaults - used when /proc is missing, cpu!=x86 etc. */
+    config.realcpu = CPU_386;
+    config.pci = 0;
+    config.rdtsc = 0;
+    config.mathco = 0;
+    config.smp = 0;
+#ifdef X86_EMULATOR
+    config.emuspeed = 50;	/* instruction cycles per us */
+#endif
+
+    fs=fopen("/proc/cpuinfo","r");
+    if (fs) {
+	while (fgets(Line,250,fs)) {
+	  /* get the CPU (processor#0) info */
+	  if (!strncmp(Line,"cpu",3) && isspace(Line[3])) {
+	    char *p = Line+4;
+	    while (*p && (*p!=':')) p++;
+
+	    /* if it's an x86, check which one */
+	    if ((sscanf(p+1,"%d",&k)==1) && ((k%100)==86)) {
+	      switch ((k/100)%10) {
+	        case 3: config.realcpu = CPU_386;	/* redundant */
+	        	break;
+	        case 5:
+	        case 6: config.realcpu = CPU_586;
+			/* bogospeed currently returns 0; should it deny
+			 * pentium features, fall back into 486 case */
+			if (!bogospeed(&config.cpu_spd, &config.cpu_tick_spd)) {
+			    config.pci = 1;	/* fair guess */
+			    config.rdtsc = 1;
+			    break;
+			}
+			k = 486;
+			/* fall thru */
+	        case 4: config.realcpu = CPU_486;
+	        	break;
+	        default:
+	        	exit(1);	/* no 186,286,786.. */
+	      }
+	    }
+	  }
+	  else if (!strncmp(Line,"fpu",3) && isspace(Line[3])) {
+	    /* get the fpu info. this is currently unused. */
+	    char *p = Line+4;
+	    while (*p && (*p!=':')) p++;
+	    if (*p) {
+	      p++; while (*p && isspace(*p)) p++;
+	      config.mathco = (*p=='y');
+	    }
+	  }
+	  else if (!strncmp(Line,"processor",9)) {
+	    /* if a 'processor:n' statement is found with n>0, meaning we
+	     * are on a SMP machine, deny use of pentium timer, force
+	     * use of kernel timing and stop parsing the file.
+	     */
+	    char *p = Line+9;
+	    while (*p && (*p!=':')) p++;
+	    if (*p) {
+	      p++; while (*p && isspace(*p)) p++;
+	      if (isdigit(*p) && (*p>'0') && config.rdtsc) {
+	  	fprintf(stderr,"Denying use of pentium timer on SMP machine\n");
+		config.smp = 1;		/* for checking overrides, later */
+	      	config.rdtsc = 0;
+	      	break;
+	      }
+	    }
+	  }
+	}
+	fclose(fs);
+    }
+    fprintf(stderr,"Running on CPU=%d86, FPU=%d\n",config.realcpu,config.mathco);
+
     config.hdiskboot = 1;	/* default hard disk boot */
+#ifdef X86_EMULATOR
+    config.cpuemu = 0;
+#endif
     config.mem_size = 640;
     config.ems_size = 0;
     config.ems_frame = 0xd000;
@@ -59,10 +171,8 @@ config_defaults(void)
     config.dpmi = 0;
     config.secure = 1;  /* need to have it 'on', else user may trick it out
                            via -F option */
-    config.mathco = 1;
     config.mouse_flag = 0;
     config.mapped_bios = 0;
-    config.mapped_sbios = 0;
     config.vbios_file = NULL;
     config.vbios_copy = 0;
     config.vbios_seg = 0xc000;
@@ -90,6 +200,18 @@ config_defaults(void)
     config.X_blinkrate = 8;
     config.X_sharecmap = 0;     /* Don't share colourmap in graphics modes */
     config.X_mitshm = 0;
+    config.X_fixed_aspect = 1;
+    config.X_aspect_43 = 0;
+    config.X_lin_filt = 0;
+    config.X_bilin_filt = 0;
+    config.X_mode13fact = 2;
+    config.X_winsize_x = 0;
+    config.X_winsize_y = 0;
+    config.X_gamma = 100;
+    config.vgaemu_memsize = 0;
+    config.vesamode_list = NULL;
+    config.X_lfb = 1;
+    config.X_pm_interface = 1;
     config.X_keycode = 0;
     config.X_font = "vga";
     config.usesX = 0;
@@ -97,6 +219,7 @@ config_defaults(void)
     config.hogthreshold = 10;	/* bad estimate of a good garrot value */
     config.chipset = PLAINVGA;
     config.cardtype = CARD_VGA;
+    config.pci_video = 0;
     config.fullrestore = 0;
     config.graphics = 0;
     config.gfxmemsize = 256;
@@ -112,8 +235,10 @@ config_defaults(void)
      * better that 55555 (108Hz) because of integer math.
      * see timer_interrupt_init() in init.c
      */
-    config.update = 54945;	/* should be = 1E6/config.freq */
+    config.update = 54925;	/* should be = 1E6/config.freq */
     config.freq = 18;		/* rough frequency (real PC = 18.2065) */
+    config.wantdelta = 9154;	/* requested value for setitimer */
+    config.realdelta = 9154;
 
     config.timers = 1;		/* deliver timer ints */
     config.keybint = 1;		/* keyboard interrupts */
@@ -125,7 +250,6 @@ config_defaults(void)
 
     config.num_ser = 0;
     config.num_lpt = 0;
-    vm86s.cpu_type = CPU_386;
     config.fastfloppy = 1;
 
     config.emusys = (char *) NULL;
@@ -182,7 +306,7 @@ open_terminal_pipe(char *path)
     leave_priv_setting();
     if (terminal_fd == -1) {
 	terminal_pipe = 0;
-	error("ERROR: open_terminal_pipe failed - cannot open %s!\n", path);
+	error("open_terminal_pipe failed - cannot open %s!\n", path);
 	return;
     } else
 	terminal_pipe = 1;
@@ -194,7 +318,7 @@ open_Xkeyboard_pipe(char *path)
     keypipe = DOS_SYSCALL(open(path, O_RDWR));
     if (keypipe == -1) {
 	keypipe = 0;
-	error("ERROR: open_Xkeyboard_pipe failed - cannot open %s!\n", path);
+	error("open_Xkeyboard_pipe failed - cannot open %s!\n", path);
 	return;
     }
     return;
@@ -206,7 +330,7 @@ open_Xmouse_pipe(char *path)
     mousepipe = DOS_SYSCALL(open(path, O_RDWR));
     if (mousepipe == -1) {
 	mousepipe = 0;
-	error("ERROR: open_Xmouse_pipe failed - cannot open %s!\n", path);
+	error("open_Xmouse_pipe failed - cannot open %s!\n", path);
 	return;
     }
     return;
@@ -232,8 +356,12 @@ config_init(int argc, char **argv)
     char           *basename;
     char           *dexe_name = 0;
 
-    config_defaults();
+    if (argv[1] && !strcmp("--version",argv[1])) {
+      usage();
+      exit(0);
+    }
 
+    config_defaults();
     basename = strrchr(argv[0], '/');	/* parse the program name */
     basename = basename ? basename + 1 : argv[0];
 
@@ -252,7 +380,7 @@ config_init(int argc, char **argv)
 
     opterr = 0;
     confname = CONFIG_FILE;
-    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:VNtsgx:KL:m234e:E:dXY:Z:o:Ou:")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:VNtsgx:KL:m23456e:E:dXY:Z:o:Ou:")) != EOF) {
 	switch (c) {
 	case 'F':
 	    if (get_orig_uid()) {
@@ -296,6 +424,20 @@ config_init(int argc, char **argv)
 		exit(1);
 	    }
 	    break;
+	case '2': case '3': case '4': case '5': case '6':
+#if 1
+	    {
+		int cpu = cpu_override (c-'0');
+		if (cpu > 0) {
+			fprintf(stderr,"CPU set to %d86\n",cpu);
+			vm86s.cpu_type = cpu;
+		}
+		else
+			fprintf(stderr,"error in CPU user override\n");
+	    }
+#endif
+	    break;
+
 	case 'u': {
 		extern int define_config_variable(char *name);
 		char *s=malloc(strlen(optarg)+3);
@@ -329,6 +471,11 @@ config_init(int argc, char **argv)
     if (config.exitearly)
 	leavedos(0);
 
+    if (vm86s.cpu_type > config.realcpu) {
+    	vm86s.cpu_type = config.realcpu;
+    	fprintf(stderr, "CONF: emulated CPU forced down to real CPU: %ld86\n",vm86s.cpu_type);
+    }
+
 #ifdef __NetBSD__
     optreset = 1;
     optind = 1;
@@ -337,7 +484,7 @@ config_init(int argc, char **argv)
     optind = 0;
 #endif
     opterr = 0;
-    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:v:VNtT:sgx:KLm2345e:dXY:Z:E:o:Ou:")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:v:VNtT:sgx:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
 	switch (c) {
 	case 'F':		/* previously parsed config file argument */
 	case 'I':
@@ -346,6 +493,7 @@ config_init(int argc, char **argv)
 	case 'O':
 	case 'L':
 	case 'u':
+	case '2': case '3': case '4': case '5': case '6':
 	    break;
 	case 'A':
 	    if (!dexe_running) config.hdiskboot = 0;
@@ -410,7 +558,7 @@ config_init(int argc, char **argv)
 	    if (terminal_fd == -1) {
 		open_terminal_pipe(optarg);
 	    } else
-		error("ERROR: terminal pipe already open\n");
+		error("terminal pipe already open\n");
 	    break;
 	case 'V':
 	    g_printf("Configuring as VGA video card & mapped ROM\n");
@@ -421,7 +569,7 @@ config_init(int argc, char **argv)
 	    break;
 	case 'v':
 	    config.cardtype = atoi(optarg);
-	    if (config.cardtype > 7)	/* keep it updated when adding a new card! */
+	    if (config.cardtype > MAX_CARDTYPE)	/* keep it updated when adding a new card! */
 		config.cardtype = 1;
 	    g_printf("Configuring cardtype as %d\n", config.cardtype);
 	    break;
@@ -462,26 +610,6 @@ config_init(int argc, char **argv)
 	    config.mouse_flag = 1;
 	    break;
 
-	case '2':
-	    g_printf("CPU set to 286\n");
-	    vm86s.cpu_type = CPU_286;
-	    break;
-
-	case '3':
-	    g_printf("CPU set to 386\n");
-	    vm86s.cpu_type = CPU_386;
-	    break;
-
-	case '4':
-	    g_printf("CPU set to 486\n");
-	    vm86s.cpu_type = CPU_486;
-	    break;
-
-	case '5':
-	    g_printf("CPU set to 586\n");
-	    vm86s.cpu_type = CPU_586;
-	    break;
-
 	case 'E':
 	    g_printf("DOS command given on command line\n");
 	    misc_e6_store_command(optarg);
@@ -497,11 +625,13 @@ config_init(int argc, char **argv)
 	}
     }
     if (config.X) {
+#ifndef NEW_KBD_CODE
 	if (!config.X_keycode) {
 	    extern void keyb_layout(int layout);
 	    keyb_layout(-1);
 	    c_printf("CONF: Forceing neutral Keyboard-layout, X-server will translate\n");
 	}
+#endif /* NEW_KBD_CODE */
 	config.console_video = config.vga = config.graphics = 0;
     }
     check_for_env_autoexec_or_config();
@@ -570,9 +700,9 @@ int parse_debugflags(const char *s, unsigned char flag)
     char            c;
 
 #ifdef X_SUPPORT
-    const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrS";
+    const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrSe";
 #else
-    const char      allopts[] = "dRWDCvkiTsm#pgcwhIExMnPrS";
+    const char      allopts[] = "dRWDCvkiTsm#pgcwhIExMnPrSe";
 #endif
 
     /*
@@ -672,12 +802,21 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'S':		/* SOUND */
 	    d.sound = flag;
 	    break;
+#ifdef X86_EMULATOR
+	case 'e':		/* cpu-emu */
+	    d.emu = flag;
+	    break;
+#endif
 	case 'a':{		/* turn all on/off depending on flag */
 		char           *newopts = (char *) malloc(strlen(allopts) + 2);
 
 		newopts[0] = flag ? '+' : '-';
 		newopts[1] = 0;
 		strcat(newopts, allopts);
+#ifdef X86_EMULATOR
+		/* hack-do not set 'e' flag if not explicitly specified */
+		{char *p=newopts; while (*p) {if (*p=='e') *p='g'; p++;}}
+#endif
 		parse_debugflags(newopts, flag);
 		free(newopts);
 		break;
@@ -697,51 +836,58 @@ int parse_debugflags(const char *s, unsigned char flag)
 static void
 usage(void)
 {
-    fprintf(stdout, "dosemu 0.66\n");
-    fprintf(stdout, "USAGE:\n dos [-ABCckbVNtsgxKm234e] [-D flags] [-M SIZE] [-P FILE] [ {-F|-L} File ] 2> dosdbg\n");
-    fprintf(stdout, "    -2,3,4,5 choose 286, 386, 486 or 586 CPU\n");
-    fprintf(stdout, "    -A boot from first defined floppy disk (A)\n");
-    fprintf(stdout, "    -B boot from second defined floppy disk (B) (#)\n");
-    fprintf(stdout, "    -b map BIOS into emulator RAM (%%)\n");
-    fprintf(stdout, "    -C boot from first defined hard disk (C)\n");
-    fprintf(stdout, "    -c use PC console video (!%%)\n");
-    fprintf(stdout, "    -d detach (?)\n");
+    fprintf(stdout,
+	"dosemu-" VERSTR "\n\n"
+	"USAGE:\n"
+	"  dos  [-ABCckbVNtsgxKm23456ez] \\\n"
+	"       [-D flags] [-M SIZE] [-P FILE] [ {-F|-L} File ] \\\n"
+	"       [-o dbgfile] 2> vital_logs\n"
+	"  dos --version\n\n"
+	"    -2,3,4,5,6 choose 286, 386, 486 or 586 or 686 CPU\n"
+	"    -A boot from first defined floppy disk (A)\n"
+	"    -B boot from second defined floppy disk (B) (#)\n"
+	"    -b map BIOS into emulator RAM (%%)\n"
+	"    -C boot from first defined hard disk (C)\n"
+	"    -c use PC console video (!%%)\n"
+	"    -d detach (?)\n"
 #ifdef X_SUPPORT
-    fprintf(stdout, "    -X run in X Window (#)\n");
+	"    -X run in X Window (#)\n"
 /* seems no longer valid bo 18.7.95
-    fprintf(stdout, "    -Y NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n");
-    fprintf(stdout, "    -Z NAME use FIFO NAME for mouse (only with x2dos!)\n");
+	"    -Y NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n"
+	"    -Z NAME use FIFO NAME for mouse (only with x2dos!)\n"
 */
-    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWXcdghikmnprsvwx}\n");
+	"    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWXcdghikmnprsvwx}\n"
 #else				/* X_SUPPORT */
-    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWcdghikmnprsvwx}\n");
+	"    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWcdghikmnprsvwx}\n"
 #endif				/* X_SUPPORT */
-    fprintf(stdout, "       #=defint  C=cdrom    D=dos    E=ems       I=ipc     M=dpmi\n");
-    fprintf(stdout, "       P=packet  R=diskread S=sound  W=diskwrite c=config  d=disk\n");
-    fprintf(stdout, "       g=general h=hardware i=i/o    k=keyb      m=mouse   n=ipxnet\n");
-    fprintf(stdout, "       p=printer r=pic      s=serial v=video     w=warning x=xms\n");
-    fprintf(stdout, "    -E STRING pass DOS command on command line\n");
-    fprintf(stdout, "    -e SIZE enable SIZE K EMS RAM\n");
-    fprintf(stdout, "    -F use config-file File\n");
-    fprintf(stdout, "    -L load and execute DEXE File\n");
-    fprintf(stdout, "    -I insert config statements (on commandline)\n");
-    fprintf(stdout, "    -g enable graphics modes (!%%#)\n");
-    fprintf(stdout, "    -K Do int9 (!#)\n");
-    fprintf(stdout, "    -k use PC console keyboard (!)\n");
-    fprintf(stdout, "    -M set memory size to SIZE kilobytes (!)\n");
-    fprintf(stdout, "    -m enable mouse support (!#)\n");
-    fprintf(stdout, "    -N No boot of DOS\n");
-    fprintf(stdout, "    -O write debugmessages to stderr\n");
-    fprintf(stdout, "    -o FILE put debugmessages in file\n");
-    fprintf(stdout, "    -P copy debugging output to FILE\n");
-    fprintf(stdout, "    -T DIR set tmpdir\n");
-    fprintf(stdout, "    -t try new timer code (#)\n");
-    fprintf(stdout, "    -V use BIOS-VGA video modes (!#%%)\n");
-    fprintf(stdout, "    -v NUM force video card type\n");
-    fprintf(stdout, "    -x SIZE enable SIZE K XMS RAM\n");
-    fprintf(stdout, "    (!) BE CAREFUL! READ THE DOCS FIRST!\n");
-    fprintf(stdout, "    (%%) require dos be run as root (i.e. suid)\n");
-    fprintf(stdout, "    (#) options do not fully work yet\n\n");
-    fprintf(stdout, "xdos [options]           == dos [options] -X\n");
-    fprintf(stdout, "dosexec [options] <file> == dos [options] -L <file>\n");
+	"       #=defint  C=cdrom    D=dos    E=ems       I=ipc     M=dpmi\n"
+	"       P=packet  R=diskread S=sound  W=diskwrite c=config  d=disk\n"
+	"       g=general h=hardware i=i/o    k=keyb      m=mouse   n=ipxnet\n"
+	"       p=printer r=pic      s=serial v=video     w=warning x=xms\n"
+	"    -E STRING pass DOS command on command line\n"
+	"    -e SIZE enable SIZE K EMS RAM\n"
+	"    -F use config-file File\n"
+	"    -L load and execute DEXE File\n"
+	"    -I insert config statements (on commandline)\n"
+	"    -g enable graphics modes (!%%#)\n"
+	"    -K Do int9 (!#)\n"
+	"    -k use PC console keyboard (!)\n"
+	"    -M set memory size to SIZE kilobytes (!)\n"
+	"    -m enable mouse support (!#)\n"
+	"    -N No boot of DOS\n"
+	"    -O write debugmessages to stderr\n"
+	"    -o FILE put debugmessages in file\n"
+	"    -P copy debugging output to FILE\n"
+	"    -T DIR set tmpdir\n"
+	"    -t try new timer code (#)\n"
+	"    -V use BIOS-VGA video modes (!#%%)\n"
+	"    -v NUM force video card type\n"
+	"    -x SIZE enable SIZE K XMS RAM\n"
+	"    --version, print version of dosemu\n"
+	"    (!) BE CAREFUL! READ THE DOCS FIRST!\n"
+	"    (%%) require dos be run as root (i.e. suid)\n"
+	"    (#) options do not fully work yet\n\n"
+	"xdos [options]           == dos [options] -X\n"
+	"dosexec [options] <file> == dos [options] -L <file>\n"
+    );
 }
