@@ -53,6 +53,7 @@
 #include "iodev.h"
 #include "priv.h"
 #include "doshelpers.h"
+#include "speaker.h"
 
 #ifdef NEW_KBD_CODE
 #include "keyb_clients.h"
@@ -185,13 +186,31 @@ void time_setting_init(void)
 void timer_interrupt_init(void)
 {
   struct itimerval itv;
-  int delta = (config.update / TIMER_DIVISOR);
+  int delta;
 
+  /* DANG_FIXTHIS config.timer is undocumented and probably obsolete
+   */
+  delta = (config.update / TIMER_DIVISOR);
+  config.wantdelta = delta;	/* save it */
+  /* Check that the kernel actually supports such a frequency - we
+   * can't go faster than jiffies with setitimer()
+   */
+  if (((delta/1000)+1) < (1000/HZ)) {
+    c_printf("TIME: FREQ too fast, using defaults\n");
+    config.update = 54925; config.freq = 18;
+    delta = 54925 / TIMER_DIVISOR;
+  }
+  config.realdelta = delta;
+
+#ifdef X86_EMULATOR
+  if (config.cpuemu>1) return;
+#endif
   itv.it_interval.tv_sec = 0;
   itv.it_interval.tv_usec = delta;
   itv.it_value.tv_sec = 0;
   itv.it_value.tv_usec = delta;
-  k_printf("Used %d for updating timers\n", delta);
+  c_printf("TIME: using %d usec for updating ALRM timer\n", delta);
+
   setitimer(TIMER_TIME, &itv, NULL);
 }
 
@@ -232,7 +251,7 @@ void hardware_setup(void)
   /* DMA Init */
   /* dma_init(); - Currently in dev_list */
 
-  g_printf("PIC,mouse,IPX,DMA initialized\n");
+  g_printf("PIC,mouse,IPX initialized\n");
 }
 
 /*
@@ -300,7 +319,7 @@ static inline void map_hardware_ram(void)
       size = (i - j) << 12;
       if (mmap((caddr_t) addr, (size_t) size, PROT_READ | PROT_WRITE, 
 	       MAP_SHARED | MAP_FIXED, mem_fd, addr) == (caddr_t) -1) {
-	error("ERROR: mmap error in map_hardware_ram %s\n", strerror (errno));
+	error("mmap error in map_hardware_ram %s\n", strerror (errno));
 	close_kmem();
 	return;
       }
@@ -328,6 +347,8 @@ static inline void map_custom_bios(void)
 
   ptr = (u_char *) (BIOSSEG << 4);
   memcpy(ptr, bios_f000, (u_long)bios_f000_end - (u_long)bios_f000);
+
+#if 0 /* this apparently is obsolete stuff, we have all in bios.S */
   if (!config.mapped_sbios) {
     memset((char *) 0xffff0, 0xF4, 16);
 
@@ -341,6 +362,7 @@ static inline void map_custom_bios(void)
   *ptr++ = 0xff;
   *ptr++ = 0xcd;                /* int DOS_HELPER_INT(0xe6) */
   *ptr++ = DOS_HELPER_INT;
+#endif
 
 #if 0 /* for debugging only */
   f = fopen("/tmp/bios","w");
@@ -426,7 +448,7 @@ void memory_init(void)
 #if 0
   if( first_call == 0) {
 #ifdef __linux__
-    /* make interrupts vector read-write */
+    /* make interrupt vectors read-write */
     mprotect((void *)(BIOSSEG<<4), 0x1000, PROT_WRITE|PROT_READ|PROT_EXEC);
     /* make memory area 0xf8000 to 0xfffff read write */
     mprotect((void *)(ROMBIOSSEG<<4), 0x8000, PROT_WRITE|PROT_READ|PROT_EXEC);
@@ -507,23 +529,21 @@ void memory_init(void)
  */
 void device_init(void)
 {
-  port_init();
-
 #ifdef NEW_KBD_CODE
   /* check whether we are running on the console */
   check_console();
 
   if (!keyb_server_init()) {
-    error("ERROR: can't init keyboard server\n");
+    error("can't init keyboard server\n");
     leavedos(19);
   }
   if (!keyb_client_init()) {
-    error("ERROR: can't open keyboard client\n");
+    error("can't open keyboard client\n");
     leavedos(19);
   }
 #else
   if (keyboard_init() != 0) {
-    error("ERROR: can't open keyboard\n");
+    error("can't open keyboard\n");
     leavedos(19);
   }
   keyboard_flags_init();
@@ -534,6 +554,10 @@ void device_init(void)
  
   scr_state_init();
   video_config_init();
+  if (config.console && (config.speaker == SPKR_EMULATED)) {
+    register_speaker((void *)console_fd,
+		     console_speaker_on, console_speaker_off);
+  }
   iodev_init();
   mouse_init();
   printer_init();
