@@ -7,6 +7,7 @@
 #include <sys/types.h>
 #include "slang.h"
 #include "emu.h"
+#include "timers.h"
 #include "keymaps.h"
 #include "keyb_clients.h"
 #include "keyboard.h"
@@ -537,7 +538,8 @@ static int getkey_callback(void)
  */
 static int sltermio_input_pending(void)
 {
-  struct timeval scr_tv, t_start, t_end;
+  struct timeval scr_tv;
+  hitimer_t t_start;
   fd_set fds;
   long t_dif;
 
@@ -551,12 +553,10 @@ static int sltermio_input_pending(void)
   scr_tv.tv_sec = 0L;
   scr_tv.tv_usec = THE_TIMEOUT;
 
-  gettimeofday(&t_start, NULL);
+  t_start = GETusTIME(0);
   errno = 0;
   while ((int)select(kbd_fd + 1, &fds, NULL, NULL, &scr_tv) < (int)1) {
-    gettimeofday(&t_end, NULL);
-    t_dif = ((t_end.tv_sec * 1000000 + t_end.tv_usec) -
-	     (t_start.tv_sec * 1000000 + t_start.tv_usec));
+    t_dif = (long)(GETusTIME(0) - t_start);
 
     if ((t_dif >= THE_TIMEOUT) || (errno != EINTR))
       return 0;
@@ -598,6 +598,29 @@ static void slang_send_scancode(unsigned long lscan, unsigned char ch)
     presskey(KEY_L_ALT,0);
   }
 
+  /* Sanity adjustments on ch */
+
+  if (lscan & (STICKY_SHIFT_MASK | SHIFT_MASK)) {
+    if ((ch >= 0x61) && (ch <= 0x7a)) {
+      ch -= 0x20;
+    }
+  }
+
+  if (lscan & (STICKY_CTRL_MASK | CTRL_MASK)) {
+    /* actually this is probably a little extra */
+    /* taken form n_serv_xlate.c ... translate .. (shiftstate & CTRL) */
+    if ((ch >= 0x40) && (ch <= 0x7e)) {
+      ch &= 0x1f;
+    } 
+    else if (ch == 0x2D) {
+      ch = 0x1f;
+    }
+  }
+
+  if (lscan & (STICKY_ALT_MASK | ALT_MASK)) {
+    ch = 0;
+  }
+
   presskey(lscan & 0x0000ffff, ch); 
   releasekey(lscan & 0x0000ffff); 
 
@@ -805,7 +828,7 @@ void do_slang_getkeys(void)
  * 
  * Code is called at start up to set up the terminal line for non-raw mode.
  * 
- * DANG_END_REMARK
+ * DANG_END_FUNCTION
  */
    
 int slang_keyb_init(void) {
@@ -819,7 +842,7 @@ int slang_keyb_init(void) {
   fcntl(kbd_fd, F_SETFL, O_RDONLY | O_NONBLOCK);
    
   if (tcgetattr(kbd_fd, &save_termios) < 0) {
-    error("ERROR: slang_keyb_init(): Couldn't tcgetattr(kbd_fd,...) !\n");
+    error("slang_keyb_init(): Couldn't tcgetattr(kbd_fd,...) !\n");
     leavedos(66);
   }
 
@@ -838,10 +861,10 @@ int slang_keyb_init(void) {
   cfgetospeed(&buf);
 #endif
   if (tcsetattr(kbd_fd, TCSANOW, &buf) < 0) {
-    error("ERROR: slang_keyb_init(): Couldn't tcsetattr(kbd_fd,TCSANOW,...) !\n");
+    error("slang_keyb_init(): Couldn't tcsetattr(kbd_fd,TCSANOW,...) !\n");
   }
   if (-1 == init_slang_keymaps()) {
-    error("ERROR: Unable to initialize S-Lang keymaps.\n");
+    error("Unable to initialize S-Lang keymaps.\n");
     leavedos(31);
   }
 
@@ -860,7 +883,7 @@ int slang_keyb_init(void) {
 
 void slang_keyb_close(void)  {
    if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1) {
-      error("ERROR: slang_keyb_close(): failed to restore keyboard termios settings!\n");
+      error("slang_keyb_close(): failed to restore keyboard termios settings!\n");
    }
    if (save_kbd_flags != -1) {
       fcntl(kbd_fd, F_SETFL, save_kbd_flags);
