@@ -5,20 +5,24 @@
  * 
  * Maintainer: Rainer Zimmermann <zimmerm@mathematik.uni-marburg.de>
  * 
+ * REMARK
  * This module contains the the translation part of the keyboard 'server',
  * which translates keysyms into the form in which they can be sent do DOS.
  *
  * The frontends will call one of the following functions to send
  * keyboard events to DOS:
  *
+ * VERB
  *     putrawkey(t_rawkeycode code);
  *     putkey(Boolean make, t_keysym key)
  *     set_shiftstate(t_shiftstate s);
+ * /VERB
  *
  * Interface to serv_backend.c is through write_queue(bios_key, shift, raw).
  *
  * More information about this module is in doc/README.newkbd
  *
+ * /REMARK
  * DANG_END_MODULE
  *
  */
@@ -232,8 +236,10 @@ static Bit16u make_bios_code(Boolean make, t_keysym key, uchar ascii) {
             bios_scan=0xe0;
          }
          else if (key&0xff00) {
-            if (key>=E0_MINIDX && key<=E0_MAXIDX)
+            if (key>=E0_MINIDX && key<=E0_MAXIDX) {
                bios_scan=bios_ctrl_e0_scancodes[key-E0_MINIDX];
+			   ascii=0xE0;
+			}
          }
          else {
             bios_scan=bios_ctrl_scancodes[key&0xff];
@@ -280,6 +286,8 @@ static Bit16u make_bios_code(Boolean make, t_keysym key, uchar ascii) {
           case KEY_PRTSCR:
           case KEY_PAD_AST:
                   bios_scan=0; break;
+		  case 0xe047 ... 0xe053:
+				  ascii=0xe0; bios_scan=key&0x7f; break;
           case KEY_PAD_SLASH:
                   ascii='/';   /* fallthrough */
           default:
@@ -302,6 +310,8 @@ static Bit16u make_bios_code(Boolean make, t_keysym key, uchar ascii) {
                   bios_scan=0xe0; break;
           case KEY_PRTSCR:
                   bios_scan=0; break;
+          case 0xe047 ... 0xe053:
+                  ascii=0xe0;
           default:
                   bios_scan=key&0x7f; break;
          }
@@ -309,6 +319,10 @@ static Bit16u make_bios_code(Boolean make, t_keysym key, uchar ascii) {
       /* all shiftstates */
       if (key==KEY_SPACE) ascii=' ';
       if (key==KEY_DEL) ascii=0;
+      
+      /* distinction cursor block / keypad */
+      if (ascii == 0 && (key&0xff00) == 0xe000)
+         ascii = 0xe0;
    }
    else { /* !make */
       if (key==KEY_SYSRQ) {
@@ -419,7 +433,7 @@ static uchar translate(t_keysym key, Boolean *is_accent) {
    else if (key>=0x47 && key<=0x53)  {          /* keypad keys */
 
       if (((!!(shiftstate & NUM_LOCK)) ^ (!!(shiftstate & SHIFT))) ||
-          (shiftstate&ALT))
+          (shiftstate&ALT) || (key==KEY_PAD_PLUS) || (key==KEY_PAD_MINUS) )
       {
         ch = config.num_table[key-0x47];
       }
@@ -521,12 +535,13 @@ static Boolean handle_dosemu_keys(t_keysym key) {
 
    if ((shiftstate&CTRL) && (shiftstate&ALT)) {
       switch(key) {
+#if 0	/* C-A-D is disabled */
        case KEY_DEL:
        case KEY_PGUP:
              k_printf("KBD: Ctrl-Alt-{Del|PgUp}: rebooting dosemu\n");
              dos_ctrl_alt_del();
              return 1;
-
+#endif
        case KEY_PGDN:
              k_printf("KBD: Ctrl-Alt-PgDn: bye bye!\n");
              leavedos(0);
@@ -583,7 +598,7 @@ static Boolean handle_dosemu_keys(t_keysym key) {
  */
 
 void putrawkey(t_rawkeycode code) {
-   Boolean make,is_shift,is_accent;
+   Boolean make,is_shift,is_accent=0;
    t_scancode scan;
    t_keysym key;
    Bit16u bios_key;
@@ -632,9 +647,17 @@ void putrawkey(t_rawkeycode code) {
 
       is_shift = do_shift_keys(make,key);
 
-      ascii = (make && !is_shift) ? translate(key, &is_accent) : 0;
+      ascii = (make && !is_shift) ? translate(key, &is_accent) : (is_accent=0);
 
-#if 0     
+      /* quick hack: translate ALT+letter keys */
+      if (   config.keyboard != KEYB_US && (shiftstate & ALT)
+          && config.key_map[key] >= 'a' && config.key_map[key] <= 'z')
+      {
+        extern const Bit8u ascii_keys[];
+        key = ascii_keys[config.key_map[key] - 0x20];
+      }
+
+#if 1     
       k_printf("KBD: in putrawkey(): make=%d is_shift=%d key=%08x isaccent=%d ascii=0x%02x\n",
                make,is_shift,key,is_accent,ascii);
 #endif
