@@ -93,11 +93,7 @@ int
 read_sectors(struct disk *dp, char *buffer, long head, long sector,
 	     long track, long count)
 {
-#ifdef __linux__
   loff_t  pos;
-#else
-  off_t  pos;
-#endif
   long already = 0;
   long tmpread;
 
@@ -119,7 +115,7 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
     int mbrcount, mbrinc;
 
     if (dp->type != PARTITION) {
-      error("ERROR: negative offset on non-partition disk type\n");
+      error("negative offset on non-partition disk type\n");
       return -DERR_NOTFOUND;
     }
 
@@ -140,7 +136,7 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
        * the dest. buffer?  No matter, I guess.
        */
     if ((mbroff + mbrcount) > dp->part_info.mbr_size) {
-      error("ERROR: %s writing in the forbidden fake partition zone!\n",
+      error("%s writing in the forbidden fake partition zone!\n",
 	    dp->dev_name);
       mbrcount = dp->part_info.mbr_size - mbroff;
     }
@@ -180,7 +176,7 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
 #else
   if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
 #endif
-    error("ERROR: Sector not found in read_sector, error = %s!\n",
+    error("Sector not found in read_sector, error = %s!\n",
 	  strerror(errno));
     return -DERR_NOTFOUND;
   }
@@ -196,11 +192,7 @@ int
 write_sectors(struct disk *dp, char *buffer, long head, long sector,
 	      long track, long count)
 {
-#ifdef __linux__
   loff_t  pos;
-#else
-  off_t pos;
-#endif
   int tmpwrite;
 
   if (dp->rdonly) {
@@ -237,7 +229,7 @@ write_sectors(struct disk *dp, char *buffer, long head, long sector,
 #else
   if (pos != lseek(dp->fdesc, pos, SEEK_SET)) {
 #endif
-    error("ERROR: Sector not found in write_sector!\n");
+    error("Sector not found in write_sector!\n");
     return -DERR_NOTFOUND;
   }
 
@@ -287,13 +279,13 @@ image_auto(struct disk *dp)
   lseek(dp->fdesc, 0, SEEK_SET);/* Use lseek here, as nobody want
 				   > 2^^31 bytes for the image file */
   if (RPT_SYSCALL(read(dp->fdesc, header, HEADER_SIZE)) != HEADER_SIZE) {
-    error("ERROR: could not read full header in image_init\n");
+    error("could not read full header in image_init\n");
     leavedos(19);
   }
 
   if (strncmp(header, IMAGE_MAGIC, IMAGE_MAGIC_SIZE)
 		&& (*((long *)header) != DEXE_MAGIC) ) {
-    error("ERROR: IMAGE %s header lacks magic string - cannot autosense!\n",
+    error("IMAGE %s header lacks magic string - cannot autosense!\n",
 	  dp->dev_name);
     leavedos(20);
   }
@@ -317,7 +309,7 @@ hdisk_auto(struct disk *dp)
 
   if (ioctl(dp->fdesc, DIOCGDINFO, &label) != 0 ||
       fstat(dp->fdesc, &stb) != 0) {
-      error("ERROR: can't fstat %s: %s\n", dp->dev_name,
+      error("can't fstat %s: %s\n", dp->dev_name,
 	    strerror(errno));
       leavedos(21);
   } else {
@@ -333,7 +325,7 @@ hdisk_auto(struct disk *dp)
   struct hd_geometry geo;
 
   if (ioctl(dp->fdesc, HDIO_GETGEO, &geo) < 0) {
-    error("ERROR: can't get GEO of %s: %s\n", dp->dev_name,
+    error("can't get GEO of %s: %s\n", dp->dev_name,
 	  strerror(errno));
     leavedos(21);
   }
@@ -389,7 +381,7 @@ partition_setup(struct disk *dp)
   part_fd = DOS_SYSCALL(open(hd_name, O_RDONLY));
   leave_priv_setting();
   if (part_fd == -1) {
-    error("ERROR: opening device %s to read MBR for PARTITION %s\n",
+    error("opening device %s to read MBR for PARTITION %s\n",
 	  hd_name, dp->dev_name);
     leavedos(22);
   }
@@ -427,7 +419,7 @@ partition_setup(struct disk *dp)
 			     dp->part_info.beg_cyl) +
 		 (SECTOR_SIZE * (dp->part_info.pre_secs - 1)));
 #else
-  dp->header = -(SECTOR_SIZE * (dp->part_info.pre_secs));
+  dp->header = -(SECTOR_SIZE * (loff_t) (dp->part_info.pre_secs));
 #endif
 
   dp->part_info.mbr_size = SECTOR_SIZE;
@@ -479,7 +471,7 @@ set_part_ent(struct disk *dp, char *tmp_mbr)
 
   if (ioctl(dp->fdesc, DIOCGDINFO, &label) != 0 ||
       fstat(dp->fdesc, &stb) != 0) {
-      error("ERROR: can't fstat %s: %s\n", dp->dev_name,
+      error("can't fstat %s: %s\n", dp->dev_name,
 	    strerror(errno));
       leavedos(21);
   } else
@@ -487,7 +479,7 @@ set_part_ent(struct disk *dp, char *tmp_mbr)
 #endif
 #ifdef __linux__
   if (ioctl(dp->fdesc, BLKGETSIZE, &length)) {
-    error("ERROR: calling ioctl BLKGETSIZE for PARTITION %s\n", dp->dev_name);
+    error("calling ioctl BLKGETSIZE for PARTITION %s\n", dp->dev_name);
     leavedos(22);
   }
 #endif
@@ -584,7 +576,7 @@ disk_open(struct disk *dp)
       dp->tracks = 0;
       return;
     }
-    error("ERROR: can't get floppy parameter of %s (%s)\n", dp->dev_name, strerror(err));
+    error("can't get floppy parameter of %s (%s)\n", dp->dev_name, strerror(err));
     fatalerr = 5;
     return;
   }
@@ -657,9 +649,14 @@ disk_open(struct disk *dp)
    * during the ioctl. (well, not what we really like)
    * ( 19 May 1996, Hans Lermen ) */
   int res;
+ #ifdef USE_THREADS
+  static int background_ioctl(int fd, int request, void *param);
+  res = background_ioctl(dp->fdesc, FDGETPRM, &fl);
+ #else
   sigalarm_onoff(0);
   res = ioctl(dp->fdesc, FDGETPRM, &fl);
   sigalarm_onoff(1);
+ #endif
   if (res == -1) {
 #else
   if (ioctl(dp->fdesc, FDGETPRM, &fl) == -1) {
@@ -670,7 +667,7 @@ disk_open(struct disk *dp)
       dp->tracks = 0;
       return;
     }
-    error("ERROR: can't get floppy parameter of %s (%s)\n", dp->dev_name, sys_errlist[errno]);
+    error("can't get floppy parameter of %s (%s)\n", dp->dev_name, sys_errlist[errno]);
     fatalerr = 5;
     return;
   }
@@ -740,14 +737,14 @@ disk_init(void)
         bootdisk.fdesc = open(bootdisk.dev_name, O_RDONLY, 0);
         leave_priv_setting();
         if (bootdisk.fdesc < 0) {
-          error("ERROR: can't open bootdisk %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
+          error("can't open bootdisk %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(23);
         } else {
           bootdisk.rdonly = 1;
           d_printf("(disk) can't open bootdisk %s for read/write. Readonly did work though\n", bootdisk.dev_name);
         }
       } else {
-        error("ERROR: can't open bootdisk %s: %sn", dp->dev_name, strerror(errno));
+        error("can't open bootdisk %s: %sn", dp->dev_name, strerror(errno));
         leavedos(23);
       }
     else bootdisk.rdonly = bootdisk.wantrdonly;
@@ -755,7 +752,7 @@ disk_init(void)
   }
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
     if (stat(dp->dev_name, &stbuf) < 0) {
-      error("ERROR: can't stat %s\n", dp->dev_name);
+      error("can't stat %s\n", dp->dev_name);
       leavedos(24);
     }
     if (S_ISBLK(stbuf.st_mode))
@@ -789,14 +786,14 @@ disk_init(void)
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
         leave_priv_setting();
         if (dp->fdesc < 0) {
-          error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
+          error("can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(25);
         } else {
           dp->rdonly = 1;
           d_printf("(disk) can't open %s for read/write. Readonly did work though\n", dp->dev_name);
         }
       } else {
-        error("ERROR: can't open %s: %s\n", dp->dev_name, strerror(errno));
+        error("can't open %s: %s\n", dp->dev_name, strerror(errno));
         leavedos(25);
       }
     else dp->rdonly = dp->wantrdonly;
@@ -821,14 +818,14 @@ disk_init(void)
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
         leave_priv_setting();
         if (dp->fdesc < 0) {
-          error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
+          error("can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(26);
         } else {
           dp->rdonly = 1;
           d_printf("(disk) can't open %s for read/write. Readonly did work though\n", dp->dev_name);
         }
       } else {
-        error("ERROR: can't open %s: #%d - %s\n", dp->dev_name, errno, strerror(errno));
+        error("can't open %s: #%d - %s\n", dp->dev_name, errno, strerror(errno));
         leavedos(26);
       }
     }
@@ -854,7 +851,7 @@ disk_init(void)
      */
 #ifdef SILLY_GET_GEOMETRY
     if (RPT_SYSCALL(read(dp->fdesc, buf, 512)) != 512) {
-      error("ERROR: can't read disk info of %s\n", dp->dev_name);
+      error("can't read disk info of %s\n", dp->dev_name);
       leavedos(27);
     }
 
@@ -885,7 +882,7 @@ disk_init(void)
 	     *(unsigned long *) &buf[39], label);
 
     if (s % (dp->sectors * dp->heads) != 0) {
-      error("ERROR: incorrect track number of %s\n", dp->dev_name);
+      error("incorrect track number of %s\n", dp->dev_name);
       /* leavedos(28); */
     }
 #endif
@@ -912,13 +909,10 @@ int13(u_char i)
 {
   unsigned int disk, head, sect, track, number;
   int res;
-#ifdef __linux__
   loff_t  pos;
-#else
-  off_t  pos;
-#endif
   char *buffer;
   struct disk *dp;
+  int checkdp_val;
 
   disk = LO(dx);
   if (disk < FDISKS) {
@@ -961,10 +955,16 @@ int13(u_char i)
   case 2:			/* read */
     FLUSHDISK(dp);
     disk_open(dp);
-    head = HI(dx);
+    checkdp_val = checkdp(dp);
+    if (!checkdp_val && dp->diskcyl4096 && dp->heads <= 64)
+      head = HI(dx) & 0x3f;
+    else
+      head = HI(dx);
     sect = (REG(ecx) & 0x3f) - 1;
     track = (HI(cx)) |
       ((REG(ecx) & 0xc0) << 2);
+    if (!checkdp_val && dp->diskcyl4096 && dp->heads <= 64 && (HI(dx) & 0xc0))
+      track |= (HI(dx) & 0xc0) << 4;
     buffer = SEG_ADR((char *), es, bx);
     number = LO(ax);
 #if 0
@@ -972,9 +972,9 @@ int13(u_char i)
 	     disk, head, sect, track, number, (void *) buffer);
 #endif
 
-    if (checkdp(dp) || head >= dp->heads ||
+    if (checkdp_val || head >= dp->heads ||
 	sect >= dp->sectors || track >= dp->tracks) {
-      error("ERROR: Sector not found 1!\n");
+      error("Sector not found 1!\n");
       d_printf("DISK %d read [h:%d,s:%d,t:%d](%d)->%p\n",
 	       disk, head, sect, track, number, (void *) buffer);
       if (dp) {
@@ -998,7 +998,7 @@ int13(u_char i)
       break;
     }
     else if (res & 511) {	/* must read multiple of 512 bytes */
-      error("ERROR: sector_corrupt 1, return = %d!\n", res);
+      error("sector_corrupt 1, return = %d!\n", res);
       HI(ax) = DERR_BADSEC;	/* sector corrrupt */
       CARRY;
       break;
@@ -1013,18 +1013,24 @@ int13(u_char i)
   case 3:			/* write */
     FLUSHDISK(dp);
     disk_open(dp);
-    head = HI(dx);
+    checkdp_val = checkdp(dp);
+    if (!checkdp_val && dp->diskcyl4096 && dp->heads <= 64)
+      head = HI(dx) & 0x3f;
+    else
+      head = HI(dx);
     sect = (REG(ecx) & 0x3f) - 1;
     track = (HI(cx)) |
       ((REG(ecx) & 0xc0) << 2);
+    if (!checkdp_val && dp->diskcyl4096 && dp->heads <= 64 && (HI(dx) & 0xc0))
+      track |= (HI(dx) & 0xc0) << 4;
     buffer = SEG_ADR((char *), es, bx);
     number = LO(ax);
     W_printf("DISK write [h:%d,s:%d,t:%d](%d)->%p\n",
 	     head, sect, track, number, (void *) buffer);
 
-    if (checkdp(dp) || head >= dp->heads ||
+    if (checkdp_val || head >= dp->heads ||
 	sect >= dp->sectors || track >= dp->tracks) {
-      error("ERROR: Sector not found 3!\n");
+      error("Sector not found 3!\n");
       show_regs(__FILE__, __LINE__);
       HI(ax) = DERR_NOTFOUND;
       REG(eflags) |= CF;
@@ -1032,7 +1038,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly) {
-      error("ERROR: write protect!\n");
+      error("write protect!\n");
       show_regs(__FILE__, __LINE__);
       if (dp->removeable)
 	HI(ax) = DERR_WP;
@@ -1053,7 +1059,7 @@ int13(u_char i)
       break;
     }
     else if (res & 511) {	/* must write multiple of 512 bytes */
-      error("ERROR: Write sector corrupt 2 (wrong size)!\n");
+      error("Write sector corrupt 2 (wrong size)!\n");
       HI(ax) = DERR_BADSEC;
       CARRY;
       break;
@@ -1080,7 +1086,7 @@ int13(u_char i)
 	sect >= dp->sectors || track >= dp->tracks) {
       HI(ax) = DERR_NOTFOUND;
       REG(eflags) |= CF;
-      error("ERROR: test: sector not found 5\n");
+      error("test: sector not found 5\n");
       dbug_printf("hds: %d, sec: %d, tks: %d\n",
 		  dp->heads, dp->sectors, dp->tracks);
       break;
@@ -1100,7 +1106,7 @@ int13(u_char i)
 #endif
       HI(ax) = DERR_NOTFOUND;
       REG(eflags) |= CF;
-      error("ERROR: test: sector not found 6\n");
+      error("test: sector not found 6\n");
       break;
     }
 #if 0
@@ -1108,7 +1114,7 @@ int13(u_char i)
     if (res & 0x1ff) {		/* must read multiple of 512 bytes  and res != -1 */
       HI(ax) = DERR_BADSEC;
       REG(eflags) |= CF;
-      error("ERROR: test: sector corrupt 3\n");
+      error("test: sector corrupt 3\n");
       break;
     }
     LWORD(eax) = res >> 9;
@@ -1247,7 +1253,7 @@ int13(u_char i)
 	NOCARRY;
       }
       else {
-	error("ERROR: gettype: no disk %d\n", disk);
+	error("gettype: no disk %d\n", disk);
 	HI(ax) = 0;		/* disk not there */
 	REG(eflags) |= CF;	/* error */
       }
@@ -1294,6 +1300,180 @@ int13(u_char i)
   case 0x28:			/* DRDOS 6.0 call ??? */
     d_printf("int 13h, ax=%04x...DRDOS call\n", LWORD(eax));
     break;
+
+  case 0x41:                    /* IBM/MS Extensions, install check */
+    LWORD(ebx) = IMEXT_MAGIC;
+    HI(ax) = IMEXT_VER_MAJOR;
+    LWORD(ecx) = IMEXT_API_SUPPORT_BITS;
+    NOCARRY;
+    break;
+
+  case 0x42: {                  /* IBM/MS Extensions, read */
+    struct ibm_ms_diskaddr_pkt *diskaddr;
+
+    FLUSHDISK(dp);
+    disk_open(dp);
+    diskaddr = SEG_ADR((struct ibm_ms_diskaddr_pkt *), ds, si);
+    sect = diskaddr->block_lo % dp->sectors;
+    head = (diskaddr->block_lo / dp->sectors) % dp->heads;
+    track = diskaddr->block_lo / (dp->heads * dp->sectors);
+    buffer = (char *)(((unsigned long)diskaddr->buf_seg << 4)+diskaddr->buf_ofs);
+    number = diskaddr->blocks;
+    diskaddr->blocks = 0;
+
+    if (checkdp(dp) || track >= dp->tracks) {
+      error("ERROR: Sector not found, AH=0x42!\n");
+      d_printf("DISK %d ext read [h:%d,s:%d,t:%d](%d)->%p\n",
+	       disk, head, sect, track, number, (void *) buffer);
+      if (dp) {
+	  d_printf("DISK dev %s GEOM %d heads %d sects %d trk\n",
+		   dp->dev_name, dp->heads, dp->sectors, dp->tracks);
+      } else {
+	  d_printf("DISK %x undefined.\n", disk);
+      }
+
+      HI(ax) = DERR_NOTFOUND;
+      REG(eflags) |= CF;
+      show_regs(__FILE__, __LINE__);
+      break;
+    }
+
+    res = read_sectors(dp, buffer, head, sect, track, number);
+
+    if (res < 0) {
+      HI(ax) = -res;
+      CARRY;
+      break;
+    }
+    else if (res & 511) {	/* must read multiple of 512 bytes */
+      error("ERROR: sector_corrupt 1, return = %d!\n", res);
+      HI(ax) = DERR_BADSEC;	/* sector corrrupt */
+      CARRY;
+      break;
+    }
+
+    diskaddr->blocks = res >> 9;
+    HI(ax) = 0;
+    REG(eflags) &= ~CF;
+    R_printf("DISK ext read @%d/%d/%d (%d) -> %p OK.\n",
+	     head, track, sect, res >> 9, (void *)buffer);
+    break;
+  }
+  
+  case 0x43: {                  /* IBM/MS Extensions, write */
+    struct ibm_ms_diskaddr_pkt *diskaddr;
+
+    FLUSHDISK(dp);
+    disk_open(dp);
+    diskaddr = SEG_ADR((struct ibm_ms_diskaddr_pkt *), ds, si);
+    sect = diskaddr->block_lo % dp->sectors;
+    head = (diskaddr->block_lo / dp->sectors) % dp->heads;
+    track = diskaddr->block_lo / (dp->heads * dp->sectors);
+    buffer = (char *)(((unsigned long)diskaddr->buf_seg << 4)+diskaddr->buf_ofs);
+    number = diskaddr->blocks;
+    diskaddr->blocks = 0;
+
+    if (checkdp(dp) || track >= dp->tracks) {
+      error("ERROR: Sector not found, AH=0x42!\n");
+      d_printf("DISK %d ext write [h:%d,s:%d,t:%d](%d)->%p\n",
+	       disk, head, sect, track, number, (void *) buffer);
+      if (dp) {
+	  d_printf("DISK dev %s GEOM %d heads %d sects %d trk\n",
+		   dp->dev_name, dp->heads, dp->sectors, dp->tracks);
+      } else {
+	  d_printf("DISK %x undefined.\n", disk);
+      }
+
+      HI(ax) = DERR_NOTFOUND;
+      REG(eflags) |= CF;
+      show_regs(__FILE__, __LINE__);
+      break;
+    }
+
+    if (dp->rdonly) {
+      error("ERROR: write protect!\n");
+      show_regs(__FILE__, __LINE__);
+      if (dp->removeable)
+	HI(ax) = DERR_WP;
+      else
+	HI(ax) = DERR_WRITEFLT;
+      REG(eflags) |= CF;
+      break;
+    }
+
+    if (dp->rdonly)
+      error("CONTINUED!!!!!\n");
+    res = write_sectors(dp, buffer, head, sect, track, number);
+
+    if (res < 0) {
+      HI(ax) = -res;
+      CARRY;
+      break;
+    }
+    else if (res & 511) {	/* must read multiple of 512 bytes */
+      error("ERROR: sector_corrupt 1, return = %d!\n", res);
+      HI(ax) = DERR_BADSEC;	/* sector corrrupt */
+      CARRY;
+      break;
+    }
+
+    diskaddr->blocks = res >> 9;
+    HI(ax) = 0;
+    REG(eflags) &= ~CF;
+    R_printf("DISK ext write @%d/%d/%d (%d) -> %p OK.\n",
+	     head, track, sect, res >> 9, (void *)buffer);
+    break;
+  }
+
+  case 0x44:                    /* IBM/MS Extensions, verify */
+    /* Always succeeds. Should perhaps check validity of sector address. */
+    NOCARRY;
+    HI(ax) = 0;
+    break;
+
+  case 0x47:                    /* IBM/MS Extensions, extended seek */
+    NOCARRY;
+    HI(ax) = 0;
+    break;
+
+  case 0x48: {                  /* IBM/MS Extensions, get drive parameters */
+    struct ibm_ms_drive_params *params;
+
+    FLUSHDISK(dp);
+    disk_open(dp);
+    params = SEG_ADR((struct ibm_ms_drive_params *), ds, si);
+
+    if (checkdp(dp)) {
+      error("ERROR: Invalid drive, AH=0x48!\n");
+      if (dp) {
+	  d_printf("DISK dev %s GEOM %d heads %d sects %d trk\n",
+		   dp->dev_name, dp->heads, dp->sectors, dp->tracks);
+      } else {
+	  d_printf("DISK %x undefined.\n", disk);
+      }
+
+      HI(ax) = DERR_NOTFOUND;
+      REG(eflags) |= CF;
+      show_regs(__FILE__, __LINE__);
+      break;
+    }
+
+    params->flags = IMEXT_INFOFLAG_CHSVALID;
+    if (dp->removeable)
+      params->flags |= IMEXT_INFOFLAG_REMOVABLE;
+    params->tracks = dp->tracks;
+    params->heads = dp->heads;
+    params->sectors = dp->sectors;
+    params->total_sectors_lo = dp->tracks*dp->heads*dp->sectors;
+    params->total_sectors_hi = 0;
+    params->bytes_per_sector = SECTOR_SIZE;
+    if (params->len >= 0x1e)
+      params->edd_cfg_ofs = params->edd_cfg_seg = 0xffff;
+    NOCARRY;
+    HI(ax) = 0;
+    break;
+  }
+
   case 0x5:			/* format */
     NOCARRY;			/* successful */
     HI(ax) = DERR_NOERR;
@@ -1310,7 +1490,7 @@ int13(u_char i)
     NOCARRY;
     break;
   default:
-    error("ERROR: disk error, unknown command: int13, ax=0x%x\n",
+    error("disk error, unknown command: int13, ax=0x%x\n",
 	  LWORD(eax));
     show_regs(__FILE__, __LINE__);
     CARRY;
@@ -1333,3 +1513,49 @@ floppy_tick(void)
     secs = 0;
   }
 }
+
+#ifdef USE_THREADS
+
+#include "lt-threads.h"
+
+static mbox_handle ioctlmbox_out;
+static mbox_handle ioctlmbox_in;
+
+struct {
+  struct msg msg;
+  int fd;
+  int request;
+  void *param;
+  int result;
+} ioctlmsg;
+
+void background_ioctl_thread(int start)
+{
+  if (!start) {
+    /* we come here _before_ any thread is started
+     * but threading is active and we run at scope of thread0.
+     * We now create the resoeurce we need.
+     */
+     ioctlmbox_in = create_local_mailbox("diskioctl_in", 2);
+     ioctlmbox_out = create_local_mailbox("diskioctl_out", 2);
+     return;
+  }
+  /* here we are a separate thread */
+  while (1) {
+    receivemessage(ioctlmbox_in);
+    ioctlmsg.result = ioctl(ioctlmsg.fd, ioctlmsg.request, ioctlmsg.param);
+    sendmessage(ioctlmbox_out,0);
+  }
+}
+
+static int background_ioctl(int fd, int request, void *param)
+{
+  ioctlmsg.fd = fd;
+  ioctlmsg.request = request;
+  ioctlmsg.param = param;
+  sendmessage(ioctlmbox_in,0);
+  receivemessage(ioctlmbox_out);
+  return ioctlmsg.result;
+}
+
+#endif
