@@ -6,11 +6,6 @@
  *
  *
  */
-#define DBGTIME(x) {\
-                        struct timeval tv;\
-                        gettimeofday(&tv,NULL);\
-                        fprintf(stderr,"%c %06d:%06d\n",x,(int)tv.tv_sec,(int)tv.tv_usec);\
-                   }
 #include <features.h>
 #include <sys/types.h>
 #include <sys/ipc.h>
@@ -65,7 +60,7 @@ extern void DOSEMUMouseEvents(void);
 extern void xms_init(void);
 extern void video_memory_setup(void);
 extern void dump_kbuffer(void);
-extern int_count[];
+extern int int_count[];
 extern int in_readkeyboard, keybint;
 
 #define PAGE_SIZE	4096
