@@ -38,6 +38,7 @@
 #include <stdlib.h>   /* exit                   */
 #include <string.h>
 #include "emulib.h"
+#include "doshelpers.h"
 
 typedef unsigned char uint8;
 typedef unsigned int uint16;
@@ -47,19 +48,6 @@ typedef unsigned int uint16;
 
 #define MAX_DEBUG_STRING_LENGTH   100
 
-#define DOS_HELPER_INT            0xE6
-#define DOS_HELPER_DOSEMU_CHECK   0x00
-#define DOS_HELPER_SHOW_REGS      0x01
-#define DOS_HELPER_SHOW_INTS      0x02
-#define DOS_HELPER_ADJUST_IOPERMS 0x03  /* CY indicates get or set      */
-#define DOS_HELPER_CONTROL_VIDEO  0x04  /* BL indicates init or release */
-#define DOS_HELPER_SHOW_BANNER    0x05
-#define DOS_HELPER_MFS_HELPER     0x20
-#define DOS_HELPER_EMS_HELPER     0x21
-#define DOS_HELPER_EMS_BIOS       0x22
-#define DOS_HELPER_GET_DEBUG_STRING 0x10
-#define DOS_HELPER_SET_DEBUG_STRING 0x11
-
 
 void
 Usage(void)
@@ -81,7 +69,7 @@ i  port I/O     s  serial       m  mouse        #  interrupt\n\
 p  printer      g  general      c  config       w  warnings\n\
 h  hardware     I  IPC          E  EMS          x  XMS\n\
 M  DPMI         n  network      P  pktdrv       r  PIC\n\
-S  sound\n");
+S  sound        T  I/O-trace    e  cpu-emu\n");
 
 #else
 
@@ -90,7 +78,8 @@ C  cdrom        v  video        k  keyboard     i  port I/O\n\
 s  serial       m  mouse        #  interrupt    p  printer\n\
 g  general      c  config       w  warnings     h  hardware\n\
 I  IPC          E  EMS          x  XMS          M  DPMI\n\
-n  network      P  pktdrv       r  PIC          S  sound\n");
+n  network      P  pktdrv       r  PIC          S  sound\n\
+T  I/O-trace    e  cpu-emu\n");
 
 #endif
 
@@ -167,6 +156,10 @@ printDebugClass(char class, char value)
               printf("i  port I/O   ");
               break;
 
+         case 'T':
+              printf("T  I/O-trace  ");
+              break;
+
          case 's':
               printf("s  serial     ");
               break;
@@ -231,6 +224,10 @@ printDebugClass(char class, char value)
               printf("S  sound      ");
               break;
 
+         case 'e':
+              printf("e  cpu-emu    ");
+              break;
+
          default:
               printf("%c  unknown    ", class);
               break;
@@ -316,9 +313,9 @@ uint16 ParseAndSetDebugString(char *userDebugStr)
     char class, value;
 
 #ifdef X_SUPPORT
-    const char debugAll[] = "-d-R-W-D-C-v-X-k-i-s-m-#-p-g-c-w-h-I-E-x-M-n-P-r-S";
+    const char debugAll[] = "-d-R-W-D-C-v-X-k-i-T-s-m-#-p-g-c-w-h-I-E-x-M-n-P-r-S-e";
 #else
-    const char debugAll[] = "-d-R-W-D-C-v-k-i-s-m-#-p-g-c-w-h-I-E-x-M-n-P-r-S";
+    const char debugAll[] = "-d-R-W-D-C-v-k-i-T-s-m-#-p-g-c-w-h-I-E-x-M-n-P-r-S-e";
 #endif
 
     //expand the user string to a canonical form
