@@ -1,3 +1,6 @@
+#ifndef MEMINFO_H
+#define MEMINFO_H
+
 struct meminfo {
   int meminfofd;
   int total;
@@ -16,3 +19,5 @@ struct meminfo {
 };
 
 struct meminfo *readMeminfo(void);
+
+#endif
