@@ -1,12 +1,14 @@
 /*
  * DANG_BEGIN_MODULE
  *
+ * REMARK
  * This is the file redirector code for DOSEMU. It was built on the Mach
  * DOS redirector and as such continues that copyright as well in 
  * addition the GNU copyright. This redirector uses the 
  * DOS int2f fnx 11 calls to give running DOS programs access to any 
  * Unix mounted drives that permissions exist for.
  *
+ * /REMARK
  * DANG_END_MODULE
  * DANG_FIXTHIS We probably should use llseek here for file > 2 GBytes
  *
@@ -750,11 +752,11 @@ init_drive(int dd, char *path, char *options)
     dos_root_len = 1;
     dos_root = "/";
     if (!find_file(dos_roots[dd], &st)) {
-      error("MFS ERROR: couldn't find root path %s\n", dos_roots[dd]);
+      error("MFS: couldn't find root path %s\n", dos_roots[dd]);
       return (0);
     }
     if (!(st.st_mode & S_IFDIR)) {
-      error("MFS ERROR: root path is not a directory %s\n", dos_roots[dd]);
+      error("MFS: root path is not a directory %s\n", dos_roots[dd]);
       return (0);
     }
     dos_roots[dd][dos_root_lens[dd] - 1] = '/';
@@ -944,7 +946,7 @@ make_entry(void)
 /* DANG_BEGIN_REMARK
  * The msdos_dir_ent structure has much more than 28 bytes. 
  * Is this significant?
- * DANG_END_FUNCTION
+ * DANG_END_REMARK
  */
   entry = (struct dir_ent *) malloc(sizeof(struct dir_ent)+10);
   entry->hidden = FALSE;
@@ -1007,7 +1009,7 @@ _get_dir(char *name, char *mname, char *mext)
  * Added compares to NUL so that newer versions of Foxpro which test directories
  * using xx\yy\nul perform closer to whats DOS does.
  * 
- * DANG_END_FUNCTION
+ * DANG_END_REMARK
  */
   if (strncasecmp(mname, "NUL     ", strlen(mname)) == 0 &&
       strncasecmp(mext, "   ", strlen(mext)) == 0) {
