@@ -26,6 +26,7 @@
 /*
  * DANG_BEGIN_MODULE
  *
+ * REMARK
  * This provides the EMM Memory Management for DOSEMU. It was originally part
  * of the Mach Dos Emulator. 
  *
@@ -40,6 +41,7 @@
  * support is missing, and raw page size is 16k (instead of 4k).  Other than
  * that, EMS 4.0 support appears complete.
  *
+ * /REMARK
  * DANG_END_MODULE
  *
  * Purpose:
@@ -775,7 +777,7 @@ static inline boolean_t
 remap_page(physical_page)
      int physical_page;
 {
-  E_printf("EMS: remaping physical page 0x%01x\n", physical_page);
+  E_printf("EMS: remapping physical page 0x%01x\n", physical_page);
 
   return __map_page(physical_page);
 }
