@@ -48,7 +48,7 @@ far_t ESRPopRegistersIRet;
 far_t ESRFarCall;
 
 static ipx_socket_t *ipx_socket_list = NULL;
-static IPXRunning = 0;
+static int IPXRunning = 0;
 
 /* DANG_FIXTHIS - get a real value for my address !! */
 static unsigned char MyAddress[10] =
@@ -1068,7 +1068,7 @@ IPXRelinquishControl(void)
     break;
 
   case -1:			/* error (not EINTR) */
-    error("ERROR: bad ipc_select: %s\n", strerror(errno));
+    error("bad ipc_select: %s\n", strerror(errno));
     break;
 
   default:			/* has at least 1 descriptor ready */
