@@ -21,13 +21,10 @@
 #include <sys/times.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/times.h>
 #include <limits.h>
 #ifndef __NetBSD__
 #include <getopt.h>
 #endif
-#include <assert.h>
-
 
 #if X_GRAPHICS
 #include <sys/mman.h>           /* root@sjoerd*/
@@ -87,10 +84,6 @@
 
 #include "dma.h"
 
-#ifdef bon
-extern void     vm86_GP_fault();
-#endif
-
 /*  */
 /* vm86_GP_fault @@@  32768 MOVED_CODE_BEGIN @@@ 01/23/96, ./src/arch/linux/async/sigsegv.c --> src/emu-i386/do_vm86.c  */
 /*
@@ -109,13 +102,14 @@ void vm86_GP_fault(void)
   unsigned char *csp, *lina;
   Bit32u org_eip;
   int pref_seg;
-  int done,is_rep,is_32bit;
+  int done,is_rep,prefix66,prefix67;
 
 #if 0
   u_short *ssp;
   static int haltcount = 0;
 #endif
 
+#define LWECX	    (prefix66 ^ prefix67 ? REG(ecx) : LWORD(ecx))
 #define MAX_HALT_COUNT 3
 
 #if 0
@@ -131,12 +125,12 @@ void vm86_GP_fault(void)
 #endif
 
   if (ignore_segv) {
-    error("ERROR: sigsegv ignored!\n");
+    error("sigsegv ignored!\n");
     return;
   }
 
   if (in_sigsegv)
-    error("ERROR: in_sigsegv=%d!\n", in_sigsegv);
+    error("in_sigsegv=%d!\n", in_sigsegv);
 
   /* in_vm86 = 0; */
   in_sigsegv++;
@@ -168,7 +162,7 @@ void vm86_GP_fault(void)
 
   /* DANG_BEGIN_REMARK
    * Here we handle all prefixes prior switching to the appropriate routines
-   * The exception CS:EIP will point to the first prefix that effects the
+   * The exception CS:EIP will point to the first prefix that effects
    * the faulting instruction, hence, 0x65 0x66 is same as 0x66 0x65.
    * So we collect all prefixes and remember them.
    * - Hans Lermen
@@ -178,12 +172,13 @@ void vm86_GP_fault(void)
   #define __SEG_ADR(type, seg, reg)  type((seg << 4) + LWORD(e##reg))
   done=0;
   is_rep=0;
-  is_32bit=0;
+  prefix66=prefix67=0;
   pref_seg=-1;
 
   do {
     switch (*(csp++)) {
-       case 0x66:      /* operand prefix */  is_32bit=1; break;
+       case 0x66:      /* operand prefix */  prefix66=1; break;
+       case 0x67:      /* address prefix */  prefix67=1; break;
        case 0x2e:      /* CS */              pref_seg=REG(cs); break;
        case 0x3e:      /* DS */              pref_seg=REG(ds); break;
        case 0x26:      /* ES */              pref_seg=REG(es); break;
@@ -204,30 +199,50 @@ void vm86_GP_fault(void)
   switch (*csp) {
 
   case 0x6c:                    /* insb */
-    /* NOTE: ES can't be overwritten */
+    /* NOTE: ES can't be overwritten; prefixes 66,67 should use esi,edi,ecx
+     * but is anyone using extended regs in real mode? */
+#ifdef NEW_PORT_CODE
+    /* WARNING: no test for DI wrapping! */
+    LWORD(edi) += port_rep_inb(LWORD(edx), SEG_ADR((Bit8u *),es,di),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    if (is_rep) LWECX = 0;
+#else
     if (is_rep) {
       int delta = 1;
       if(LWORD(eflags) &DF ) delta = -1;
       i_printf("Doing REP F3 6C (rep insb) %04x bytes, DELTA %d\n",
               LWORD(ecx),delta);
       while (LWORD(ecx))  {
-        *(SEG_ADR((unsigned char *),es,di)) = inb((int) LWORD(edx));
+        *(SEG_ADR((Bit8u *),es,di)) = inb((int) LWORD(edx));
         LWORD(edi) += delta;
         LWORD(ecx)--;
       }
     }
     else {
-      *(SEG_ADR((unsigned char *),es,di)) = inb((int) LWORD(edx));
+      *(SEG_ADR((Bit8u *),es,di)) = inb((int) LWORD(edx));
       i_printf("insb(0x%04x) value %02x\n",
-              LWORD(edx),*(SEG_ADR((unsigned char *),es,di)));   
+              LWORD(edx),*(SEG_ADR((Bit8u *),es,di)));   
       if(LWORD(eflags) & DF) LWORD(edi)--;
       else LWORD(edi)++;
     }
+#endif	/* NEW_PORT_CODE */
     LWORD(eip)++;
     break;
 
   case 0x6d:			/* (rep) insw / insd */
     /* NOTE: ES can't be overwritten */
+#ifdef NEW_PORT_CODE
+    /* WARNING: no test for _DI wrapping! */
+    if (prefix66) {
+      LWORD(edi) += port_rep_ind(LWORD(edx), SEG_ADR((Bit32u *),es,di),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    }
+    else {
+      LWORD(edi) += port_rep_inw(LWORD(edx), SEG_ADR((Bit16u *),es,di),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    }
+    if (is_rep) LWECX = 0;
+#else
     if (is_rep) {
       #define ___LOCALAUX(typ,iotyp,incr,x) \
         int delta =incr; \
@@ -240,13 +255,13 @@ void vm86_GP_fault(void)
           LWORD(ecx)--; \
         }
 
-      if (is_32bit) {
+      if (prefix66) {
                                 /* rep insd */
-        ___LOCALAUX((unsigned long *),ind,4,'d')
+        ___LOCALAUX((Bit32u *),ind,4,'d')
       }
       else {
                                 /* rep insw */
-        ___LOCALAUX((unsigned short *),inw,2,'w')
+        ___LOCALAUX((Bit16u *),inw,2,'w')
       }
       #undef  ___LOCALAUX
     }
@@ -258,45 +273,65 @@ void vm86_GP_fault(void)
         if(LWORD(eflags) & DF) LWORD(edi) -= incr; \
         else LWORD(edi) +=incr;
       
-      if (is_32bit) {
+      if (prefix66) {
                                 /* insd */
-        ___LOCALAUX((unsigned long *),ind,4,'d')
+        ___LOCALAUX((Bit32u *),ind,4,'d')
       }
       else {
                                 /* insw */
-        ___LOCALAUX((unsigned short *),inw,2,'w')
+        ___LOCALAUX((Bit16u *),inw,2,'w')
       }
       #undef  ___LOCALAUX
     }
+#endif	/* NEW_PORT_CODE */
     LWORD(eip)++;
     break;
       
 
   case 0x6e:			/* (rep) outsb */
     if (pref_seg < 0) pref_seg = LWORD(ds);
+#ifdef NEW_PORT_CODE
+    /* WARNING: no test for _SI wrapping! */
+    LWORD(esi) += port_rep_outb(LWORD(edx), __SEG_ADR((Bit8u *),pref_seg,si),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    if (is_rep) LWECX = 0;
+#else
     if (is_rep) {
       int delta = 1;
       i_printf("untested: rep outsb\n");
       if(LWORD(eflags) & DF) delta = -1;
       while(LWORD(ecx)) {
-        outb(LWORD(edx), *(__SEG_ADR((unsigned char *),pref_seg,si)));
+        outb(LWORD(edx), *(__SEG_ADR((Bit8u *),pref_seg,si)));
         LWORD(esi) += delta;
         LWORD(ecx)--;
       }
     }
     else {
       i_printf("untested: outsb port 0x%04x value %02x\n",
-              LWORD(edx),*(__SEG_ADR((unsigned char *),pref_seg,si)));
-      outb(LWORD(edx), *(__SEG_ADR((unsigned char *),pref_seg,si)));
+              LWORD(edx),*(__SEG_ADR((Bit8u *),pref_seg,si)));
+      outb(LWORD(edx), *(__SEG_ADR((Bit8u *),pref_seg,si)));
       if(LWORD(eflags) & DF) LWORD(esi)--;
       else LWORD(esi)++;
     }
+#endif	/* NEW_PORT_CODE */
     LWORD(eip)++;
     break;
 
 
   case 0x6f:			/* (rep) outsw / outsd */
     if (pref_seg < 0) pref_seg = LWORD(ds);
+#ifdef NEW_PORT_CODE
+    /* WARNING: no test for _SI wrapping! */
+    if (prefix66) {
+      LWORD(esi) += port_rep_outd(LWORD(edx), __SEG_ADR((Bit32u *),pref_seg,si),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    }
+    else {
+      LWORD(esi) += port_rep_outw(LWORD(edx), __SEG_ADR((Bit16u *),pref_seg,si),
+	LWORD(eflags)&DF, (is_rep? LWECX:1));
+    } 
+    if (is_rep) LWECX = 0;
+#else
     if (is_rep) {
       #define ___LOCALAUX(typ,iotyp,incr,x) \
         int delta = incr; \
@@ -308,13 +343,13 @@ void vm86_GP_fault(void)
           LWORD(ecx)--; \
         }
       
-      if (is_32bit) {
+      if (prefix66) {
                                 /* rep outsd */
-        ___LOCALAUX((unsigned long *),outd,4,'d')
+        ___LOCALAUX((Bit32u *),outd,4,'d')
       }
       else {
                                 /* rep outsw */
-        ___LOCALAUX((unsigned short *),outw,2,'w')
+        ___LOCALAUX((Bit16u *),outw,2,'w')
       }
       #undef  ___LOCALAUX
     }
@@ -326,21 +361,22 @@ void vm86_GP_fault(void)
         if(LWORD(eflags) & DF ) LWORD(esi) -= incr; \
         else LWORD(esi) +=incr;
       
-      if (is_32bit) {
+      if (prefix66) {
                                 /* outsd */
-        ___LOCALAUX((unsigned long *),outd,4,'d')
+        ___LOCALAUX((Bit32u *),outd,4,'d')
       }
       else {
                                 /* outsw */
-        ___LOCALAUX((unsigned short *),outw,2,'w')
+        ___LOCALAUX((Bit16u *),outw,2,'w')
       }
       #undef  ___LOCALAUX
     } 
+#endif	/* NEW_PORT_CODE */
     LWORD(eip)++;
     break;
 
   case 0xe5:			/* inw xx, ind xx */
-    if (is_32bit) REG(eax) = ind((int) csp[1]);
+    if (prefix66) REG(eax) = ind((int) csp[1]);
     else LWORD(eax) = inw((int) csp[1]);
     LWORD(eip) += 2;
     break;
@@ -351,7 +387,7 @@ void vm86_GP_fault(void)
     break;
 
   case 0xed:			/* inw dx, ind dx */
-    if (is_32bit) REG(eax) = ind(LWORD(edx));
+    if (prefix66) REG(eax) = ind(LWORD(edx));
     else LWORD(eax) = inw(LWORD(edx));
     LWORD(eip) += 1;
     break;
@@ -362,7 +398,7 @@ void vm86_GP_fault(void)
     break;
 
   case 0xe7:			/* outw xx */
-    if (is_32bit) outd((int)csp[1], REG(eax));
+    if (prefix66) outd((int)csp[1], REG(eax));
     else outw((int)csp[1], LWORD(eax));
     LWORD(eip) += 2;
     break;
@@ -372,7 +408,7 @@ void vm86_GP_fault(void)
     break;
 
   case 0xef:			/* outw dx */
-    if (is_32bit) outd(REG(edx), REG(eax));
+    if (prefix66) outd(REG(edx), REG(eax));
     else outw(REG(edx), REG(eax));
     LWORD(eip) += 1;
     break;
@@ -418,29 +454,16 @@ void vm86_GP_fault(void)
     }
     break;
 
-  case 0x0f: /* RDE hack */
-    if (csp[1] == 0x06)
-      /*
-	 CLTS - ignore
-      */
-      { LWORD(eip)+=2;
-	break;
-      }
-      
-   if (csp[1] == 0x20)
-      /*
-    mov ???,cr?? - ignore
-      */
-      { if (csp[2] == 0xC0) REG(eax)=0;  /* mov eax,cr0 */
-        LWORD(eip)+=3;
-   break;
-      }
+  case 0x0f: /* was: RDE hack, now handled in cpu.c */
+    if (!cpu_trap_0f(csp, NULL)) goto op0ferr;
+    break;
 
   case 0xf0:			/* lock */
   default:
     if (is_rep) fprintf(stderr, "Nope REP F3,CSP = 0x%04x\n", csp[0]);
     /* er, why don't we advance eip here, and
 	 why does it work??  */
+op0ferr:
     REG(eip) = org_eip;
 #ifdef USE_MHPDBG
     mhp_debug(DBG_GPF, 0, 0);
@@ -448,7 +471,7 @@ void vm86_GP_fault(void)
     error("general protection at %p: %x\n", lina,*lina);
     show_regs(__FILE__, __LINE__);
     show_ints(0, 0x33);
-    error("ERROR: SIGSEGV, protected insn...exiting!\n");
+    error("SIGSEGV, protected insn...exiting!\n");
     fatalerr = 4;
     leavedos(fatalerr);		/* shouldn't return */
   }				/* end of switch() */
@@ -577,6 +600,11 @@ run_vm86(void)
   PFLAG(VIP);
   k_printf(" IOPL: %u\n", (unsigned) ((vflags & IOPL_MASK) >> 12));
 #endif
+    /* This will protect us from Mr.Norton's bugs */
+    if (_EFLAGS & (AC|ID)) {
+      g_printf("BUG: flags changed to %08x\n",_EFLAGS);
+      _EFLAGS &= ~(AC|ID);
+    }
 
     in_vm86 = 0;
     switch VM86_TYPE
