@@ -30,6 +30,9 @@
 #ifdef __NetBSD__
 #include <errno.h>
 #endif
+#ifdef __linux__
+#include <asm/bitops.h>
+#endif
 #if X_GRAPHICS
 #include <sys/mman.h>           /* root@sjoerd*/
 #endif
@@ -77,6 +80,7 @@
 #include "keyboard.h"
 
 #include "dma.h"
+#include "cpu-emu.h"
 
 extern Bit8u spkr_io_read(Bit32u port);
 extern void spkr_io_write(Bit32u port, Bit8u value);
@@ -101,11 +105,6 @@ p2bin (unsigned char c)
   return s+3;
 }
 
-#ifndef USE_MRP_JOYSTICK
-static int fake201 = 50;	/* for diagnostic loops, do not run games
-				   with THIS on */
-#endif
-
 /* ====================================================================== */
 /*  */
 /* inb,inw,ind,outb,outw,outd @@@  32768 MOVED_CODE_BEGIN @@@ 01/23/96, ./src/arch/linux/async/sigsegv.c --> src/emu-i386/ports.c  */
@@ -176,6 +175,9 @@ inb(unsigned int port)
             ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
               ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
               ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
+#ifdef NEW_X_CODE
+              ((port>=0x3d4) && (port<=0x3d5)) || /* sw: crt controller */
+#endif
               ((port>=0x3D0) && (port<=0x3DD)) ) )
     {
         r=VGA_emulate_inb((u_int)port);
@@ -253,7 +255,7 @@ inb(unsigned int port)
     r = pit_inp((u_int)port);
     return r;
   case 0x43:
-    r = port_inb((u_int)port);
+    r = inb((u_int)port);
     return r;
   case 0x3ba:
   case 0x3da:
@@ -304,6 +306,10 @@ inb(unsigned int port)
     }
     /* It seems that we might need 388, but this is write-only, at least in the
        older chip... */
+    if ((port & ~3) == 0x388) {
+      r=adlib_io_read(port);
+      return r;
+    }
     /* MPU401 */
     if ((port & ~1) == config.mpu401_base) {
 	r=mpu401_io_read(port);
@@ -376,7 +382,16 @@ inw(int port)
 unsigned int
 ind(int port)
 {
-  int v=read_port_w(port) & 0xffff;
+  int v;
+  if (config.pci && (port >= 0xcf8) && (port < 0xd00)) {
+    enter_priv_on();
+    iopl(3);
+    v=port_real_ind(port);
+    iopl(0);
+    leave_priv_setting();
+    return v;
+  }
+  v=read_port_w(port) & 0xffff;
   return (read_port_w(port+2)<< 16) | v;
 }
 
@@ -457,14 +472,23 @@ outb(unsigned int port, unsigned int byte)
       port_out(byte, port);
       set_ioperm(port, 1, 0);
     }
+#ifndef NEW_X_CODE
     return;
   }
   else {
+#else
+  }
+  /* the above stuff should someday be moved to crtcemu.c -- sw */
+  {
+#endif
 #if X_GRAPHICS
     if ( (config.X) &&
          ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
            ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
            ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
+#ifdef NEW_X_CODE
+           ((port>=0x3d4) && (port<=0x3d5)) || /* sw: crt controller */
+#endif
            ((port>=0x3D0) && (port<=0x3DD)) ) )
       {
         VGA_emulate_outb(port, (unsigned char)byte);
@@ -619,7 +643,7 @@ outb(unsigned int port, unsigned int byte)
       sb_io_write(port, byte);
       break;
     }
-    else if ((port & ~3) == 388) {
+    else if ((port & ~3) == 0x388) {
       adlib_io_write(port, byte);
       break;
     }
@@ -652,10 +676,10 @@ outw(unsigned int port, unsigned int value)
   if ((config.chipset == ATI) && isATIport(port) && (port & 0xfc00)) {
     enter_priv_on();
     iopl(3);
-    port_out(value, port);
+    port_out_w(value, port);
     iopl(0);
     leave_priv_setting();
-    v_printf("ATI outb [0x%04x] = 0x%02x\n", port, value);
+    v_printf("ATI outw [0x%04x] = 0x%04x\n", port, value);
     return;
   }
   if(!write_port_w(value,port) ) {
@@ -669,12 +693,102 @@ void
 outd(unsigned int port, unsigned int value)
 {
   port &= 0xffff;
+  if (config.pci && (port >= 0xcf8) && (port < 0xd00)) {
+    enter_priv_on();
+    iopl(3);
+    port_real_outd(port, value);
+    iopl(0);
+    leave_priv_setting();
+    return;
+  }
   outw(port,value & 0xffff);
   outw(port+2,(unsigned int)value >> 16);
 }
 /* @@@ MOVE_END @@@ 32768 */
 
 
+/* The following functions were previously in portss.c */
+
+Bit8u port_safe_inb(Bit32u port)
+{
+  Bit8u res = 0;
+
+  i_printf("PORT: safe_inb ");
+  if (i_am_root) {
+    enter_priv_on();
+    iopl(3);
+    res = port_in(port);
+    iopl(0);
+
+    leave_priv_setting();
+  }
+  else
+    i_printf("want to ");
+  i_printf("in(%lx)", port);
+  if (i_am_root)
+    i_printf(" = %hx", res);
+  i_printf("\n");
+  return res;
+}
+
+void port_safe_outb(Bit32u port, Bit8u byte)
+{
+  i_printf("PORT: safe_outb ");
+  if (i_am_root) {
+    enter_priv_on();
+    iopl(3);
+    port_out(byte, port);
+    iopl(0);
+    leave_priv_setting();
+  }
+  else
+    i_printf("want to ");
+  i_printf("outb(%lx, %hx)\n", port, byte);
+}
+
+Bit16u port_safe_inw(Bit32u port)
+{
+  Bit16u res = 0;
+
+  i_printf("PORT: safe_inw ");
+  if (i_am_root) {
+    enter_priv_on();
+    iopl(3);
+    res = port_in_w(port);
+    iopl(0);
+    leave_priv_setting();
+  }
+  else
+    i_printf("want to ");
+  i_printf("inw(%lx)", port);
+  if (i_am_root)
+    i_printf(" = %x", res);
+  i_printf("\n");
+  return res;
+}
+
+void port_safe_outw(Bit32u port, Bit16u word)
+{
+  i_printf("PORT: safe_outw ");
+  if (i_am_root) {
+    enter_priv_on();
+    iopl(3);
+    port_out_w(word, port);
+    iopl(0);
+    leave_priv_setting();
+  }
+  else
+    i_printf("want to ");
+  i_printf("outw(%lx, %x)\n", port, word);
+}
+
+
+int port_init(void)
+{
+  /* this function intentionally left blank */
+  return 0;
+}
+
 
 /*  */
 /* set_ioperm @@@  40960 MOVED_CODE_BEGIN @@@ 01/23/96, ./src/emu.c --> src/emu-i386/ports.c  */
@@ -687,6 +801,14 @@ set_ioperm(int start, int size, int flag)
     if (!i_am_root)
 	return -1;		/* don't bother */
 
+#ifdef X86_EMULATOR
+    if (config.cpuemu) {
+	int i;
+	for (i=start; i<start+size; i++)
+		(flag? set_bit(i,io_bitmap):clear_bit(i,io_bitmap));
+	tmp = 0;
+    }
+#endif
     enter_priv_on();
     tmp = ioperm(start, size, flag);
     leave_priv_setting();
