@@ -3,6 +3,7 @@
  * 
  * DANG_BEGIN_MODULE
  * 
+ * REMARK
  * Here is where DOSEMU gets booted. From emu.c external calls are made to
  * the specific I/O systems (video/keyboard/serial/etc...) to initialize
  * them. Memory is cleared/set up and the boot sector is read from the
@@ -17,6 +18,7 @@
  * The top level function emulate() is called from dos.c by way of a dll
  * entry point.
  * 
+ * /REMARK
  * DANG_END_MODULE
  * 
  */
@@ -116,17 +118,25 @@ __asm__("___START___: jmp _emulate\n");
 #include "serial.h"
 #include "int.h"
 #include "bitops.h"
+#include "port.h"
 #include "pic.h"
 #include "dpmi.h"
 #include "priv.h"   /* for priv_init */
+#include "port.h"   /* for port_init */
+#include "pci.h"
 #ifdef __NetBSD__
 #include <setjmp.h>
 #endif
+#include "speaker.h"
 
 #ifdef NEW_KBD_CODE
 #include "keyb_clients.h"
 #endif
 
+#ifdef USE_SBEMU
+#include "sound.h"
+#endif
+
 extern void     stdio_init(void);
 extern void     time_setting_init(void);
 extern void     tmpdir_init(void);
@@ -142,6 +152,10 @@ extern void	disk_open(struct disk *dp);
 
 extern void io_select_init(void);
 
+#ifdef USE_THREADS
+extern void treads_init(void);
+#endif
+
 static int      special_nowait = 0;
 
 #ifndef NEW_KBD_CODE
@@ -169,7 +183,7 @@ boot(void)
 	dp = &disktab[1];
 	break;
     default:
-	error("ERROR: unexpected value for config.hdiskboot\n");
+	error("unexpected value for config.hdiskboot\n");
 	leavedos(15);
     }
 
@@ -185,11 +199,11 @@ boot(void)
         d_printf ("Booting from bootfile=%s...\n",dp->boot_name);
         bfd = open (dp->boot_name, O_RDONLY);
         if (bfd == -1) {/* Abort with error */
-            error("ERROR: Boot file %s missing\n",dp->boot_name);
+            error("Boot file %s missing\n",dp->boot_name);
             leavedos(16);
         }
         if (read(bfd, buffer, SECTOR_SIZE) != SECTOR_SIZE) {
-            error("ERROR: Failed to read exactly %d bytes from %s\n",
+            error("Failed to read exactly %d bytes from %s\n",
                   SECTOR_SIZE, dp->boot_name);
             leavedos(16);
         }
@@ -199,14 +213,14 @@ boot(void)
     if (dp->type == PARTITION) {/* we boot partition boot record, not MBR! */
 	d_printf("Booting partition boot record from part=%s....\n", dp->dev_name);
 	if (RPT_SYSCALL(read(dp->fdesc, buffer, SECTOR_SIZE)) != SECTOR_SIZE) {
-	    error("ERROR: reading partition boot sector using partition %s.\n", dp->dev_name);
+	    error("reading partition boot sector using partition %s.\n", dp->dev_name);
 	    leavedos(16);
 	}
     } else if (read_sectors(dp, buffer, 0, 0, 0, 1) != SECTOR_SIZE) {
-	error("ERROR: can't boot from %s, using harddisk\n", dp->dev_name);
+	error("can't boot from %s, using harddisk\n", dp->dev_name);
 	dp = hdisktab;
 	if (read_sectors(dp, buffer, 0, 0, 0, 1) != SECTOR_SIZE) {
-	    error("ERROR: can't boot from hard disk\n");
+	    error("can't boot from hard disk\n");
 	    leavedos(16);
 	}
     }
@@ -383,8 +397,19 @@ emulate(int argc, char **argv)
     /* the transposal of (config_|stdio_)init allows the addition of -o */
     /* to specify a debug out filename, if you're wondering */
 
+    get_time_init();		/* debug can use CPUtime */
     io_select_init();
+    port_init();		/* setup port structures, before config! */
     config_init(argc, argv);	/* parse the commands & config file(s) */
+    get_time_init();
+#ifdef USE_THREADS
+    treads_init();		/* init the threads system,
+				 * after this we will be thread0.
+				 * Because treads_init() captures SIGTERM
+				 * and signal_init() also does it,
+				 * we capture it via 'exit'.
+    				 */
+#endif
     stdio_init();		/* initialize stdio & open debug file */
     module_init();
     low_mem_init();		/* initialize the lower 1Meg */
@@ -392,7 +417,11 @@ emulate(int argc, char **argv)
     signal_init();		/* initialize sig's & sig handlers */
     device_init();		/* initialize keyboard, disk, video, etc. */
     cpu_setup();		/* setup the CPU */
+    pci_setup();
     hardware_setup();		/* setup any hardware */
+#ifdef NEW_PORT_CODE
+    extra_port_init();		/* setup ports dependent on config */
+#endif
     memory_init();		/* initialize the memory contents */
     boot();			/* read the boot sector & get moving */
     timer_interrupt_init();	/* start sending int 8h int signals */
@@ -425,7 +454,12 @@ emulate(int argc, char **argv)
 	pic_request(16);
 #endif
 #endif
+#ifdef USE_INT_QUEUE
 	int_queue_run();
+#endif
+#ifdef USE_SBEMU
+	run_sb(); /* Beat Karcher to this one .. 8-) - AM */
+#endif
     }
 
     error("error exit: (%d,0x%04x) in_sigsegv: %d ignore_segv: %d\n",
@@ -436,6 +470,7 @@ emulate(int argc, char **argv)
     leavedos(99);
 }
 
+#if 0    /* disable C-A-D until someone will fix it (if really needed) */
 extern void HMA_MAP(int);
 
 void
@@ -467,17 +502,6 @@ dos_ctrl_alt_del(void)
     boot();
     timer_interrupt_init();	/* start sending int 8h int signals */
 }
-
-#if 0    /* this appears to be unused. */
-void
-dos_ctrlc(void)
-{
-    k_printf("DOS ctrl-c!\n");
-    p_dos_str("^C\n\r");	/* print ctrl-c message */
-    keybuf_clear();
-
-    do_soft_int(0x23);
-}
 #endif
 
 static void
@@ -497,11 +521,14 @@ ign_sigs(int sig)
 #define LEAVEDOS_TIMEOUT (3 * config.freq)
 #define LEAVEDOS_SIGOUT  5
     if ((timerints >= LEAVEDOS_TIMEOUT) || (otherints >= LEAVEDOS_SIGOUT)) {
-	error("ERROR: timed/signalled out in leavedos()\n");
+	error("timed/signalled out in leavedos()\n");
 	exit(1);
     }
 }
 
+int leavedos_recurse_check = 0;
+
+
 /* "graceful" shutdown */
 void
 leavedos(int sig)
@@ -510,16 +537,15 @@ leavedos(int sig)
     struct itimerval itv;
     extern int errno;
 
-    static int recurse_check = 0;
    
-    warn("leavedos(%d) called - shutting down\n", sig);
-    if (recurse_check)
+    if (leavedos_recurse_check)
       {
        error("leavedos called recursively, forgetting the graceful exit!\n");
        flush_log();
        _exit(sig);
       }
-    recurse_check = 1;
+    leavedos_recurse_check = 1;
+    warn("leavedos(%d) called - shutting down\n", sig);
 #if 1 /* BUG CATCHER */
     if (in_vm86) {
       g_printf("\nkilled while in vm86(), trying to dump DOS-registers:\n");
@@ -543,11 +569,19 @@ leavedos(int sig)
     SETSIG(SIGTRAP, ign_sigs);
     warn("leavedos(%d) called - shutting down\n", sig);
 
-    if (config.speaker==SPKR_NATIVE ||
-       (config.speaker==SPKR_EMULATED && config.console && !config.X))
-    {
+    if (config.speaker == SPKR_EMULATED) {
+      g_printf("SPEAKER: sound off\n");
+      speaker_off();		/* turn off any sound */
+    }
+    if (config.speaker==SPKR_NATIVE) {
        g_printf("SPEAKER: sound off\n");
-       do_ioctl(console_fd, KIOCSOUND, 0);	       /* turn off any sound */
+       /* Since the speaker is native hardware use port manipulation,
+	* we don't know what is actually implementing the kernel's
+	* ioctls.
+	* My port logic is acutally stolen from kd_nosound in the kernel.
+	* 		--EB 21 September 1997
+	*/
+       port_safe_outb(0x61, port_safe_inb(0x61)&0xFC); /* turn off any sound */
     }
 
     g_printf("calling close_all_printers\n");
@@ -604,7 +638,14 @@ leavedos(int sig)
 	disallocate_vt();
     }
     flush_log();
+#ifdef USE_THREADS
+    {
+	extern void Exit(int status);
+	Exit(sig);
+    }
+#else
     _exit(sig);
+#endif
 }
 
 #if 0
