@@ -1,8 +1,10 @@
 /* 
  * DANG_BEGIN_MODULE
  *
+ * REMARK
  * This module contains a text-mode video interface for the X Window 
  * System. It has mouse and selection 'cut' support.
+ * /REMARK
  *
  * DANG_END_MODULE
  *
@@ -96,6 +98,7 @@
 /*#define OLD_X_TEXT*/		/* do you want to use the old X_textmode's */
 #define CONFIG_X_SELECTION 1
 #define CONFIG_X_MOUSE 1
+#define CONFIG_X_SPEAKER 1		/* do you want speaker support in X? */
 
 /* Comment out if you wish to have a visual indication of the area
    being updated */
@@ -141,13 +144,16 @@
 #include "screen.h"
 #endif
 
+#if CONFIG_X_SPEAKER
+#include "speaker.h"
+#endif
+
 #define Bit8u byte
 #define Bit16u us
 #define Boolean boolean
 
 
 #include "vgaemu.h"
-#include "vgaemu_inside.h"
 
 static const u_char dos_to_latin[]={
    0xc7, 0xfc, 0xe9, 0xe2, 0xe4, 0xe0, 0xe5, 0xe7,  /* 80-87 */ 
@@ -1027,6 +1033,10 @@ static int X_init(void)
   
   /* HACK!!! Don't konw is video_mode is already set */
   X_setmode(TEXT, co, li);
+
+#ifdef CONFIG_X_SPEAKER
+  register_speaker(display, X_speaker_on, X_speaker_off);
+#endif
             
   return(0);                        
 }
@@ -1046,6 +1056,13 @@ static void X_close(void)
   if (display==NULL)
     return;
 
+#ifdef CONFIG_X_SPEAKER
+  /* turn off the sound, and */
+  speaker_off();
+  /* reset the speaker to it's default */
+  register_speaker(NULL, NULL, NULL);
+#endif
+
   XUnloadFont(display,vga_font);
   XDestroyWindow(display,mainwindow);
 
