@@ -65,7 +65,7 @@ enum cirrusChipTable {
 		port_out(tmp | (m), 0x3cf); \
 	}
 
-/* We use a seperate banking routine with 16K granularity for some accel. */
+/* We use a separate banking routine with 16K granularity for some accel. */
 /* functions. Knows about different bank granularity for 2Mb cards. */
 
 #define setwritebank(n) \
@@ -185,8 +185,8 @@ enum cirrusChipTable {
 
 extern void vga_init_cirrus(void);
 
-extern u_char cirrus_ext_video_port_in(int port);
-extern void cirrus_ext_video_port_out(u_char value, int port); 
+extern unsigned char cirrus_ext_video_port_in(ioport_t port);
+extern void cirrus_ext_video_port_out(ioport_t port, unsigned char value);
 
 extern void cirrus_set_bank_read(u_char bank);
 extern void cirrus_set_bank_write(u_char bank);
