@@ -1,6 +1,7 @@
 /*
  * DANG_BEGIN_MODULE
  *
+ * REMARK
  * Here's all the calls to the code to try and properly save & restore
  * the video state between VC's and the attempts to control updates to
  * the VC whilst the user is using another. We map between the real
@@ -11,6 +12,7 @@
  * the emulator try to execute the card's initialization routine which is
  * normally located at address c000:0003. This can now be changed as an
  * option.
+ * /REMARK
  *
  * DANG_END_MODULE
  *
@@ -60,6 +62,7 @@ extern int munmap __P ((caddr_t __addr, size_t __len));
 #endif
 
 #define INIT_C2TRAP
+#define NO_VC_NO_DEBUG
 
 #include "emu.h"
 #include "memory.h"
@@ -85,7 +88,9 @@ extern inline void console_update_cursor (int, int, int, int);
 static void set_dos_video ();
 void put_video_ram ();
 
+#ifdef NO_VC_NO_DEBUG
 static struct debug_flags save_debug_flags;
+#endif
 
 extern int
   dosemu_sigaction (int sig, struct sigaction *, struct sigaction *);
@@ -95,10 +100,10 @@ static void SIGACQUIRE_call (void);
 void dump_video_regs (void);
 
 #ifdef WANT_DUMP_VIDEO
-static void dump_video ();
+static void dump_video (void);
 #endif
 
-void dump_video_linux ();
+void dump_video_linux (void);
 
 
 static int  color_text;
@@ -128,7 +133,7 @@ forbid_switch (void)
 /* check whether we are running on the console; initialise
  * config.console, console_fd and scr_state.console_no
  */
-void check_console() {
+void check_console(void) {
 
 #ifdef __NetBSD__
     struct screeninfo info;
@@ -232,7 +237,9 @@ acquire_vt (int sig, struct sigcontext_struct context)
 {
   dos_has_vt = 1;
 
+#ifdef NO_VC_NO_DEBUG
   d = save_debug_flags;
+#endif
   v_printf ("VID: Acquiring VC\n");
   forbid_switch ();
 #ifdef __NetBSD__
@@ -311,7 +318,9 @@ set_linux_video (void)
 static void
 SIGRELEASE_call (void)
 {
+#ifdef NO_VC_NO_DEBUG
   save_debug_flags = d;
+#endif
 
   if (scr_state.current == 1)
     {
@@ -363,7 +372,9 @@ SIGRELEASE_call (void)
   else {
    /* we don't want any debug activity while VC is not active */
     flush_log();
+#ifdef NO_VC_NO_DEBUG
     memset(&d, 0, sizeof(struct debug_flags));
+#endif
   }
 }
 
@@ -374,7 +385,7 @@ wait_vc_active (void)
     {
       if (errno != EINTR)
 		{
-			error ("ERROR: VT_WAITACTIVE for %d gave %d: %s\n", scr_state.console_no,
+			error("VT_WAITACTIVE for %d gave %d: %s\n", scr_state.console_no,
 			     errno, strerror (errno));
 	    }
       return -1;
@@ -523,7 +534,7 @@ get_video_ram (int waitflag)
 
       if ((long) graph_mem < 0)
 	{
-	  error ("ERROR: mmap error in get_video_ram (text): %lx, errno %d\n",
+	  error("mmap error in get_video_ram (text): %x, errno %d\n",
 		 (Bit32u)graph_mem, errno);
 	  return;
 	}
@@ -660,7 +671,7 @@ open_kmem (void)
   leave_priv_setting();
   if (mem_fd < 0)
     {
-      error ("ERROR: can't open /dev/mem: errno=%d, %s \n",
+      error("can't open /dev/mem: errno=%d, %s \n",
 	     errno, strerror (errno));
       leavedos (0);
       return;
@@ -738,15 +749,29 @@ get_perm (void)
 	  v_printf ("VGA: can't get I/O permissions \n");
 	  exit (-1);
 	}
-      if (((config.chipset==S3) || (config.chipset==CIRRUS)) &&
-		(set_ioperm (0x102, 1, 1) || set_ioperm (0x2ea, 4, 1)))
-	{
-	  v_printf ("S3/CIRRUS: can't get I/O permissions \n");
+      if ((config.chipset == S3) &&
+	(set_ioperm(0x102, 1, 1) || set_ioperm(0x2ea, 4, 1))) {
+	  v_printf("S3: can't get I/O permissions");
 	  exit (-1);
 	}
-      if (config.chipset == ATI && (set_ioperm (0x102, 1, 1) || set_ioperm (0x1ce, 2, 1) || set_ioperm (0x2ec, 4, 1)))
-	{
-	  v_printf ("ATI: can't get I/O permissions \n");
+      if ((config.chipset == CIRRUS) &&
+	(set_ioperm(0x102, 2, 1) || set_ioperm(0x2ea, 4, 1))) {
+	  v_printf("CIRRUS: can't get I/O permissions");
+	  exit (-1);
+	}
+      if ((config.chipset == ATI) &&
+	(set_ioperm(0x102, 1, 1) || set_ioperm(0x1ce, 2, 1) || set_ioperm(0x2ec, 4, 1))) {
+	  v_printf("ATI: can't get I/O permissions");
+	  exit (-1);
+	}
+      if ((config.chipset == MATROX) &&
+	(set_ioperm(0x102, 2, 1) || set_ioperm(0x2ea, 4, 1) || set_ioperm(0x3de, 2, 1))) {
+	  v_printf("MATROX: can't get I/O permissions");
+	  exit (-1);
+	}
+      if ((config.chipset == WDVGA) &&
+	(set_ioperm(0x102, 2, 1) || set_ioperm(0x3de, 2, 1))) {
+	  v_printf("WDVGA: can't get I/O permissions");
 	  exit (-1);
 	}
       /* color or monochrome text emulation? */
@@ -804,16 +829,29 @@ release_perm (void)
 	      v_printf ("VGA: can't release I/O permissions \n");
 	      leavedos (-1);
 	    }
-	  if (((config.chipset==S3) || (config.chipset==CIRRUS)) && 
-		(set_ioperm (0x102, 1, 0) || set_ioperm (0x2ea, 4, 0)))
-	    {
-	      v_printf ("S3/CIRRUS: can't release I/O permissions\n");
+	  if ((config.chipset == S3) &&
+		(set_ioperm(0x102, 1, 0) || set_ioperm(0x2ea, 4, 0))) {
+	      v_printf ("S3: can't release I/O permissions\n");
 	      leavedos (-1);
 	    }
-	  if (config.chipset == ATI && 
-		(set_ioperm (0x102, 1, 0) || set_ioperm (0x1ce, 2, 0) || set_ioperm (0x2ec, 4, 0)))
-	    {
-	      v_printf ("ATI: can't release I/O permissions \n");
+	  if ((config.chipset == CIRRUS) &&
+		(set_ioperm(0x102, 2, 0) || set_ioperm(0x2ea, 4, 0))) {
+	      v_printf ("CIRRUS: can't release I/O permissions\n");
+	      leavedos (-1);
+	    }
+	  if ((config.chipset == ATI) &&
+		(set_ioperm(0x102, 1, 0) || set_ioperm(0x1ce, 2, 0) || set_ioperm(0x2ec, 4, 0))) {
+	      v_printf ("ATI: can't release I/O permissions\n");
+	      leavedos (-1);
+	    }
+	  if ((config.chipset == MATROX) &&
+		(set_ioperm(0x102, 2, 0) || set_ioperm(0x2ea, 4, 0) || set_ioperm(0x3de, 2, 0))) {
+	      v_printf ("MATROX: can't release I/O permissions\n");
+	      leavedos (-1);
+	    }
+	  if ((config.chipset == WDVGA) &&
+		(set_ioperm(0x102, 2, 0) || set_ioperm(0x3de, 2, 0))) {
+	      v_printf ("WDVGA: can't release I/O permissions\n");
 	      leavedos (-1);
 	    }
 	}
@@ -908,10 +946,10 @@ reset_att (void)
 /* Attempt to virtualize calls to video ports */
 
 u_char att_d_index = 0;
-static isr_read = 0;
+static int isr_read = 0;
 
 u_char
-video_port_in (int port)
+video_port_in (ioport_t port)
 {
   /* v_printf("Video read on port 0x%04x.\n",port); */
   switch (port)
@@ -995,7 +1033,7 @@ video_port_in (int port)
 }
 
 void
-video_port_out (u_char value, int port)
+video_port_out (ioport_t port, u_char value)
 {
   /* v_printf("Video write on port 0x%04x,byte 0x%04x.\n",port, value); */
   switch (port)
@@ -1013,7 +1051,7 @@ video_port_out (u_char value, int port)
 	  dosemu_regs.regs[CRT + dosemu_regs.regs[CRTI]] = value;
 	}
       else
-	ext_video_port_out (value, port);
+	ext_video_port_out (port, value);
       break;
     case GRA_I:
       v_printf ("Set Index GRAI 0x%02x\n", value);
@@ -1026,7 +1064,7 @@ video_port_out (u_char value, int port)
 	  dosemu_regs.regs[GRA + dosemu_regs.regs[GRAI]] = value;
 	}
       else
-	ext_video_port_out (value, port);
+	ext_video_port_out (port, value);
       break;
     case SEQ_I:
       v_printf ("Set Index SEQI 0x%02x\n", value);
@@ -1039,7 +1077,7 @@ video_port_out (u_char value, int port)
 	  dosemu_regs.regs[SEQ + dosemu_regs.regs[SEQI]] = value;
 	}
       else
-	ext_video_port_out (value, port);
+	ext_video_port_out (port, value);
       break;
     case ATT_IW:
       if (isr_read)
@@ -1052,7 +1090,7 @@ video_port_out (u_char value, int port)
 	{
 	  isr_read = 1;
 	  if (att_d_index > 20)
-	    ext_video_port_out (value, port);
+	    ext_video_port_out (port, value);
 	  else
 	    {
 	      v_printf ("Write Data at ATT Index 0x%02x = 0x%02x \n", att_d_index, value);
@@ -1090,7 +1128,7 @@ video_port_out (u_char value, int port)
       dosemu_regs.regs[GR2P] = value;
       break;
     default:
-      ext_video_port_out (value, port);
+      ext_video_port_out (port, value);
     }
   return;
 }
@@ -1182,12 +1220,9 @@ dump_video_linux (void)
 /*
  * install_int_10_handler - install a handler for the video-interrupt (int 10)
  *                          at address INT10_SEG:INT10_OFFS. Currently
- *                          it's f100:4100.
+ *                          it's f800:4200.
  *                          The new handler is only installed, if the bios
- *                          handler at f100:4100 is not the appropriate on
- *                          that means, if we use not mda with X
- *                          The new handler is only installed, if the bios
- *                          handler at f100:4100 is not the appropriate on
+ *                          handler at f800:4200 is not the appropriate on
  *                          that means, if we use not mda with X
  */
 
@@ -1242,6 +1277,9 @@ install_int_10_handler (void)
     extern void bios_f000_int10ptr(), bios_f000();
     ptr = (u_char *)((BIOSSEG << 4) + ((long)bios_f000_int10ptr - (long)bios_f000));
     *((long *)ptr) = 0xe0000003;
+    v_printf("VID: new int10 handler at %p\n",ptr);
   }
+  else
+    v_printf("VID: install_int_10_handler: do nothing\n");
 #endif
 }
