@@ -20,6 +20,8 @@
 #include "trident.h"
 #include "avance.h"
 #include "cirrus.h"
+#include "matrox.h"
+#include "wdvga.h"
 #if 0
 #include "hgc.h"
 #endif
@@ -61,13 +63,13 @@ void set_bank_write_dummy(u_char bank)
   return;
 }
 
-u_char dummy_ext_video_port_in(int port)
+u_char dummy_ext_video_port_in(ioport_t port)
 {
   v_printf("Bad Read on port 0x%04x\n", port);
   return (0);
 }
 
-void dummy_ext_video_port_out(u_char value, int port)
+void dummy_ext_video_port_out(ioport_t port, u_char value)
 {
   v_printf("Bad Write on port 0x%04x with value 0x%02x\n", port, value);
 }
@@ -254,7 +256,7 @@ void store_vga_mem(u_char * mem, u_char mem_size[], u_char banks)
 
 /*      memcpy((caddr_t) (mem + position), (caddr_t) GRAPH_BASE, p[plane]); */
       MEMCPY_2UNIX((caddr_t) (mem + position), (caddr_t) GRAPH_BASE, p[plane]);
-      v_printf("READ Bank=%d, plane=0x%02x, *mem=0x%x, GRAPH_BASE=%08x, mem=0x%lx\n",
+      v_printf("READ Bank=%d, plane=0x%02x, *mem=0x%x, GRAPH_BASE=%08x, mem=0x%x\n",
 	 banks, plane, *(int *)(mem + position), *(int *) GRAPH_BASE, (Bit32u) (mem + position));
       position = position + p[plane];
     }
@@ -466,6 +468,14 @@ int vga_initialize(void)
   dosemu_regs.mem = NULL;
   get_perm();
 
+  /* defaults to override */
+  save_ext_regs = save_ext_regs_dummy;
+  restore_ext_regs = restore_ext_regs_dummy;
+  set_bank_read = set_bank_read_dummy;
+  set_bank_write = set_bank_write_dummy;
+  ext_video_port_in = dummy_ext_video_port_in;
+  ext_video_port_out = dummy_ext_video_port_out;
+
   switch (config.chipset) {
   case TRIDENT:
     vga_init_trident();
@@ -491,14 +501,16 @@ int vga_initialize(void)
     vga_init_cirrus();
     v_printf("Cirrus CARD in use\n");
     break;
+  case MATROX:
+    vga_init_matrox();
+    v_printf("Matrox CARD in use\n");
+    break;
+  case WDVGA:
+    vga_init_wd();
+    v_printf("Paradise CARD in use\n");
+    break;
 
   default:
-    save_ext_regs = save_ext_regs_dummy;
-    restore_ext_regs = restore_ext_regs_dummy;
-    set_bank_read = set_bank_read_dummy;
-    set_bank_write = set_bank_write_dummy;
-    ext_video_port_in = dummy_ext_video_port_in;
-    ext_video_port_out = dummy_ext_video_port_out;
     v_printf("Unspecific VIDEO selected = 0x%04x\n", config.chipset);
   }
 
@@ -648,7 +660,7 @@ void init_vga_card(void)
 #endif
 
   if (!config.mapped_bios) {
-    error("ERROR: CAN'T DO VIDEO INIT, BIOS NOT MAPPED!\n");
+    error("CAN'T DO VIDEO INIT, BIOS NOT MAPPED!\n");
     return;
   }
   if (set_ioperm(0x3b0, 0x3db - 0x3b0, 1))
