@@ -63,8 +63,10 @@
  *
  * DANG_BEGIN_MODULE
  *
+ * REMARK
  * The VGA emulator for dosemu. Emulated are the video meory and the VGA
  * register set (CRTC, DAC, etc.).
+ * /REMARK
  *
  * DANG_END_MODULE
  *
@@ -104,10 +106,6 @@
 #include "port.h"
 #include "video.h"
 #include "vgaemu.h"
-#include "vgaemu_inside.h"
-#ifdef VESA
-#include "vesa.h"
-#endif
 #include "priv.h"
 
 
@@ -240,7 +238,7 @@ static int selfmem_fd;
  * DANG_END_FUNCTION
  *
  */
-void VGA_emulate_outb(int port, unsigned char value)
+void VGA_emulate_outb(ioport_t port, unsigned char value)
 {
 #ifdef DEBUG_IO
   v_printf("VGAemu: VGA_emulate_outb(): outb(0x%03x, 0x%02x)\n", port, value);
@@ -248,6 +246,16 @@ void VGA_emulate_outb(int port, unsigned char value)
 
   switch(port)
     {
+#ifdef NEW_X_CODE
+    case CRTC_INDEX:
+      CRTC_set_index(value);
+      break;
+
+    case CRTC_DATA:
+      CRTC_write_value(value);
+      break;
+#endif
+
     case SEQUENCER_INDEX:
       Seq_set_index(value);
       break;
@@ -307,14 +315,29 @@ void VGA_emulate_outb(int port, unsigned char value)
  * DANG_END_FUNCTION
  *
  */
-unsigned char VGA_emulate_inb(int port)
+unsigned char VGA_emulate_inb(ioport_t port)
 {
+#ifdef NEW_X_CODE
+  unsigned char uc = 0xff;
+#else
+  unsigned char uc = 0;
+#endif
 #ifdef DEBUG_IO
   v_printf("VGAemu: VGA_emulate_inb(): inb(0x%03x)\n", port);
 #endif
 
   switch(port)
     {
+#ifdef NEW_X_CODE
+    case CRTC_INDEX:
+      return(CRTC_get_index());
+      break;
+
+    case CRTC_DATA:
+      return(CRTC_read_value());
+      break;
+#endif
+
     case SEQUENCER_INDEX:
       return(Seq_get_index());
       break;
@@ -340,7 +363,7 @@ unsigned char VGA_emulate_inb(int port)
       break;
 
     case DAC_WRITE_INDEX: /* this is undefined, but we have to do something */
-      return(0);
+      return(uc);
       break;
 
     case DAC_DATA:
@@ -356,7 +379,7 @@ unsigned char VGA_emulate_inb(int port)
       v_printf("VGAemu: not (yet) smart enough to emulate read from"
 	       " port 0x%04x\n", port);
 #endif
-      return(0); /* do something */
+      return(uc); /* do something */
       break;
     }
 }
@@ -368,15 +391,15 @@ unsigned char VGA_emulate_inb(int port)
  * DANG_BEGIN_FUNCTION vga_emu_fault(struct sigcontext_struct *scp)        
  *
  * description:
- *  vga_emu_fault() is used to catch video acces, and handle it.
+ *  vga_emu_fault() is used to catch video access, and handle it.
  *  This function is called from dosemu/sigsegv.c:dosemu_fault()
  *  The sigcontext_struct is defined in include/cpu.h
  *  Now it catches only changes in a 4K page, but maybe it is useful to
- *  catch each video acces. The problem when you do that is, you have to 
- *  simulate each intruction which could write to the video-memory.
- *  it is easy to get the place where the exeption happens (scp->cr2),
+ *  catch each video access. The problem when you do that is, you have to 
+ *  simulate each instruction which could write to the video-memory.
+ *  it is easy to get the place where the exception happens (scp->cr2),
  *  but what are those changes?
- *  An other problem is, it could eat a lot time, but it does now also.
+ *  An other problem is, it could eat a lot of time, but it does now also.
  *  
  * DANG_END_FUNCTION                        
  */     
@@ -456,7 +479,9 @@ static inline caddr_t vga_mmap(caddr_t  addr,  size_t  len,
  */     
 unsigned char* vga_emu_init(void)
 {
+#ifdef NEW_PORT_CODE
   emu_iodev_t io_device;
+#endif
 
   vga_emu_memory=(unsigned char*)valloc(VGAEMU_BANK_SIZE*VGAEMU_BANKS);
   if(vga_emu_memory==NULL)
@@ -493,36 +518,56 @@ unsigned char* vga_emu_init(void)
   DAC_init();
   Attr_init();
   Seq_init();
+#ifdef NEW_X_CODE
+  CRTC_init();
+#endif
 
+#ifdef NEW_PORT_CODE
   /* register VGA ports */
   io_device.read_portb = VGA_emulate_inb;
   io_device.write_portb = VGA_emulate_outb;
   io_device.read_portw = NULL;
   io_device.write_portw = NULL;
+  io_device.read_portd = NULL;
+  io_device.write_portd = NULL;
   io_device.irq = EMU_NO_IRQ;
+  io_device.fd = -1;
   
   /* register attribute controller */
   io_device.handler_name = "VGAemu Attribute controller";
   io_device.start_addr = ATTRIBUTE_INDEX;
   io_device.end_addr = INPUT_STATUS_0;
-  port_register_handler(io_device);
+  port_register_handler(io_device, 0);
 
   /* register sequencer */
   io_device.handler_name = "VGAemu Sequencer";
   io_device.start_addr = SEQUENCER_INDEX;
   io_device.end_addr = SEQUENCER_DATA;
-  port_register_handler(io_device);
+  port_register_handler(io_device, 0);
 
   /* register DAC */
   io_device.handler_name = "VGAemu DAC";
   io_device.start_addr = DAC_BASE;
   io_device.end_addr = DAC_DATA;
-  port_register_handler(io_device);
+  port_register_handler(io_device, 0);
 
-#ifdef VESA
-  vesa_init();
+#ifdef NEW_X_CODE
+  /* register crt controller */
+  io_device.handler_name = "VGAemu CRT Controller";
+  io_device.start_addr = CRTC_INDEX;
+  io_device.end_addr = CRTC_DATA;
+  port_register_handler(io_device, 0);
 #endif
 
+  /* register graphics - just to trap these registers and do no harm */
+  io_device.handler_name = "VGAemu Graph";
+  io_device.start_addr = GRAPHICS_BASE;
+  io_device.end_addr = GRAPHICS_DATA;
+  port_register_handler(&io_device, 0);
+#endif
+
+  vesa_init();
+
   return vga_emu_memory;
 }
 
@@ -533,10 +578,9 @@ unsigned char* vga_emu_init(void)
  * DANG_BEGIN_FUNCTION int vgaemu_get_changes_in_pages
  *
  * description:
- *  vgaemu_get_changes_in_pages() is vgaemu_get_changes() is used 
- *  to get the changed 4K pages .
+ *  vgaemu_get_changes_in_pages() is used to get the changed 4K pages.
  *  This function is only called from video/vgaemu.c .
- *  It has to called several times to make sure grabbing all the changed
+ *  It has to be called several times to make sure we grab all the changed
  *  pages.
  *
  * should be updated for other video modes than 0x13
@@ -811,7 +855,7 @@ int vgaemu_get_changes_and_update_XImage_0x13(unsigned char **base, unsigned lon
  *  This function isn't called anywhere, but has to be used, with
  *  other videomodes.
  *  This function just remaps his 'own' memory into the 0xA000-0xB0000
- *  area and returns True on succes and False on error.
+ *  area and returns True on success and False on error.
  *
  *  At the moment just a stupid function, but it is a start.
  *  Jou must be sure, you've got all changes before you switch a bank!
@@ -820,6 +864,9 @@ int vgaemu_get_changes_and_update_XImage_0x13(unsigned char **base, unsigned lon
  */     
 int vgaemu_switch_page(unsigned int pagenumber)
 {
+#ifdef NEW_X_CODE
+  int i;
+#endif
   /* remapping the one bank of the allocated memmory */
 
  /* Is this < or <= ? -- Erik */
@@ -841,7 +888,17 @@ int vgaemu_switch_page(unsigned int pagenumber)
                pagenumber);
     }
   
+#ifdef NEW_X_CODE
+  v_printf("VGAemu: vga_emu_switch_page: switched to bank %d\n", current_bank);
+
+  for(i = 0; i < 16; i++) {
+    if(vgaemu_graphic_dirty_page[current_bank*16 + i] == False) {
+      mprotect((void *) (0xA0000 + i * 0x1000), 0x1000, PROT_READ);
+    }
+  }
+#else
   mprotect((void *)0xA0000,0xFFFF,PROT_READ); /* should not be needed, but... */
+#endif
   return True;
 }
 
@@ -1068,6 +1125,9 @@ int vgaemu_update(unsigned char **base, unsigned long int *offset, unsigned long
   
   switch(current_mode_info->memorymodel)
     {
+#ifdef NEW_X_CODE
+    case PL4:
+#endif
     case P8:
       return(vgaemu_get_changes_and_update_XImage_0x13(base, offset, len, method, modewidth));
       break;
