@@ -28,11 +28,6 @@
 
 #include "extern.h"
 
-#ifdef X86_EMULATOR
-#include "x86_emulator.h"
-#endif
-
-#ifndef X86_EMULATOR
 /* all registers as a structure */
 #ifdef __linux__
 #define REGS  vm86s.regs
@@ -133,8 +128,6 @@
 #define READ_FLAGS()        LWORD(eflags)
 #define READ_FLAGSE()        REG(eflags)
 
-#endif /* #ifndef X86_EMULATOR */
-
 /*
  * nearly directly stolen from Linus : linux/kernel/vm86.c
  *
@@ -357,7 +350,11 @@ void dosemu_fault(int, struct sigcontext_struct);
 #endif
 
 void show_regs(char *, int), show_ints(int, int);
-__inline__ int do_hard_int(int), do_soft_int(int);
+#ifdef USE_INT_QUEUE
+__inline__ int do_hard_int(int);
+#endif
+
+int cpu_trap_0f (unsigned char *, struct sigcontext_struct *);
 
 extern unsigned int read_port_w(unsigned short port);
 extern int write_port_w(unsigned int value,unsigned short port);
