@@ -18,9 +18,6 @@
 #include "machcompat.h"
 #include "cpu.h"
 #include "vm86plus.h"
-
-/* Note: priv.h needs 'error' !!! */
-#define error(f,a...)	 	fprintf(stderr, f, ##a)
 #include "priv.h"
 
 #include "extern.h"
@@ -121,8 +118,9 @@ EXTERN int scanseq;
 EXTERN int cursor_row;
 EXTERN int cursor_col;
 
-void dos_ctrlc(void), dos_ctrl_alt_del(void);
-int ext_fs(int, char *, char *, int);
+#if 0
+void dos_ctrl_alt_del(void);	/* disabled */
+#endif
 
 EXTERN void run_vm86(void);
 EXTERN void     vm86_GP_fault();
@@ -134,128 +132,7 @@ EXTERN void     vm86_GP_fault();
 
 void getKeys(void);
 
-
-struct debug_flags {
-  unsigned char
-   disk,		/* disk msgs         "d" */
-   read,		/* disk read         "R" */
-   write,		/* disk write        "W" */
-   dos,			/* unparsed int 21h  "D" */
-   cdrom,               /* cdrom             "C" */
-   video,		/* video             "v" */
-   X,			/* X support         "X" */
-   keyb,		/* keyboard          "k" */
-   io,			/* port I/O          "i" */
-   io_trace,		/* port I/O tracing  "T" */
-   serial,		/* serial            "s" */
-   mouse,		/* mouse             "m" */
-   defint,		/* default ints      "#" */
-   printer,		/* printer           "p" */
-   general,		/* general           "g" */
-   config,		/* configuration     "c" */
-   warning,		/* warning           "w" */
-   hardware,		/* hardware          "h" */
-   IPC,			/* IPC               "I" */
-   EMS,			/* EMS               "E" */
-   xms,			/* xms               "x" */
-   dpmi,		/* dpmi              "M" */
-   network,		/* IPX network       "n" */
-   pd,			/* Packet driver     "P" */
-   request,		/* PIC               "r" */
-   sound;		/* SOUND	     "S" */
-};
-
-#if __GNUC__ >= 2
-# define FORMAT(T,A,B)  __attribute__((format(T,A,B)))
-#else
-# define FORMAT(T,A,B)
-#endif
-
-#if __GNUC__ >= 2
-# define NORETURN	__attribute__((noreturn))
-#else
-# define NORETURN
-#endif
-
-EXTERN void saytime(char *m_str);
-
-int log_printf(int, const char *,...) FORMAT(printf, 2, 3);
-
-void p_dos_str(char *,...) FORMAT(printf, 1, 2);
-
-#if 0  /* set this to 0, if you want dosemu to honor the -D flags */
- #define NO_DEBUGPRINT_AT_ALL
-#endif
-
-extern FILE *dbg_fd;
-
-#define ifprintf(flg,fmt,a...)	do{ if (flg) log_printf(flg,fmt,##a); }while(0)
-
-/* Note: moved it at top of emu.h, we need to define 'error' for priv.h 
-  #define error(f,a...)	 	fprintf(stderr, f, ##a) */
-
-#define hard_error(f, a...)	fprintf(stderr, f, ##a)
-#define dbug_printf(f,a...)	ifprintf(2,f,##a)
-#define flush_log()		{ if (dbg_fd) log_printf(-1, "\n"); }
-
-#ifndef NO_DEBUGPRINT_AT_ALL
-
-#define k_printf(f,a...) 	ifprintf(d.keyb,f,##a)
-#define h_printf(f,a...) 	ifprintf(d.hardware,f,##a)
-#define v_printf(f,a...) 	ifprintf(d.video,f,##a)
-#define X_printf(f,a...)        ifprintf(d.X,f,##a)
-#define s_printf(f,a...) 	ifprintf(d.serial,f,##a)
-#define p_printf(f,a...) 	ifprintf(d.printer,f,##a)
-#define d_printf(f,a...) 	ifprintf(d.disk,f,##a)
-#define i_printf(f,a...) 	ifprintf(d.io,f,##a)
-#define T_printf(f,a...) 	ifprintf(d.io_trace,f,##a)
-#define R_printf(f,a...) 	ifprintf(d.read,f,##a)
-#define W_printf(f,a...) 	ifprintf(d.write,f,##a)
-#define C_printf(f,a...)        ifprintf(d.cdrom,f,##a)
-#define g_printf(f,a...)	ifprintf(d.general,f,##a)
-#define x_printf(f,a...)	ifprintf(d.xms,f,##a)
-#define D_printf(f,a...)	ifprintf(d.dpmi,f,##a)
-#define m_printf(f,a...)	ifprintf(d.mouse,f,##a)
-#define I_printf(f,a...) 	ifprintf(d.IPC,f,##a)
-#define E_printf(f,a...) 	ifprintf(d.EMS,f,##a)
-#define c_printf(f,a...) 	ifprintf(d.config,f,##a)
-#define e_printf(f,a...) 	ifprintf(1,f,##a)
-#define n_printf(f,a...)        ifprintf(d.network,f,##a)  /* TRB     */
-#define pd_printf(f,a...)       ifprintf(d.pd,f,##a)	   /* pktdrvr */
-#define r_printf(f,a...)        ifprintf(d.request,f,##a)
-#define warn(f,a...)     	ifprintf(d.warning,f,##a)
-#define S_printf(f,a...)     	ifprintf(d.sound,f,##a)
-#define ds_printf(f,a...)     	ifprintf(d.dos,f,##a)
-
-#else
-
-#define k_printf(f,a...)
-#define h_printf(f,a...)
-#define v_printf(f,a...)
-#define X_printf(f,a...)
-#define s_printf(f,a...)
-#define p_printf(f,a...)
-#define d_printf(f,a...)
-#define i_printf(f,a...)
-#define R_printf(f,a...)
-#define W_printf(f,a...)
-#define C_printf(f,a...)
-#define g_printf(f,a...)
-#define x_printf(f,a...)
-#define D_printf(f,a...)
-#define m_printf(f,a...)
-#define I_printf(f,a...)
-#define E_printf(f,a...)
-#define c_printf(f,a...)
-#define e_printf(f,a...)
-#define n_printf(f,a...)
-#define pd_printf(f,a...)
-#define r_printf(f,a...)
-#define warn(f,a...)
-#define S_printf(f,a...)
-#define ds_printf(f,a...)
-
-#endif
+#include "dosemu_debug.h"
 
      void char_out(u_char, int);
 
@@ -270,8 +147,13 @@ extern FILE *dbg_fd;
 
      int set_ioperm(int, int, int);
 
-     EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
-     EXTERN u_char in_sighandler, in_ioctl;
+#ifdef X86_EMULATOR
+EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
+#else
+EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
+#endif
+ 
+EXTERN u_char in_sighandler, in_ioctl;
 /* one-entry queue ;-( for ioctl's */
 EXTERN struct ioctlq iq INIT({0, 0, 0, 0}); 
 EXTERN u_char in_ioctl INIT(0);
@@ -350,16 +232,28 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
 #define False FALSE
 #endif
 
-     typedef unsigned char boolean;
+#ifndef DOSEMU_BOOL_IS_CHAR
+typedef unsigned char boolean;
+#define DOSEMU_BOOL_IS_CHAR
+#endif
 
 /* would like to have this in memory.h (but emu.h is included before memory.h !) */
 #define HARDWARE_RAM_START 0xc8000
 #define HARDWARE_RAM_STOP  0xf0000
 
+typedef struct vesamode_type_struct {
+  struct vesamode_type_struct *next;
+  unsigned width, height, color_bits;
+} vesamode_type;
+
 
      typedef struct config_info {
        int hdiskboot;
 
+#ifdef X86_EMULATOR
+       boolean cpuemu;
+       int emuspeed;
+#endif
        /* for video */
        boolean console;
        boolean console_video;
@@ -368,6 +262,8 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        boolean X;
        u_short cardtype;
        u_short chipset;
+       boolean pci;
+       boolean pci_video;
        u_short gfxmemsize;		/* for SVGA card, in K */
        /* u_short term_method; */	/* Terminal method: ANSI or NCURSES */
        u_short term_color;		/* Terminal color support on or off */
@@ -385,6 +281,18 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        int     X_blinkrate;
        int     X_sharecmap;
        int     X_mitshm;                /* use MIT SHM extension */
+       int     X_fixed_aspect;          /* keep initial aspect ratio while resizing windows */
+       int     X_aspect_43;             /* set aspect ratio to 4:3 */
+       int     X_lin_filt;              /* interpolate linear */
+       int     X_bilin_filt;            /* dto, bilinear */
+       int     X_winsize_x;             /* initial window width */
+       int     X_mode13fact;            /* initial size factor for mode 0x13 */
+       int     X_winsize_y;             /* initial window height */
+       unsigned X_gamma;		/* gamma correction value */
+       u_short vgaemu_memsize;		/* for VGA emulation */
+       vesamode_type *vesamode_list;	/* chained list of VESA modes */
+       int     X_lfb;			/* support VESA LFB modes */
+       int     X_pm_interface;		/* support protected mode interface */
        boolean fullrestore;
        boolean force_vt_switch;         /* in case of console_video force switch to emu VT at start */
        int     dualmon;
@@ -395,15 +303,16 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        boolean kbd_tty;
        boolean X_keycode;	/* use keycode field of event structure */
        boolean exitearly;
-       boolean mathco;
+       int     realcpu;
+       boolean mathco, smp;
        boolean ipxsup;
        boolean keybint;
        boolean dosbanner;
        boolean allowvideoportaccess;
+       boolean rdtsc;
        boolean timers;
        boolean mouse_flag;
        boolean mapped_bios;	/* video BIOS */
-       boolean mapped_sbios;	/* system BIOS */
        char *vbios_file;	/* loaded VBIOS file */
        boolean vbios_copy;
        int vbios_seg;           /* VGA-BIOS-segment for mapping */
@@ -425,6 +334,9 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        int pktflags;		/* global flags for packet driver */
 
        unsigned int update, freq;	/* temp timer magic */
+       unsigned int wantdelta, realdelta;
+       unsigned long cpu_spd;		/* (1/speed)<<32 */
+       unsigned long cpu_tick_spd;	/* (1.19318/speed)<<32 */
 
        unsigned int hogthreshold;
 
@@ -560,7 +472,7 @@ EXTERN void sigio(int, int, struct sigcontext *);
 
 /* signals for Linux's process control of consoles */
 #define SIG_RELEASE     SIGWINCH
-#define SIG_ACQUIRE     SIGUSR1
+#define SIG_ACQUIRE     SIGUSR2
 
  /* DANG_BEGIN_REMARK
   * We assume system call restarting... under linux 0.99pl8 and earlier,
@@ -693,4 +605,6 @@ EXTERN void read_next_scancode_from_queue (void);
 EXTERN unsigned short detach (void);
 EXTERN void HMA_init(void);
 
+extern char path_cdrom[];
+
 #endif /* EMU_H */
