@@ -12,6 +12,7 @@
 #include "keyb.h"
 #endif
 #include "port.h"
+#include "timers.h"
 
 #include "lpt.h"
 #include "pic.h"
@@ -28,33 +29,14 @@ extern void iodev_term(void);
  * Programmable Interrupt Timer (PIT) chip                         *
  *******************************************************************/
 
-#define PIT_TICK_RATE   1193180      /* underlying clock rate in HZ */
-#ifndef MONOTON_MICRO_TIMING
-#define PIT_TICK_1000   1193         /* PIT tick rate / 1000        */
-#else /* MONOTON_MICRO_TIMING */
-#ifdef i386
-#   define PIT_MS2TICKS(n) ({ \
-			       int res; \
-			       __asm__ ("imull %%edx\n\tidivl %%ecx" \
-					: "=a" (res) \
-					: "a" (n), "d" (59659), "c" (50000)); \
-			       res; \
-			   })
-#else
-#   define PIT_MS2TICKS(n) ((int)(((long long)(n)*59659)/50000))
-#endif
-#endif /* MONOTON_MICRO_TIMING */
-
-#define PIT_TIMERS      4            /* 4 timers (w/opt. at 0x44)   */
-
 typedef struct {
   Bit16u         read_state;
   Bit16u         write_state;
-  Bit16u         mode;
+  Bit8u          mode, outpin;
   Bit32u         read_latch;
   Bit16u         write_latch;
-  Bit32u         cntr;
-  struct timeval time;
+  Bit32s         cntr;
+  hitimer_u	 time;
 } pit_latch_struct;
 
 EXTERN pit_latch_struct pit[PIT_TIMERS];
