@@ -12,6 +12,14 @@
 
 #ifdef REQUIRES_VM86PLUS
 
+#ifdef X86_EMULATOR
+int e_vm86(void);
+
+#define E_VM86(x) ( \
+    (x)->vm86plus.force_return_for_pic = (pic_irr & ~(pic_isr | pice_imr)) != 0, \
+    e_vm86() )
+#endif
+
 static inline int vm86_plus(int function, int param)
 {
 	int __res;
@@ -22,10 +30,17 @@ static inline int vm86_plus(int function, int param)
 
   #undef vm86
   #define vm86(x) vm86_plus(VM86_ENTER, (int) /* struct vm86_struct* */(x))
-  #define _DO_VM86_(x) ( \
+  #define _DO_VM86__(x) ( \
     (x)->vm86plus.force_return_for_pic = (pic_irr & ~(pic_isr | pice_imr)) != 0, \
     vm86((struct vm86_struct *)(x)) )
-  #ifdef USE_MHPDBG
+ #ifdef X86_EMULATOR
+  #define _DO_VM86_(x) ( \
+    config.cpuemu? E_VM86(x) : _DO_VM86__(x) )
+  #define TRUE_VM86(x)	_DO_VM86__(x)
+ #else
+  #define _DO_VM86_(x)	_DO_VM86__(x)
+ #endif
+   #ifdef USE_MHPDBG
     #if 1
       #define DO_VM86(x) _DO_VM86_(x)
     #else
@@ -36,6 +51,9 @@ static inline int vm86_plus(int function, int param)
     #define DO_VM86(x) _DO_VM86_(x)
   #endif
 #else
+#ifdef X86_EMULATOR
+#error You need vm86plus with cpuemu
+#endif
 static inline int vm86_old(struct vm86_struct* v86)
 {
 	int __res;
